/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADCSTATUSTABLEMODEL_H
#define ADCSTATUSTABLEMODEL_H

#include "base/DeviceIndex.h"
#include "mvc/DeviceIndexTableModel.h"

class AdcStatusTable;
class DeviceIndexEnabledMap;
class AdcStatus;


class AdcStatusTableModel : public DeviceIndexTableModel
{
    Q_OBJECT

public:
    AdcStatusTableModel(QObject *parent = nullptr);
    ~AdcStatusTableModel() override;

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    virtual QVariant data(const QModelIndex &index, int role) const override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;
    DeviceIndex deviceIndexOf(int row) const override;
    int rowOf(const DeviceIndex &sourceIndex) const override;
    QMap<int, bool> defaultColumnState() const override;

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &map);
    void updateAdcStatus(const AdcStatus &status);

private:
    QVariant data_DisplayRole(const DeviceIndex &index, int column) const;

    AdcStatusTable * const devices;
};

#endif // ADCSTATUSTABLEMODEL_H
