#include "LemoMuxDialog.h"
#include "ui_LemoMuxDialog.h"

#include "base/DeviceIndexEnabled.h"
#include "base/gui_utils.h"

#include <QComboBox>

LemoMuxDialog::LemoMuxDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::LemoMuxDialog)
{
    ui->setupUi(this);

    updateComboBoxOutList();
    updateComboBoxInList();
    restoreGui();
}

LemoMuxDialog::~LemoMuxDialog()
{
    delete ui;
}

void LemoMuxDialog::setDevices(const DeviceIndexEnabledMap &map)
{
//    auto set = map.getEnabled();
//    if(devSet == set)
//        return;
//    devSet = set;
}

void LemoMuxDialog::setAdcLemoMuxBp(const DeviceIndex &index, const AdcLemoMuxBuildParams &bp)
{
    if(devBp.contains(index) && devBp[index] == bp)
        return;
    devBp[index] = bp;
    updateComboBoxOutList();
    restoreGui();
}

void LemoMuxDialog::restoreGui()
{
    RecursiveWidgetSignalBlocker blocker(this);
    ui->comboBoxOut->setCurrentIndex(config.out);
    ui->comboBoxIn->setCurrentIndex(config.in);
}

void LemoMuxDialog::updateComboBoxOutList()
{
    auto cb = ui->comboBoxOut;
    RecursiveWidgetSignalBlocker blocker(cb);

    AdcLemoMuxBuildParams bp;
    if(!devBp.isEmpty())
        bp = devBp.first();

    while(cb->count() > bp.size)
        cb->removeItem(cb->count()-1);

    for(auto i=0; i<bp.size; ++i) {
        auto name = bp.names.value(i);
        if(name.isEmpty())
            break;
        if(i >= cb->count())
            cb->addItem(name);
        else
            cb->setItemText(i, name);
    }
    restoreGui();
}

void LemoMuxDialog::updateComboBoxInList()
{
    auto cb = ui->comboBoxIn;
    RecursiveWidgetSignalBlocker blocker(cb);
    cb->addItems({"Trigger", "Ext. XOff"});
    restoreGui();
}

void LemoMuxDialog::on_comboBoxOut_currentIndexChanged(int index)
{
    if(config.out == index)
        return;
    config.out = index;
    emit adcLemoMuxConfigChanged(config);
}

void LemoMuxDialog::on_comboBoxIn_currentIndexChanged(int index)
{
    if(config.in == index)
        return;
    config.in = index;
    emit adcLemoMuxConfigChanged(config);
}

void LemoMuxDialog::setAdcLemoMuxConfig(const AdcLemoMuxCtrlConfig &cfg)
{
    if(config == cfg)
        return;
    config = cfg;
    restoreGui();
}
