#ifndef ADC64SIGNALROUTER_H
#define ADC64SIGNALROUTER_H

#include "lib-common/ReadoutAppSignalRouter.h"

class AdcAppConfig;
class AdcCalibrationDialog;
class ReadoutAppConfigDb;
class ReadoutAppConfigDispatcher;
class Adc64Core;
class Adc64MainWindow;
class AdcDeviceProxy;
class DominoDevice;
class DominoDeviceManager;
class MonitorEventCombiner;

class Adc64SignalRouter : public ReadoutAppSignalRouter
{
    Adc64SignalRouter();
    Q_DISABLE_COPY_MOVE(Adc64SignalRouter)
public:
    // Singleton
    static Adc64SignalRouter & instance()
    {
        // thread-safe since C++11
        static Adc64SignalRouter obj;
        return obj;
    }

//    void addClientManager(int id, ClientManager *p)
//    { clients[id] = p; }

    void setCore(Adc64Core *p);
    void setMainWindow(Adc64MainWindow *p);

    void setDominoDeviceManager(DominoDeviceManager *p)
    { deviceMgr = p; }

    void setMonitorEventCombiner(MonitorEventCombiner *p)
    { monEvCombiner = p; }

    void setConfigDb(ReadoutAppConfigDb * p)
    { configDb = p; }

    void setReadoutAppConfigDispatcher(ReadoutAppConfigDispatcher * p);

    void setCalibrationDialog(AdcCalibrationDialog * p)
    { calibDialog = p; }

    void connectAll() const override;
    void connect_DeviceGlobal(const AdcDeviceProxy *devGlob, DominoDevice *dev) const;

    virtual void connect_FSM() const override;
    virtual void connect_RemoteControlServer() const override;
    virtual void connect_MainWindow() const override;

private:
    void connect_ConfigurationDispatcher() const override;
    void connect_DominoDeviceManager() const;
    void connect_deviceDispatcher() const override;
    void connect_lldp() const;
    void connect_CalibrationDialog() const;

    Adc64Core *core = nullptr;
    Adc64MainWindow *mainWindow = nullptr;
    ReadoutAppConfigDb * configDb = nullptr;
    DominoDeviceManager * deviceMgr = nullptr;
    MonitorEventCombiner * monEvCombiner = nullptr;
    AdcCalibrationDialog * calibDialog = nullptr;
    ReadoutAppConfigDispatcher * readoutConfDisp = nullptr;
};
#endif // ADC64SIGNALROUTER_H
