#include "RcCountersModel.h"
#include "lib-common/BaseDeviceAppConfig.h"

//#include <QDebug>
#include <QPoint>

namespace {
QString getRateStr(double rate) {
    QString mask;
    double downScale;
    if(rate<1e3) {
        mask = "%1 Hz";
        downScale = 1;
    } else if(rate<1e6) {
        mask = "%1 kHz";
        downScale = 1e3;
    } else {
        mask = "%1 MHz";
        downScale = 1e6;
    }
    return mask.arg(rate/downScale, 0, 'f', 2);
}

QPoint getDimension(const BaseDevCounters &status)
{
    QPoint p{status.cnt.size(), 0};
    for(auto tIt=status.cnt.cbegin(); tIt!= status.cnt.cend(); ++tIt)
        p.setY(std::max(p.y(), tIt.value().size()));

    return p;
}
}

RcCountersModel::RcCountersModel(DevCountersTypes t, QObject *parent)
    : QAbstractTableModel(parent)
    , type(t)
{
}

QVariant RcCountersModel::data(const QModelIndex &index, int role) const
{
    if(!index.isValid())
        return QVariant();
    switch (role) {
    case Qt::DisplayRole:
        return displayData(index);
    }
    return QVariant();
}

QVariant RcCountersModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if(role != Qt::DisplayRole)
        return QVariant();
    if(orientation == Qt::Vertical) {
        auto name = devChNames.value(section);
        if(!name.isEmpty())
            return name;
        return QString("ch #%1").arg(section+1);
    } else {
        return getDevCountersType(type, section);
    }
}

int RcCountersModel::rowCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return getDimension(dev_status).y();
}

int RcCountersModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return getDimension(dev_status).x();
}

void RcCountersModel::deviceStatusUpdated(const BaseDevCountersMap &st)
{
    if(!st.contains(type))
        return;
    auto status = st[type];

    if(dev_status == status)
        return;
    //    QPoint old, cur;
    bool performReset = getDimension(dev_status) != getDimension(status);

    dev_status = status;
    if(performReset) {
        beginResetModel();
        endResetModel();
//        void beginInsertRows(const QModelIndex &parent, int first, int last);
//        void endInsertRows();
//        void beginRemoveRows(const QModelIndex &parent, int first, int last);
//        void endRemoveRows();
    } else {
        sendDataChanged();
    }
}

void RcCountersModel::setConfig(const ConfigKey &, const BaseDeviceAppConfig &newConfig)
{
    QStringList newNames;
    if(!newConfig.knownConfigs.isEmpty()) {
        auto newCfg = newConfig.knownConfigs.first();
        for (const auto &chCfg : newCfg.trcCore.phyTrig.chCfg)
            newNames << chCfg.name;
    }

    if(devChNames == newNames)
        return;

    devChNames = newNames;
    emit headerDataChanged(Qt::Vertical, 0, rowCount());
}

void RcCountersModel::sendDataChanged(QVector<int> roles)
{
    auto top = index(0,0);
    auto lastDev = index(rowCount(top)-1, columnCount(top)-1, top);
    auto lastCh = index(rowCount(lastDev)-1, columnCount(lastDev)-1, lastDev);
    emit dataChanged(top, lastCh, roles);
}

QVariant RcCountersModel::displayData(const QModelIndex &index) const
{
    int r=index.row(), c=index.column();
    auto col = dev_status.cnt.value(c).value(r);
//    auto col = dev_status.cnt.value(index.row()).value(index.column());
    return col;
}
