#include "DfpFsmMain.h"

#include <QDebug>
#include <QLoggingCategory>

#include "DfpFsmStates.h"

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.DevFlashProgFsm")
}

DfpFsmMain::DfpFsmMain(QObject *p)
    : DeviceMgrFsm(p)
{
    setObjectName(__FUNCTION__);
    recreateInput(new DevFlashProgFsmInput());
}

void DfpFsmMain::setDevState(const DeviceIndex &index, FsmState devSt)
{
    auto &keys = getInput()->deviceStates;
    auto it = keys.find(index);
    if(it == keys.end())
        keys[index] = devSt;
    else if(*it != devSt)
        *it = devSt;
    else
        return;

    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << index << devSt;
    evaluate();
}

bool DfpFsmMain::evaluate()
{
    if(BaseFsm::evaluate())
        return true;

    switch (getState()) {
    case DfpFsmStateError:
    {
        auto st = getPrefSt();
        if(st != getState()) {
            setState(st);
            return true;
        }
        break;
    }
    case DfpFsmStateMine:
    case DfpFsmStateAlien:
    {
        auto st = getPrefSt();
        if(st != getState()) {
            setState(st);
            return true;
        }
        if(hasRequestState()) {
            acceptRequestedState();
            return true;
        }
        break;
    }
    case DfpFsmStateErasing:
    case DfpFsmStateReading:
    case DfpFsmStateWriting:
        if(isAllInState(DfpFsmStateError, DfpFsmStateDone)) {
            setState(DfpFsmStateDone);
            return true;
        }
        break;
    case DfpFsmStateDone:
        if(isAllInState(DfpFsmStateMine, DfpFsmStateAlien)) {
            setState(getPrefSt());
            return true;
        }
        break;
    default:;
    }

    return false;
}

void DfpFsmMain::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

qint64 DfpFsmMain::stateTimeoutMs(const FsmState &state) const
{
    switch (state) {
    case DfpFsmStateErasing: return 5*60*1000;
    case DfpFsmStateWriting: return 10*60*1000;
    case DfpFsmStateReading: return 3*60*1000;
    case DfpFsmStateDone:    return 1000;
    default:
        return BaseFsm::stateTimeoutMs(state);
    }
}

bool DfpFsmMain::isAllInState(const FsmState &st, const std::optional<FsmState> &st2) const
{
    const auto &i = getInput();
    for(auto index : i->devSet) {
        auto it = i->deviceStates.constFind(index);
        if(it==i->deviceStates.cend())
            return false;
        if(*it == st || (st2.has_value() && *it == st2))
            continue;
//        if(*it != st && (!st2.has_value() || *it != st2))
        return false;
    }
    return true;
}

bool DfpFsmMain::isAnyInState(const FsmState &st) const
{
    const auto &i = getInput();
    for(auto index : i->devSet) {
        auto it = i->deviceStates.constFind(index);
        if(it!=i->deviceStates.cend() && *it == st)
            return true;
    }
    return false;
}

FsmState DfpFsmMain::getPrefSt() const
{
    if(!isAllInState(DfpFsmStateMine, DfpFsmStateAlien))
        return DfpFsmStateError;
    return isAnyInState(DfpFsmStateAlien) ? DfpFsmStateAlien : DfpFsmStateMine;
}

QDebug operator<<(QDebug dbg, const DevFlashProgFsmInput &in)
{
    dbg << DeviceMgrFsmInput(in);

    dbg << "deviceStates:" << in.deviceStates << "; ";

    return dbg;
}
