/*
**    Copyright 2014-2020 Ivan Filippov
**    Copyright 2014-2020 Timur Gandzhelashvili
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <QDebug>
#include <QString>

#include "MongoViewerMainWindow.h"
#include "base/app/DaqGuiApp.h"
#include "base/app/RunGuard.h"
#include "metric-sender/MetricSender.h"
#include "base/Globals.h"

#include <QJsonObject>
#include <QJsonValue>
#include <QSettings>

int main(int argc, char *argv[])
{
    Globals::setDaqModuleType(DaqModule::Type::MongoViewer);
    DaqGuiApp daqApp(argc, argv);
    daqApp.init();

    int ret = 1;
    try {
        RunGuard::runSingleOrExit();
        MetricSender::instance().install();
        MongoViewerMainWindow w;
        w.show();
        ret = daqApp.exec();
    } catch (std::exception &e) {
        std::cerr << "Exception: " << e.what () << std::endl;
    }
    return ret;
}

