#include "EvbGenApp.h"

#include <QCommandLineParser>

EvbGenApp::EvbGenApp(int &_argc, char *_argv[])
    : DaqGuiApp(_argc, _argv)
    , spillCnt_option(QStringList() << "spillCnt", "number of spill for work")
    , spillDuration_option(QStringList() << "spillDuration", "spill duration, in ms")
    , interSpillDuration_option(QStringList() << "interSpillDuration", "interSpill duration, in ms")
    , evRate_option(QStringList() << "evRate", "event rate, in Hz")
    , evLen_option(QStringList() << "evLen", "event size, in bytes")
    , numberOfClients_option(QStringList() << "numberOfClients", "number of clients")
{
}

int EvbGenApp::exec()
{
    return DaqGuiApp::exec();
}

void EvbGenApp::setupOptions()
{
    DaqGuiApp::setupOptions();
    options << spillCnt_option;
    options << spillDuration_option;
    options << interSpillDuration_option;
    options << evRate_option;
    options << evLen_option;
}

void EvbGenApp::processParseResults(QCommandLineParser &parser)
{
    DaqGuiApp::processParseResults(parser);

    bool ok;

    auto setConfigField = [&, this](int GenServerConfig::* field, const QCommandLineOption &opt) {
        if(parser.isSet(opt)) {
            so.cfgBase.*field = parser.value(opt).toUInt(&ok);
            if(!ok)
                parser.showHelp();
        }
    };

    setConfigField(&GenServerConfig::spill_count, spillCnt_option);
    setConfigField(&GenServerConfig::spillMs, spillDuration_option);
    setConfigField(&GenServerConfig::interspillMs, interSpillDuration_option);
    setConfigField(&GenServerConfig::event_length, evLen_option);
    if(parser.isSet(evLen_option))
        so.cfgBase.eventLengthMax = so.cfgBase.event_length;

    if(parser.isSet(evRate_option)) {
        so.cfgBase.evRate = parser.value(evRate_option).toDouble(&ok);
        if(!ok)
            parser.showHelp();
    }

    if(parser.isSet(numberOfClients_option)) {
        so.numberOfClients = parser.value(numberOfClients_option).toUInt(&ok);
        if(!ok)
            parser.showHelp();
    }
}
