#ifndef ANALIZER_H
#define ANALIZER_H

#include <QObject>
#include <QVector>

#include "base/ClientIndex.h"
#include "lib-common/MpdRawTypes.h"

#include "AnalyzerFsm.h"
#include "AnalyzerStat.h"
#include "EventMetaInfo.h"

template<class T> class RingBuffer;
class QSemaphore;
class QElapsedTimer;
class QTimer;

class Analizer : public AnalyzerFsm
{
    Q_OBJECT
public:
    Analizer(ClientIndex _index, RingBuffer<char> *dataRB, RingBuffer<EventMetaInfo> *evRB);
    virtual ~Analizer();
    ClientIndex getClientIndex() const { return index; }
    RingBuffer<EventMetaInfo> *getEventRingBuff() const { return eventRB; }

signals:
    void statUpdated(const AnalyzerStat &s) const;
    void evSizePeriodUpdated(const ClientIndex &i, const EventSizePeriod &s) const;

private slots:
    void onFsmStateChanged(const FsmState &cur, const FsmState &prev);
    void run();
    void sentStat() const;

protected:
    void doReqStateActions(const FsmState &st) final;

private:
    void reset();
    void startAnalize();
    void stopAnalize();
    bool cleanUpFreedSpace(int timeoutMs=-1);
    inline qint64 timeLeftForRun() const { return qMax(100-runElapsedTimer->elapsed(),(qint64)0); }
    bool checkOrWaitFullRec(int fullSize) const;

    int analizedSize;
    MpdHdr mpdHdr;
    MpdEventHdr mpdEvHdr;
    RingBuffer<char> *byteRB;
    RingBuffer<EventMetaInfo> *eventRB;
    EventMetaInfo *lastMetaInfo; // remember to trash corrupted data
    bool enabled;
    ClientIndex index;
    QElapsedTimer * const runElapsedTimer;
    QTimer * const runTimer;
    QTimer * const statTimer;
    AnalyzerStat stat;
};

#endif // ANALIZER_H
