#include "EvBConfig.h"

#include <QDataStream>
#include <QDebug>

#include "base/ClientIndexEnabled.h"
#include "base/Globals.h"

#include "params.h"

ClientIndexEnabledMap EvBConfig::getClientIndexEnabledMap() const
{
    ClientIndexEnabledMap map;
    for(auto cfg : clients.values()){
        map[cfg.index] = cfg.enabled;
    }
    return map;
}

QString EvBConfig::getOutputStr() const
{
    switch (combinerCfg.combinerOut) {
    case COMBINER_FILE: return "file";
    case COMBINER_TCP: return "tcp";
    case COMBINER_IDLE: return "idle";
    }
    return "unknown";
}

void EvBConfig::checkAndRepair()
{
    for(auto it=clients.begin(); it!=clients.end();) {
        if(it.key().isEmpty() || (it.value().index != it.key())) {
            it = clients.erase(it);
            qCritical() << __PRETTY_FUNCTION__ << it.key();
            Q_ASSERT(false);
        } else {
            it->checkAndRepair();
            ++it;
        }
    }
    combinerCfg.checkAndRepair();
    writerCfg.fixFilePrefix();
}

QDebug operator<<(QDebug dbg, const EvbFileSplitterConfig &c)
{
    dbg << "EvbFileSplitterConfig{"
        << "evNumEn:" << c.evNumEn
        << "evNumFileSplit:" << c.evNumFileSplit
        << "fileSizeEn:" << c.fileSizeEn
        << "fileSizeSplit:" << c.fileSizeSplit
        << "timeEn:" << c.timeEn
        << "timeFileSplit:" << c.timeFileSplit
        << "}";

    return dbg;
}

QDebug operator<<(QDebug dbg, const EvBConfig &c)
{
    dbg << "combinerCfg:" << c.combinerCfg
        << "isRemoteEnabled:" << c.isRemoteEnabled
        << "advancedCounters:" << c.advancedCounters
        << "evNumCfg:" << c.evNumCfg
        << "writerCfg:" << c.writerCfg
        << "clients:" << c.clients
        << "monitorPort:" << c.monitorPort
        << "fileSplit:" << c.fileSplit
        << "allowBufferAutoResize:" << c.allowBufferAutoResize;

    return dbg;
}


QDataStream &operator<<(QDataStream &out, const EvbFileSplitterConfig &r)
{
    out
            << r.evNumEn
            << r.evNumFileSplit
            << r.fileSizeEn
            << r.fileSizeSplit
            << r.timeEn
            << r.timeFileSplit;

    return out;
}

QDataStream &operator>>(QDataStream &in, EvbFileSplitterConfig &r)
{
    in
            >> r.evNumEn
            >> r.evNumFileSplit
            >> r.fileSizeEn
            >> r.fileSizeSplit
            >> r.timeEn
            >> r.timeFileSplit;

    return in;
}

QDataStream &operator<<(QDataStream &out, const EvBConfig &r)
{
    out
            << r.combinerCfg
            << r.isRemoteEnabled
            << r.advancedCounters
            << r.evNumCfg
            << r.writerCfg
            << r.clients
            << r.monitorPort
            << r.fileSplit
            << r.allowBufferAutoResize;
    return out;
}

QDataStream &operator>>(QDataStream &in, EvBConfig &r)
{
    in
            >> r.combinerCfg
            >> r.isRemoteEnabled
            >> r.advancedCounters
            >> r.evNumCfg
            >> r.writerCfg
            >> r.clients
            >> r.monitorPort
            >> r.fileSplit
            >> r.allowBufferAutoResize;
    return in;
}


void EvbCombinerConfig::checkAndRepair()
{
   constexpr const static int MIN_OUT_SIZE = 300e6;
   if(outputBufSize==0)
        outputBufSize = DATA_RING_SIZE;
    outputBufSize = std::max(outputBufSize, quint64(MIN_OUT_SIZE));
    outputBufSize = std::min(outputBufSize, quint64(DATA_RING_SIZE_MAX));
    infOutputEn &= Globals::getDebugMode();
}
