#include "EvbClientManagerFsm.h"
#include  "base/ClientIndexEnabled.h"

#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.evb.ClientManager")
}

EvbClientManagerFsm::EvbClientManagerFsm(QObject *parent)
    : BaseEvbClientFsm(parent)
{
    setObjectName(__func__);
    recreateInput(new EvbClientManagerFsmInput());
}

void EvbClientManagerFsm::setClientList(const ClientIndexEnabledMap &newList)
{
    getInput()->clientsSet = newList.getEnabled();
    auto &clientsStates = getInput()->clientsStates;

    for(auto i : clientsStates.keys()) {
        if(!newList.contains(i))
            clientsStates.remove(i);
    }

    evaluate();
}

void EvbClientManagerFsm::setClientState(ClientIndex index, FsmState state)
{
    getInput()->clientsStates[index] = state;
    evaluate();
}

void EvbClientManagerFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

bool EvbClientManagerFsm::checkSubStates(const FsmState &state) const
{
    const auto &i = getInput();
    for(ClientIndex index : i->clientsSet){
        if(state != i->clientsStates.value(index, FsmState::Unknown))
            return false;
    }
    return true;
}

bool EvbClientManagerFsm::hasAnyError() const
{
    const auto &i = getInput();
    for(ClientIndex index : i->clientsSet) {
        if(i->clientsStates.value(index, FsmState::Unknown) == FsmState::Error)
            return true;
    }
    return false;
}

QDebug operator<<(QDebug dbg, const EvbClientManagerFsmInput &in)
{
    dbg << BaseFsmInput(in);
    ClientIndexSet missingSet;
    QMap<FsmState, ClientIndexSet> knownCl;
    for(auto ci : in.clientsSet) {
        auto it = in.clientsStates.find(ci);
        if(it == in.clientsStates.end()) {
            missingSet << ci;
        } else {
            knownCl[*it] << ci;
        }
    }

    for(auto it=knownCl.begin(); it!=knownCl.end(); ++it) {
        QStringList sl;
        for(auto v : *it)
            sl << v.toString();
        dbg << fsmStateToString(it.key()) << ":"
            << sl.join(", ") << "; ";
    }
    if(!missingSet.isEmpty()) {
        QStringList sl;
        for(auto v : missingSet)
            sl << v.toString();
        dbg << "missing clients:" << sl.join(", ") << "; ";
    }

    QStringList extraClients;
    for(auto it = in.clientsStates.begin(); it != in.clientsStates.end(); ++it) {
        if(!in.clientsSet.contains(it.key()))
            extraClients << (it.key().toString()+": "+fsmStateToString(*it));
    }
    if(!extraClients.isEmpty())
        dbg << "extra clients:" << extraClients.join(", ") << "; ";

    // dbg << "clientsSet:" << in.clientsSet << "; ";
    // dbg << "clientsStates:" << in.clientsStates << "; ";
    return dbg;
}
