﻿#ifndef EVBSOCKETSTATUS_H
#define EVBSOCKETSTATUS_H

#include <QHostAddress>
#include <optional>

class QTcpSocket;
#include "RingBuffer.h"

struct SocketSizeStatus
{
    friend bool operator==(const SocketSizeStatus &, const SocketSizeStatus &) = default;
    virtual void updateSocketOptions(std::optional<int> s);

    std::optional<qint64> bytesToWrite;
    std::optional<qint64> bytesAvailable;

    std::optional<qint64> readBufferSize;
    std::optional<int> soSendBufferSize; // SendBufferSizeSocketOption (can't be changed)
    std::optional<int> soReceiveBufferSize; // ReceiveBufferSizeSocketOption (can be changed)
};

struct SocketAddress
{
    friend bool operator==(const SocketAddress &, const SocketAddress &) = default;
    void updateAddr(int s, bool remote = true);
    static SocketAddress getLocal(int s);
    void updateAddr(struct sockaddr_in addr);
    QString toString() const;

    QHostAddress host;
    QString hostName;
    quint32 ip = 0;
    quint16 port = 0;
};

struct SpeedStatus
{
    friend bool operator==(const SpeedStatus &, const SpeedStatus &) = default;
    void updateSpeed() {
        qint64 dt = 0;
        if(statusTime.isValid())
            dt = statusTime.restart();
        else
            statusTime.start();
        speed = dt ? 1e-3*(bytes - bytesPrev)/dt : 0;
        bytesPrev = bytes;
    }
    void clear() {
        bytes = 0;
        bytesPrev = 0;
        statusTime.invalidate();
        speed = 0;
    }

    qint64 bytes = 0;
    qint64 bytesPrev = 0;
    QElapsedTimer statusTime;
    double speed = 0; // in MB/s
};

struct EvbSocketStatus : public SocketSizeStatus , public SpeedStatus
{
    friend bool operator==(const EvbSocketStatus &, const EvbSocketStatus &) = default;
    virtual void updateRingBuffet(RingBuffer<char> *rb);
    void updateLocalPort(int s);

    bool connected = false;
    SocketAddress peer;
    std::optional<quint16> localPort;

    bool isEmpty = true; // getDataSize()==0;
    int ringBufSize = 0; // All size

    double freePart = 0; // % of freeSize
    double dataPart = 0; // % of dataSize
    double trashPart= 0; // % of trashSize
};

#endif // EVBSOCKETSTATUS_H
