#ifndef OUTPUTTCPSERVER_H
#define OUTPUTTCPSERVER_H

#include <QElapsedTimer>
#include <QObject>

#include "base/DnsLookup.h"

#include "OutputTcpStatus.h"
#include "RingBuffer.h"

class QTcpServer;
class QTcpSocket;
class QTimer;

class TcpServerStats;

class OutputTcpServer : public QObject
{
    Q_OBJECT
public:
    explicit OutputTcpServer(int _index);
    ~OutputTcpServer();
    RingBuffer<char> *getRingBuffer() const { return ringBuf; }
    int getIndex() const { return index; }

public slots:
    void startListen(int i, bool listen=true);
    void deleteTcp(int i);
    void clear();
    void startWork();
    void finishWork(uint fid);
    void setPreferedPortNum(quint16 portNum);
    void setInfOutput(bool en);

private slots:
    void workLoop();
    void checkWorkEnd() const;
    void sendStatus() const;
    void sendMetrics();
    void reverseEntryUpdated(const DnsReverseEntry &ent);

signals:
    void statusUpdated(OutputTcpStatus) const;
    void workFinished(int index, uint fid) const;
    void error(int index, const QString &errStr);

private:
    void disconnectFromHost();
    void closeServer();
    int updateDataSize();
    void acceptConnection();
    void writeData();
    void checkDisconnect();


    const int index;
    quint16 preferedPortNum = 0;
    quint16 tcpPort = 0;
    TcpServerStats * const stats;
    DnsLookup * const dnsLookup;
    QElapsedTimer writeLatencyElapsedTimer;
    QTimer * const workLoopTimer;
    QTimer * const statusTimer;
    RingBuffer<char> * const ringBuf;
    uint finishId = 0; // 0 - normal work, !0 - finishing state
    mutable OutputTcpStatus status = {index};
    bool runEnabled = false;

    std::optional<int> serverFd, socketFd;
    int dataSizeCache=0;
    bool infOutput = false;
    QDeadlineTimer connectionCooldown;
};

#endif // OUTPUTTCPSERVER_H
