#include "MpdTrcConfigDispatcher.h"

#include "lib-common/BaseDeviceAppConfig.h"
#include "Ut24TtlIo/Ut24TtlIoBuildParams.h"

MpdTrcConfigDispatcher::MpdTrcConfigDispatcher(QObject *parent)
    : ReadoutAppConfigDispatcher(parent)
{
}

void MpdTrcConfigDispatcher::sendCache()
{
    ReadoutAppConfigDispatcher::sendCache();
    BaseDeviceAppConfig &cache = getAppConfigCache();
    emit mpdTrcConfigChanged(cache.defaultSetup.mpdTrc);

    for (const auto &cfg : cache.knownConfigs)
        emit ut24TtlIoConfigChanged(cfg.index, cfg.ut24TtlIo);
}

void MpdTrcConfigDispatcher::setMpdTrcConfig(const MpdTrcConfig &newConfig)
{
    BaseDeviceAppConfig &cache = getAppConfigCache();

    if (cache.defaultSetup.mpdTrc == newConfig)
        return;

    cache.defaultSetup.mpdTrc = newConfig;
    cache.unify_config();
    incrementConfigKey();
}

void MpdTrcConfigDispatcher::setUt24TtlIoConfig(const DeviceIndex &index, const Ut24TtlIoConfig &newConfig)
{
    BaseDeviceAppConfig &cache = getAppConfigCache();

    auto it = cache.knownConfigs.find(index);
    if(it == cache.knownConfigs.end())
        return;
    if(it->ut24TtlIo == newConfig)
        return;

    it->ut24TtlIo = newConfig;
    incrementConfigKey();
    emit ut24TtlIoConfigChanged(index, newConfig);
}

void MpdTrcConfigDispatcher::setUt24TtlIoBuildParams(const DeviceIndex &index, const Ut24TtlIoBuildParams &bp)
{
    BaseDeviceAppConfig &cache = getAppConfigCache();
    auto it = cache.knownConfigs.find(index);
    if(it == cache.knownConfigs.end())
        return;
    auto &cfg = it->ut24TtlIo.ttlCfg;
    auto curSize = cfg.size();
    if(curSize == bp.chNum)
        return;
    if(curSize > bp.chNum) {
        cfg.resize(bp.chNum);
    } else {
        while(cfg.size() < bp.chNum) {
            auto v = ttlSturtUpCfg.value(index).ttlCfg
                         .value(cfg.size(), reg_ut24_ttl_io_t{.all = 0});
            cfg.push_back(v);
        }
    }
    incrementConfigKey();
}

void MpdTrcConfigDispatcher::setUt24TtlIoStartupCfg(const DeviceIndex &index, const Ut24TtlIoConfig &cfg)
{
    ttlSturtUpCfg[index] = cfg;
}

void MpdTrcConfigDispatcher::sendConfigKeyCache()
{
    ReadoutAppConfigDispatcher::sendConfigKeyCache();
    BaseDeviceAppConfig &cache = getAppConfigCache();
    emit mpdTrcConfigChanged(cache.defaultSetup.mpdTrc);

    for (const auto &cfg : cache.knownConfigs)
        emit ut24TtlIoConfigChanged(cfg.index, cfg.ut24TtlIo);
}
