#ifndef MPDTRCCONFIGDISPATCHER_H
#define MPDTRCCONFIGDISPATCHER_H

#include <QMap>

#include "base/DeviceIndex.h"
#include "lib-common/ReadoutAppConfigDispatcher.h"
#include "Ut24TtlIo/Ut24TtlIoConfig.h"

struct MpdTrcConfig;
struct Ut24TtlIoConfig;
struct Ut24TtlIoBuildParams;

class MpdTrcConfigDispatcher : public ReadoutAppConfigDispatcher
{
    Q_OBJECT
    explicit MpdTrcConfigDispatcher(QObject *parent = nullptr);
    Q_DISABLE_COPY_MOVE(MpdTrcConfigDispatcher)
public:
    virtual void sendCache() override;

    static MpdTrcConfigDispatcher & instance()
    {
        static MpdTrcConfigDispatcher obj;
        return obj;
    }

    template<class T>
    void subscribeMpdTrcConfig(T *obj) {
        connect(this, &MpdTrcConfigDispatcher::mpdTrcConfigChanged,
                obj, &T::setMpdTrcConfig,
                Qt::UniqueConnection);

        connect(obj, &T::mpdTrcConfigChanged,
                this, &MpdTrcConfigDispatcher::setMpdTrcConfig,
                Qt::UniqueConnection);
        logConnection(obj, "MpdTrcConfig");
    }

    template<class T>
    void subscribeUt24TtlIoConfig(T *obj) {
        connect(this, &MpdTrcConfigDispatcher::ut24TtlIoConfigChanged,
                obj, &T::setUt24TtlIoConfig,
                Qt::UniqueConnection);

        connect(obj, &T::ut24TtlIoConfigChanged,
                this, &MpdTrcConfigDispatcher::setUt24TtlIoConfig,
                Qt::UniqueConnection);
        logConnection(obj, "Ut24TtlIoConfig");
    }

signals:
    void mpdTrcConfigChanged(const MpdTrcConfig &config) const;
    void ut24TtlIoConfigChanged(const DeviceIndex &index, const Ut24TtlIoConfig &config) const;
public slots:
    void setMpdTrcConfig(const MpdTrcConfig &newConfig);
    void setUt24TtlIoConfig(const DeviceIndex &index, const Ut24TtlIoConfig &newConfig);
    void setUt24TtlIoBuildParams(const DeviceIndex &index, const Ut24TtlIoBuildParams &bp);
    void setUt24TtlIoStartupCfg(const DeviceIndex &index, const Ut24TtlIoConfig &cfg);

protected:
    virtual void sendConfigKeyCache() override;

private:
    QMap<DeviceIndex, Ut24TtlIoConfig> ttlSturtUpCfg;
};

#endif // MPDTRCCONFIGDISPATCHER_H
