#ifndef MPDTRCCORE_H
#define MPDTRCCORE_H

#include <QSharedPointer>
#include <optional>

#include "mstream-mreg-dev-global/MstreamMregDevGlobal.h"
#include "base/DeviceIndex.h"
#include "base/RcRunId.h"
#include "fsm/FsmState.h"
#include "lib-common/BaseDeviceAppConfig.h"
#include "lib-common/ReadoutAppCore.h"
#include "remote-control-server/RemoteControlServerStatus.h"

class ConfigKey;
class MStreamDeviceMgrFsm;
class ReadoutAppConfigDb;
struct DeviceIndexEnabledMap;

class MpdTrcCore : public ReadoutAppCore
{
    Q_OBJECT
public:
    explicit MpdTrcCore(QObject *parent = nullptr);

signals:
    void configChanged(const BaseDeviceAppConfig &config) const;
    void preRunReset(const QVariant &startEvNum);
    void mstreamResetAll(int mstreamSeqNum) const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig);
    void performLocalStart();
    void performLocalStop();
    void remoteStartRequest(const RcRunId &newRunId, const QVariant &startEvNum);
    void remoteStopRequest(const RcRunId &newRunId, const QVariant &stopEvNum);
    void remoteStatusUpdated(const RemoteControlServerStatus &status) override;
    void onFsmStateChange(const FsmState &state, const FsmState &oldState) override;

protected:
    virtual BaseDeviceAppConfig getBaseAppConfig() const { return config; }

private:
    virtual const BaseFsm *getFsm() const override;
    void addDevice(const DeviceIndex &index) override;
    void removeDevice(const DeviceIndex &index) override;
    void sendRunDocument() const;

    QVariant runStartEvNum;
    MStreamDeviceMgrFsm  * const deviceMgrFsm;
    ReadoutAppConfigDb * const configDb;
    BaseDeviceAppConfig config;
    QMap<DeviceIndex, QSharedPointer<MstreamMregDevGlobal> > devices;
    RemoteControlServerStatus lastRcStatus;
    int mstreamSeqNum = 0;
};

#endif // MPDTRCCORE_H
