#include "MpdTrcDevice.h"

#include "MpdTrc/MpdTrcModule.h"
#include "Ut24TtlIo/Ut24TtlIoModule.h"

MpdTrcDevice::MpdTrcDevice(const DeviceIndex &index, QObject *parent)
    : ModularMregDevice(index, statusCache, parent)
    , statusCache(index)
{
    qRegisterMetaType<Ut24TtlIoBuildParams>();
    qRegisterMetaType<Ut24TtlIoConfig>();
    qRegisterMetaType<MpdTrcBuildParams>();
}

void MpdTrcDevice::preRunReset(const QVariant &startEvNum)
{
    preRunActions(startEvNum);
}

void MpdTrcDevice::initModules()
{
    ModularMregDevice::initModules();

    if (auto m = getFirstModule<MpdTrcModule>()) {
        const auto &bp = m->getBuildParam();
        if(bp)
            emit mpdTrcBpUpdated(getDeviceIndex(), *bp);
    }

    if (auto m = getFirstModule<Ut24TtlIoModule>()) {
        // 1st ut24TtlIoBpUpdated, then ut24TtlIoStartupCfgUpdated (order is inmportant for MpdTrcConfigDispatcher)
        const auto &cfg = m->getInitConfig();
        if(cfg)
            emit ut24TtlIoStartupCfgUpdated(getDeviceIndex(), *cfg);
        const auto &bp = m->getBuildParam();
        if(bp)
            emit ut24TtlIoBpUpdated(getDeviceIndex(), *bp);
    }
}

bool MpdTrcDevice::writeConfig()
{
    bool ok = true;
    if (!config.isRun()) {
        ok &= writeSoftRun(false);
    }

    ok &= ModularMregDevice::writeConfig();

    if (config.isRun()) {
        ok &= writeSoftRun(true);
    }

    return ok;
}
