#include "MpdTrcSignalRouter.h"

#include "MpdTrcConfigDispatcher.h"
#include "MpdTrcCore.h"
#include "MpdTrcDeviceGlobal.h"
#include "MpdTrcDevice.h"
#include "MpdTrcMainWindow.h"
#include "MpdTrcCh/MpdTrcChModel.h"

#include "base/DeviceId.h"
#include "device-modules/ModularMregDevice.h"
#include "fsm/DeviceMgrFsm.h"
#include "lib-common/ReadoutAppConfigDb.h"
#include "lib-common/ReadoutAppCore.h"
#include "lib-common/ReadoutAppMainWindow.h"
#include "mldiscover/DeviceDispatcher.h"
#include "Ut24TtlIoWidget/Ut24TtlIoWidget.h"
#include "RunLogic/RunLogicController.h"
#include "RunLogic/RunLogicWidget.h"
#include "TrigCtrlStat/TrigCtrlStatWidget.h"

MpdTrcSignalRouter::MpdTrcSignalRouter()
    : ReadoutAppSignalRouter()
{
    setDeviceIdFilter({DEVICE_ID_UT24VE_MPD_TRC});
    setMpdTrcConfigDispatcher(&MpdTrcConfigDispatcher::instance());
}

void MpdTrcSignalRouter::setCore(MpdTrcCore *p)
{
    core = p;
    setReadoutAppCore(p);
}

void MpdTrcSignalRouter::setMainWindow(MpdTrcMainWindow *p)
{
    trcMw = p;
    trcChModel = p->getTrcChModel();
    ttlIoWidget = p->getTtlIoWidget();
    setReadoutAppMainWindow(p);
}

void MpdTrcSignalRouter::setMpdTrcConfigDispatcher(MpdTrcConfigDispatcher *p)
{
    trcConfDisp = p;
    setConfigDispatcher(p);
}
#include "MpdTrcLogicForm.h"
void MpdTrcSignalRouter::connect_TluDeviceGlobal(const MpdTrcDeviceGlobal *devGlob)
{
    if(!devGlob)
        return;

    auto dev = devGlob->getDevice<MpdTrcDevice>();
    connect_device(devGlob, dev);

    connect(core, &MpdTrcCore::preRunReset,
            dev, &MpdTrcDevice::preRunReset);

    connect(core, &MpdTrcCore::mstreamResetAll,
            devGlob, &MstreamMregDevGlobal::resetMStream);
    // if(cruFsm) {
    //     connect(dev, &CruDevice::preRunReseted,
    //             cruFsm, &CruFsm::setDevRunId);
    // }
    ;
    if (trcChModel) {
        connect(dev, &ModularMregDevice::tlu16SfpTrigBpUpdated,
                trcChModel, &MpdTrcChModel::setTlu16SfpTrigBp);
        connect(dev, &MpdTrcDevice::mpdTrcBpUpdated,
                trcChModel, &MpdTrcChModel::setMpdTrcBp);

        connect(dev, &ModularMregDevice::tluDecDesStatusUpdated,
                trcChModel, &MpdTrcChModel::setTluDecDesStatus);
        connect(dev, &ModularMregDevice::deviceConnected,
                trcChModel, &MpdTrcChModel::onDeviceConnected);
    }

    if(ttlIoWidget) {
        connect(dev, &MpdTrcDevice::ut24TtlIoBpUpdated,
                ttlIoWidget, &Ut24TtlIoWidget::setUt24TtlIoBuildParams);
    }

    connect(dev, &MpdTrcDevice::ut24TtlIoBpUpdated,
            trcConfDisp, &MpdTrcConfigDispatcher::setUt24TtlIoBuildParams);
    connect(dev, &MpdTrcDevice::ut24TtlIoStartupCfgUpdated,
            trcConfDisp, &MpdTrcConfigDispatcher::setUt24TtlIoStartupCfg);

    if(readoutAppMainWindow){
        connect(dev, &ModularMregDevice::statusUpdated,
                readoutAppMainWindow, &ReadoutAppMainWindow::setStatus);

        // connect(dev, &TqdcDevice::statusUpdated,
        //         mainWindow, &TqdcMainWindow::deviceStatusUpdated);
    }
#ifdef QT_WIDGETS_LIB
    if(trcMw){
        if (auto w = trcMw->getTrigCtrlStatWidget()) {
            connect(dev, &ModularMregDevice::trigCtrlBpUpdated,
                    w, &TrigCtrlStatWidget::setTrigCtrlBp);
        }
        if (auto w = trcMw->getMpdTrcLogicForm()) {
            connect(dev, &MpdTrcDevice::mpdTrcBpUpdated,
                    w, &MpdTrcLogicForm::setMpdTrcBp);
        }
    }
#endif

    deviceDispatcher->subscribeHostAddressUpdates(devGlob->getDeviceIndex(),
                                                  dev);
}

void MpdTrcSignalRouter::connect_ConfigurationDispatcher() const
{
    ReadoutAppSignalRouter::connect_ConfigurationDispatcher();

    Q_ASSERT(configDispatcher);
    if (deviceDispatcher)
        deviceDispatcher->subscribeDevices(configDispatcher); // TODO: move to ReadoutAppSignalRouter

    if(core) {
        configDispatcher->subscribeConfig(core);
    }

    if(configDb) {
        configDispatcher->subscribeConfig(configDb);
        configDispatcher->subscribeCS(configDb);
    }

    if(trcChModel)
        configDispatcher->subscribeConfig(trcChModel);

    if(ttlIoWidget)
        trcConfDisp->subscribeUt24TtlIoConfig(ttlIoWidget);
}

void MpdTrcSignalRouter::connect_FSM() const
{
    ReadoutAppSignalRouter::connect_FSM();

    if (!deviceMgrFsm)
        return;
    if (remoteControlServer) {
    }

    if(trcMw) {
        connect(trcMw, &MpdTrcMainWindow::printFsmInputs,
                deviceMgrFsm, &BaseFsm::printFsmInput);

        connect(deviceMgrFsm, &BaseFsm::stateChanged,
                trcMw, &MpdTrcMainWindow::onFsmStateChange);
    }

    if(ttlIoWidget)
        connect(deviceMgrFsm, &BaseFsm::stateChanged,
                ttlIoWidget, &Ut24TtlIoWidget::onFsmStateChange);

    if(auto w = trcMw->getTrigCtrlStatWidget())
        connect(deviceMgrFsm, &BaseFsm::stateChanged,
                w, &TrigCtrlStatWidget::onFsmStateChange);
}

void MpdTrcSignalRouter::connect_MainWindow() const
{
    ReadoutAppSignalRouter::connect_MainWindow();

    if(!trcMw)
        return;

    if(trcChModel) {
        connect(trcMw, &MpdTrcMainWindow::resetCntSfp,
                trcChModel, &MpdTrcChModel::resetCounters);
        configDispatcher->subscribeConfig(trcChModel);
    }

    if(core) {
        connect(trcMw, &MpdTrcMainWindow::performLocalStart,
                core, &MpdTrcCore::performLocalStart);
        connect(trcMw, &MpdTrcMainWindow::performLocalStop,
                core, &MpdTrcCore::performLocalStop);

        if (auto w = trcMw->getRunLogicWidget()) {
            w->setController(core->runLogicController);
        }
    }

    if(auto w = trcMw->getTrigCtrlStatWidget()) {
        configDispatcher->subscribeTrigCtrlStatConfig(w);
    }

}

void MpdTrcSignalRouter::connect_deviceDispatcher() const
{
    ReadoutAppSignalRouter::connect_deviceDispatcher();

    if(!deviceDispatcher)
        return;

    if(trcMw) {
        if (auto w = trcMw->getTrigCtrlStatWidget()) {
            connect(deviceDispatcher, &DeviceDispatcher::deviceListChanged,
                    w, &TrigCtrlStatWidget::setDeviceList);
        }
    }
}
