#include "MpdTrcChWidget.h"
#include "ui_MpdTrcChWidget.h"
#include "MpdTrcChModel.h"
#include "MpdTrcChColumns.h"
#include "ItemDelegate/CommonSpinBoxDelegate.h"

#include "base/Globals.h"
namespace {
static const int SHAPER_COL = static_cast<int>(MpdTrcChColumns::COL_INPUT_SHAPER);
static const int DELAY_COL = static_cast<int>(MpdTrcChColumns::COL_INPUT_DELAY);
}

MpdTrcChWidget::MpdTrcChWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::MpdTrcChWidget)
    , model(new MpdTrcChModel(this))
    , thrSbDelegate(new CommonSpinBoxDelegate(this))
{
    ui->setupUi(this);
    ui->treeView->setModel(model);
    connect(model, &MpdTrcChModel::modelReset, this, &MpdTrcChWidget::onModelReset);

    ui->treeView->setItemDelegateForColumn(SHAPER_COL, thrSbDelegate);
    ui->treeView->setItemDelegateForColumn(DELAY_COL, thrSbDelegate);

    ui->treeView->header()->setStretchLastSection(false);
    ui->treeView->header()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

MpdTrcChWidget::~MpdTrcChWidget()
{
    delete ui;
}

void MpdTrcChWidget::updateRemoteBlocking(bool remoteConnected) const
{

}

void MpdTrcChWidget::onFsmStateChange(const FsmState &state)
{
    bool en = state==FsmState::Ready || state==FsmState::Error
        || Globals::getDebugMode();
    setEnabled(en);
}

void MpdTrcChWidget::onModelReset()
{
    addDelegate({});
    ui->treeView->expandAll();
    updateGeometry();

    for(auto c : {SHAPER_COL, DELAY_COL}) {
        ui->treeView->resizeColumnToContents(c);
    }
}

void MpdTrcChWidget::addDelegate(const QModelIndex &index)
{
    auto view = ui->treeView;
    for(auto c : {SHAPER_COL, DELAY_COL}) {
        auto sibIndex = index.siblingAtColumn(c);
        if(!sibIndex.isValid())
            continue;
        if(!view->isPersistentEditorOpen(sibIndex))
            view->openPersistentEditor(sibIndex);
    }

    int chRows = model->rowCount(index);
    for(int i=0; i<chRows; ++i) {
        auto chIndex = model->index(i, 0, index);
        addDelegate(chIndex);
    }
}
