#include "MpdTrcMainWindow.h"
#include "ui_MpdTrcMainWindow.h"

#include "MpdTrcConfigDispatcher.h"
#include "MpdTrcSignalRouter.h"
#include "MpdTrcCh/MpdTrcChModel.h"

#include "Ut24TtlIoWidget/Ut24TtlIoDialog.h"
#include "Ut24TtlIoWidget/Ut24TtlIoWidget.h"

#include "base/Globals.h"

#include <QSettings>

namespace {
const int STATUS_UPDATE_DELAY = 1000;
}

MpdTrcMainWindow::MpdTrcMainWindow(QWidget *parent)
    : ReadoutAppMainWindow(parent)
    , ui(new Ui::MpdTrcMainWindow)
    , ttlIoDialog(new Ut24TtlIoDialog(this))
{
    ui->setupUi(this);
    ui->verticalLayoutLogicAndTrigger->setAlignment(Qt::AlignTop);
    ui->pushButtonStart->setVisible(false);
    ui->pushButtonStop->setVisible(false);
    setupMenu(ui->menuFile, ui->menuHelp, ui->menuOptions, ui->menuTools);

    init_connections();

    updateControlState();

    getTtlIoWidget()->setTrigInputNameMap(getMpdTrcInputChName);

    auto signalRouter = &MpdTrcSignalRouter::instance();
    signalRouter->setMainWindow(this);
}

MpdTrcMainWindow::~MpdTrcMainWindow()
{
    delete ui;
}

DeviceStatusTableViewFancy* MpdTrcMainWindow::getDeviceStatusTableView()
{
    return ui->deviceStatusTableView;
}

MpdTrcChModel *MpdTrcMainWindow::getTrcChModel() const
{
    return ui->widgetTrcCh->getModel();
}

Ut24TtlIoWidget *MpdTrcMainWindow::getTtlIoWidget() const
{
    return ttlIoDialog->getTtlIoWidget();
}

RunLogicWidget *MpdTrcMainWindow::getRunLogicWidget() const
{
    return ui->runLogicWidget;
}

TrigCtrlStatWidget *MpdTrcMainWindow::getTrigCtrlStatWidget() const
{
    return ui->trigCtrlStatWidget;
}

MpdTrcLogicForm *MpdTrcMainWindow::getMpdTrcLogicForm() const
{
    return ui->widgetTrcLogic;
}

void MpdTrcMainWindow::init_connections()
{
    connect(ui->actionPrint_FSM_inputs, &QAction::triggered,
            this, &MpdTrcMainWindow::printFsmInputs);
    connect(ui->actionConfigure_TLU_TTL_IO, &QAction::triggered,
            ttlIoDialog, &Ut24TtlIoDialog::show);
    connect(ui->pushButtonResetCntSfp, &QPushButton::clicked,
            this, &MpdTrcMainWindow::resetCntSfp);
    MpdTrcConfigDispatcher::instance().subscribeMpdTrcConfig(ui->widgetTrcLogic);
}

void MpdTrcMainWindow::saveAppGuiSettings(QSettings &s) const
{
    BaseMainWindow::saveAppGuiSettings(s);
    s.beginGroup("MW");
    s.setValue("showStartStop", ui->actionShow_Start_Stop_button->isChecked());
    s.endGroup();
}

void MpdTrcMainWindow::loadAppGuiSettings(const QSettings &s)
{
    BaseMainWindow::loadAppGuiSettings(s);
    bool showVar = s.value("MW/showStartStop").toBool();
    ui->actionShow_Start_Stop_button->setChecked(showVar);
}

void MpdTrcMainWindow::updateControlState()
{
    ui->pushButtonStart->setEnabled(curFsmState == FsmState::Ready && !underRemoteCtrl);
    ui->pushButtonStop->setEnabled(curFsmState == FsmState::Run && !underRemoteCtrl);

    bool en = Globals::getDebugMode() || (curFsmState == FsmState::Error || curFsmState == FsmState::Ready);
    if(auto m=getTrcChModel())
        m->setEnabled(en);
}

bool MpdTrcMainWindow::isControlBlocked() const
{
    return underRemoteCtrl;
}

void MpdTrcMainWindow::setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig)
{
    Q_UNUSED(config_key)
    if(config == newConfig)
        return;

    ReadoutAppMainWindow::setConfig(config_key, newConfig);

    updateControlState();
}

// ==== Slots from UI

void MpdTrcMainWindow::on_pushButtonStart_clicked()
{
    if(isControlBlocked()) return;
    if(!config.hasEnabledDevices()) {
        qWarning() << "There is no enabled device";
        return;
    }

    emit performLocalStart();
}

void MpdTrcMainWindow::on_pushButtonStop_clicked()
{
    if(isControlBlocked()) return;
    emit performLocalStop();
}

void MpdTrcMainWindow::on_actionShow_Start_Stop_button_toggled(bool showPb)
{
    ui->pushButtonStart->setVisible(showPb);
    ui->pushButtonStop->setVisible(showPb);
}
