#ifndef MPDTRCMAINWINDOW_H
#define MPDTRCMAINWINDOW_H

#include "lib-common/ReadoutAppMainWindow.h"

namespace Ui {
class MpdTrcMainWindow;
}

class ConfigKey;
struct DeviceIndexEnabledMap;
class MpdTrcChModel;
class MpdTrcLogicForm;
class Ut24TtlIoWidget;
class Ut24TtlIoDialog;
class RunLogicWidget;
class TrigCtrlStatWidget;

class MpdTrcMainWindow : public ReadoutAppMainWindow
{
    Q_OBJECT

public:
    explicit MpdTrcMainWindow(QWidget *parent = nullptr);
    ~MpdTrcMainWindow() override;
    DeviceStatusTableViewFancy* getDeviceStatusTableView() override;
    MpdTrcChModel *getTrcChModel() const;
    Ut24TtlIoWidget *getTtlIoWidget() const;
    RunLogicWidget *getRunLogicWidget() const;
    TrigCtrlStatWidget *getTrigCtrlStatWidget() const;
    MpdTrcLogicForm *getMpdTrcLogicForm() const;

signals:
    void prepareForStop(quint32 stopEvNum);
    void mstream_reset_all(bool doStatClear);
    void performLocalStart() const;
    void performLocalStop() const;
    void printFsmInputs() const;
    void resetCntSfp() const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig) override;

protected:
    void saveAppGuiSettings(QSettings &) const override;
    void loadAppGuiSettings(const QSettings &) override;
    void updateControlState() override;

private slots:
    void on_pushButtonStart_clicked();
    void on_pushButtonStop_clicked();

    void on_actionShow_Start_Stop_button_toggled(bool showPb);

private:
    void init_connections();
    bool isControlBlocked() const;

private:
    Ui::MpdTrcMainWindow *ui;
    Ut24TtlIoDialog * const ttlIoDialog;
};

#endif // MPDTRCMAINWINDOW_H
