#ifndef MSCMAINWINDOW_H
#define MSCMAINWINDOW_H

#include <QMainWindow>
#include <QThread>
#include <QStandardItemModel>
#include <QObject>

#include "MscHistWidget.h"
#include "lib-common/ReadoutAppMainWindow.h"

class ConfigKey;
class MscHistDialog;
class MscHistModel;
class MscStatus;
class MscThresholdDialog;
class MscThresholdModel;
class QHostAddress;
class QTableWidget;
class QTableWidgetItem;
struct DeviceIndexEnabledMap;


namespace Ui {
    class MscMainWindow;
}

class MscMainWindow : public ReadoutAppMainWindow
{
    Q_OBJECT
public:
    explicit MscMainWindow(QWidget *parent = nullptr);
    ~MscMainWindow() override;

    DeviceStatusTableViewFancy *getDeviceStatusTableView() override;
    MscThresholdDialog *getThresholdDialog() { return thresholdDialog; }

signals:
    void setTcpAddress(const QHostAddress &addr, int port) const;
    void keepSpill(int spillCnt);
    void discardSpill(int spillCnt);
    void writeSpill(int spillCnt, const QString &fileName);
    void deviceListChanged(const DeviceIndexEnabledMap &);

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig) override;
    void statusUpdated(const MscStatus &status);
    void deviceConnected(DeviceIndex index, QString fwStr);
    void deviceDisconnected(DeviceIndex index);
    void histUpdated(const DeviceIndex &index, const MscDevHists &h);

private Q_SLOTS:
    void on_actionThreshold_dialog_triggered();
    void on_actionHist_triggered();
    void on_comboBoxVisibleHist_currentIndexChanged(int index);
    void on_tableWidgetDevices_currentItemChanged(QTableWidgetItem *current, QTableWidgetItem *previous);
    void on_tableWidgetDevices_itemDoubleClicked(QTableWidgetItem *item);

protected:
    void saveAppGuiSettings(QSettings &) const override;
    void loadAppGuiSettings(const QSettings &) override;

private:
    void initGui();
    void updateGui();
    void setCurDevice(const DeviceIndex &i);

    void initTableDevice();
    void addDevice(const DeviceIndex &index);
    void addHists(const DeviceIndex &index);
    void removeDevice(DeviceIndex index);
    void fill_col_state(int row, bool connected);
    DeviceIndex getDeviceIndexByRow(int row) const;
    int getRowByDeviceIndex(DeviceIndex index) const;
    void adjustTableSize(QTableWidget *table);
    void updateVisiblePlots();

    Ui::MscMainWindow *ui;
    MscHistModel * hist_model;
    MscThresholdModel * threshold_model;
    MscThresholdDialog * const thresholdDialog;
    MscHistDialog * const histDialog;
    DeviceIndex curDevice;

    QMap<int, MscHistWidget*> timeHistWidgets;
    QMap<int, MscHistWidget*> plotHistWidgets;
};

#endif // MSCMAINWINDOW_H
