#include "MscSpinBoxDelegate.h"

#include <QSpinBox>

namespace {
void setParams(QSpinBox * spin, SpinBoxParams params)
{
    spin->setSingleStep(params.step);
    spin->setDisplayIntegerBase(params.base);
    spin->setGroupSeparatorShown(params.groupSeparator);
    spin->setSuffix(params.suffix);
    spin->setMaximum(params.max);
    spin->setMinimum(params.min);
}
}

QWidget *MscSpinBoxDelegate::createEditor(QWidget *parent,
                                          const QStyleOptionViewItem &,
                                          const QModelIndex &model_index) const
{
    auto spin = new QSpinBox(parent);

    setParams(spin, getParams(model_index.column()));

    int value_int = model_index.model()->data(model_index, Qt::EditRole).toInt();
    spin->setValue(value_int);
    connect(spin, SIGNAL(valueChanged(int)), this, SLOT(dataChanged()));
    return spin;
}

void MscSpinBoxDelegate::setEditorData(QWidget *editor,
                                       const QModelIndex &model_index) const
{
    int value = model_index.model()->data(model_index, Qt::EditRole).toInt();
    QSpinBox *spinBox = static_cast<QSpinBox*>(editor);
    spinBox->blockSignals(true);
    spinBox->setValue(value);
    spinBox->blockSignals(false);
}

void MscSpinBoxDelegate::setModelData(QWidget *editor,
                                      QAbstractItemModel *model,
                                      const QModelIndex &model_index) const
{
    QSpinBox *spin_box = static_cast<QSpinBox*>(editor);
    spin_box->interpretText();
    int value = spin_box->value();
    if(model->data(model_index, Qt::EditRole).toInt() != value)
        model->setData(model_index, value, Qt::EditRole);
}
