#include "PNPBeacon.h"

#include <iostream>

#include <QTime>
#include <QTimer>

#include "app/AppVersion.h"

namespace {
const int sendPeriod = 120;
const int PACKET_SIZE_DEFAULT = 8000;
}

PNPBeacon::PNPBeacon(QStringList args)
    : QObject(NULL)
    , sendTimer(new QTimer(this))
{
    QString index;
    if(args.size() < 2 || args[1].isEmpty())
        index = QHostInfo::localHostName();
    else
        index = args[1];
    const auto &programIndex = ProgramIndex::fromString(index);
    progDescr.setClientIndex({DaqModule(DaqModule::Type::PnpBeacon), programIndex});
    progDescr.ver_date = getAppDateStr();
    progDescr.ver_hash = getAppVersion();

    int packetSize;
    bool ok=false;
    if(args.size() > 2) {
        packetSize = args[2].toInt(&ok);
    }

    if(!ok) {
        packetSize=PACKET_SIZE_DEFAULT;
    }

    if(packetSize) {
        std::cerr << "Set packet size to " << packetSize << '\n';
        const int dataSize = packetSize - 44; // data size instead of frame size;
        std::cerr << "Data size will be " << dataSize << '\n';
        std::cerr << "pck size without payload option:"
                  << progDescr.getAnnounceStr().toLatin1().size() << '\n';
        progDescr.options["payload"] = "";
        int size1 = progDescr.getAnnounceStr().toLatin1().size();
        int sizeDiff = dataSize-size1;
        std::cerr << "pck size with empty payload option:" << size1 << '\n';
        for(int i=0;i<sizeDiff;++i) {
            progDescr.options["payload"].append("a");
        }
        std::cerr << "pck size with non-empty payload option:"
                  << progDescr.getAnnounceStr().toLatin1().size() << '\n';
    }

    connect(sendTimer, &QTimer::timeout, this, &PNPBeacon::sendMessage);
    sendTimer->start(1000);
    std::cerr << progDescr.getAnnounceStr().toStdString() << '\n';
}

PNPBeacon::~PNPBeacon()
{
    emit closeProgram(progDescr);
}

void PNPBeacon::sendMessage()
{
    emit updateProgramDescription(progDescr);
    if((sendCnt++)%sendPeriod==0)
        std::cerr << '\n'
                  << QTime::currentTime().toString().toStdString();
    std::cerr << ".";
}
