#ifndef PNPDISCOVERDIALOG_H
#define PNPDISCOVERDIALOG_H

#include "pnp-server/PNPMessage.h"

#include <QDialog>

namespace Ui {
class PNPDiscoverDialog;
}

class DnsLookup;
class PNPServer;

struct seq_stat{
    quint32 current_seq;
    quint32 prev_seq;
    quint32 first_seq;
    int missed_seq = 0;
    double missed_seq_perc = 0;
};

class PNPDiscoverDialog : public QDialog
{
    Q_OBJECT

public:
    explicit PNPDiscoverDialog(QWidget *parent = nullptr);
    ~PNPDiscoverDialog();
    void setSearchTargets(const DaqModuleSet &targets);

    void show_seq_stat(bool show);
private slots:
    void programClosed(const ProgramDescription &descr);
    void programDiscovered(const ProgramDescription &descr, int packetSize);
    void on_pushButtonRefresh_clicked();
    void on_pushButtonClear_clicked();
    void on_comboBoxType_currentIndexChanged(int index);
    void on_comboBoxHost_currentIndexChanged(int index);
    void on_comboBoxGroup_currentIndexChanged(int index);
    void on_checkBox_clicked(bool checked);
    void on_pushButtonPrintCurrent_clicked();
    void on_tableWidget_currentCellChanged(int currentRow, int, int, int);
    void updateTable();
    void dnsEntryUpdated(const QPair<quint32, QString> &ent);

signals:
    void addProgToDebugPrint(const QUuid &id);

private:
    void setupTable();
    int getRowNumber(const ProgramDescription &descr);
    void updateInfo(int row);
    void updateTableItem(const ProgramDescription &descr, int packetSize, const QDateTime &currentDateTime);
    void updateTableSize();
    bool fitFilter(const ProgramDescription &pd);
    bool fitFilter(int row);
    bool fitTypeFilter(const QString &type);
    bool fitHostFilter(const quint32 &hostIp);
    bool fitGroupFilter(const QString &name);
    bool fitMyGroupFilter(const QString &name);
    void updateAvailableCnt();
    int getIndexOfNewHost(const QString &h);
    void populateTypeFilter(const ProgramDescription &pd);
    void populateGroupFilter();
    void hideColumns();

    void graphMatrixUpdate();
    void graphAdjacencyListUpdate();
    void graphAdjacencyListComponents();
    void traversing_graph_in_depth(int v, QVector <bool> *used, QVector<int> *component_temp);
    void matrix_debug_printing(int maxN, QVector <QVector<bool>> matrix_to_print);
    void set_first_cell_current();
    QString getHostPortStr(const QHostAddress &h, quint16 p) const;

    Ui::PNPDiscoverDialog *ui;
    PNPServer *const pnpServer;
    DnsLookup * const dnsLookup;
    DaqModuleSet searchTargets;
    QMap<QUuid, ProgramDescription> programs;
    QMap<QUuid, int> progPacketSize;
    QMap<QUuid, seq_stat> seq_s;
    QString typeFilter;
    QStringList groupFilter;
    QStringList myGroupFilter;
    quint32 hostFilter = 0;
    QTimer * const updateTimer;
    QSet<QUuid> pendingToUpdate;
    QMap<quint32, QString> curHostList;
    QHash<quint32, QString> resolvedHosts;

    bool doGroupLogic = true;
    bool groups_count_changed_flag;
    QMap <int, QString> graphMap;
    QVector <QVector<int>> component;
    QVector <QVector<int>> vector_of_graphMatrix;
    QVector <QVector<bool>> graphMatrix;
    QMap <QString ,int> groups;
    QMap <QString , QVector<QString>> groups_test;
    int currentRowNumber;
    int groups_count;
};



#endif // PNPDISCOVERDIALOG_H
