/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "AbstractDeviceController.h"

#include "RcConfigDispatcher.h"
#include "mldiscover/DeviceDispatcher.h"
#include "mldiscover/LldpDiscovery.h"

AbstractDeviceController::AbstractDeviceController(QObject *parent) :
    AbstractConfigurable(parent)
{

}

QSet<DeviceId> AbstractDeviceController::getDeiveIdFilter()
{
    return {};
}

void AbstractDeviceController::setDeviceDispatcher(DeviceDispatcher *p)
{
    if (deviceDispatcher) {
        disconnect(deviceDispatcher);
    }
    deviceDispatcher = p;
    if (!deviceDispatcher)
        return;
    deviceDispatcher->subscribeDevices(this);

    const auto id_filter = getDeiveIdFilter();
    auto &lldpDiscovery = LldpDiscovery::instance();
    lldpDiscovery.subscribe(deviceDispatcher, id_filter);
}
