/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "RcConfigConverter.h"

#include "base/ConfigConverterUtil.h"

template<>
bool AbstractConfigConverter<RcProgramConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RcProgramConfig>::toJsonObject(const RcProgramConfig &in)
{
    QVariantMap m;
    m["runEnable"] = QVariant::fromValue(in.runEnable);
    m["localRunNumber"] = QVariant::fromValue(in.localRunNumber);
    m["allowRemoteControl"] = QVariant::fromValue(in.allowRemoteControl);
    return fromVariantMap(m);
}

template<>
RcProgramConfig AbstractConfigConverter<RcProgramConfig>::fromJsonObject(const QJsonObject &m)
{
    RcProgramConfig out;
    out.runEnable = QVariant::fromValue(m["runEnable"]).value<bool>();
    out.localRunNumber = QVariant::fromValue(m["localRunNumber"]).value<int>();
    out.allowRemoteControl = QVariant::fromValue(m["allowRemoteControl"]).value<bool>();

    return out;
}

template<>
QJsonValue AbstractConfigConverter<RcProgramConfig>::toJsonValue(const RcProgramConfig &in)
{
    return toJsonObject(in);
}

template<>
RcProgramConfig AbstractConfigConverter<RcProgramConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RcProgramConfig>::registerSubTypes()
{
    AbstractConfigConverter<RcRunId>::registerTypes();
}

// ---

template<>
bool AbstractConfigConverter<TrigSourceConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TrigSourceConfig>::toJsonObject(const TrigSourceConfig &in)
{
    QVariantMap m;
    m["xOffMask"] = QVariant::fromValue(in.xOffMask);
    m["trigOpen"] = QVariant::fromValue(in.trigOpen);
    m["trigSrc"] = QVariant::fromValue(in.trigSrc);
    return fromVariantMap(m);
}

template<>
TrigSourceConfig AbstractConfigConverter<TrigSourceConfig>::fromJsonObject(const QJsonObject &m)
{
    TrigSourceConfig data;
    data.xOffMask = QVariant::fromValue(m["xOffMask"]).value<quint16>();
    data.trigOpen = QVariant::fromValue(m["trigOpen"]).value<bool>();
    data.trigSrc = QVariant::fromValue(m["trigSrc"]).value<quint16>();

    return data;
}

template<>
QJsonValue AbstractConfigConverter<TrigSourceConfig>::toJsonValue(const TrigSourceConfig &in)
{
    return toJsonObject(in);
}

template<>
TrigSourceConfig AbstractConfigConverter<TrigSourceConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TrigSourceConfig>::registerSubTypes()
{
}

// ---

template<>
bool AbstractConfigConverter<SpillPlotConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<SpillPlotConfig>::toJsonObject(const SpillPlotConfig &in)
{
    QVariantMap m;
    m["spillPlotStep"] = QVariant::fromValue(in.spillPlotStep);
    m["plotSrc"] = QVariant::fromValue(static_cast<int>(in.plotSrc));
    return fromVariantMap(m);
}

template<>
SpillPlotConfig AbstractConfigConverter<SpillPlotConfig>::fromJsonObject(const QJsonObject &m)
{
    SpillPlotConfig out;
    out.spillPlotStep = QVariant::fromValue(m["spillPlotStep"]).value<quint32>();
    out.plotSrc = static_cast<quint8>(QVariant::fromValue(m["plotSrc"]).value<int>());
    return out;
}

template<>
QJsonValue AbstractConfigConverter<SpillPlotConfig>::toJsonValue(const SpillPlotConfig &in)
{
    return toJsonObject(in);
}

template<>
SpillPlotConfig AbstractConfigConverter<SpillPlotConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<SpillPlotConfig>::registerSubTypes()
{
}

// ---

template<>
bool AbstractConfigConverter<RcTrigDeviceConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RcTrigDeviceConfig>::toJsonObject(const RcTrigDeviceConfig &in)
{
    QVariantMap m;
    m["deviceIndex"] = QVariant::fromValue(in.index);
    m["channelDescr"] = ConfigConverterUtil::toJsonValue(in.channelDescr);
    m["dlnpCalibrationTrigger"] = QVariant::fromValue(in.dlnpCalibrationTrigger);
    m["trig"] = QVariant::fromValue(in.trig);
    m["trigCtrlStat"] = QVariant::fromValue(in.trigCtrlStat);
    m["spill"] = QVariant::fromValue(in.spillControl);
    m["spillPlot"] = QVariant::fromValue(in.spillPlot);
    return fromVariantMap(m);
}

template<>
RcTrigDeviceConfig AbstractConfigConverter<RcTrigDeviceConfig>::fromJsonObject(const QJsonObject &m)
{
    RcTrigDeviceConfig out;
    out.index = QVariant::fromValue(m["deviceIndex"]).value<DeviceIndex>();
    out.channelDescr = ConfigConverterUtil::toMap<int, QString>(m["channelDescr"]);
    out.dlnpCalibrationTrigger = QVariant::fromValue(m["dlnpCalibrationTrigger"]).value<DlnpCalibrationTriggerConfig>();
    out.trig = QVariant::fromValue(m["trig"]).value<TrigSourceConfig>();
    out.trigCtrlStat = QVariant::fromValue(m["trigCtrlStat"]).value<TrigCtrlStatConfig>();
    out.spillControl = QVariant::fromValue(m["spill"]).value<SpillConfig>();
    out.spillPlot = QVariant::fromValue(m["spillPlot"]).value<SpillPlotConfig>();

    return out;
}

template<>
QJsonValue AbstractConfigConverter<RcTrigDeviceConfig>::toJsonValue(const RcTrigDeviceConfig &in)
{
    return toJsonObject(in);
}

template<>
RcTrigDeviceConfig AbstractConfigConverter<RcTrigDeviceConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RcTrigDeviceConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<DlnpCalibrationTriggerConfig>::registerTypes();
    AbstractConfigConverter<TrigSourceConfig>::registerTypes();
    AbstractConfigConverter<SpillConfig>::registerTypes();
    AbstractConfigConverter<SpillPlotConfig>::registerTypes();
}

// ---

template<>
bool AbstractConfigConverter<VmeTrigConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<VmeTrigConfig>::toJsonObject(const VmeTrigConfig &in)
{
    QVariantMap m;
    m["enabled"] = QVariant::fromValue(in.enabled);
    m["checkEvNum"] = QVariant::fromValue(in.checkEvNum);
    m["checkWr"] = QVariant::fromValue(in.checkWr);
    return fromVariantMap(m);
}

template<>
VmeTrigConfig AbstractConfigConverter<VmeTrigConfig>::fromJsonObject(const QJsonObject &m)
{
    VmeTrigConfig out;
    out.enabled = QVariant::fromValue(m["enabled"]).value<bool>();
    out.checkEvNum = QVariant::fromValue(m["checkEvNum"]).value<bool>();
    out.checkWr = QVariant::fromValue(m["checkWr"]).value<bool>();
    return out;
}

template<>
QJsonValue AbstractConfigConverter<VmeTrigConfig>::toJsonValue(const VmeTrigConfig &in)
{
    return toJsonObject(in);
}

template<>
VmeTrigConfig AbstractConfigConverter<VmeTrigConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<VmeTrigConfig>::registerSubTypes()
{
}

// ---

template<>
bool AbstractConfigConverter<SlowControlConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<SlowControlConfig>::toJsonObject(const SlowControlConfig &in)
{
    QVariantMap m;
    m["enabled"] = QVariant::fromValue(in.enabled);
    m["port"] = QVariant::fromValue(in.port);
    return fromVariantMap(m);
}

template<>
SlowControlConfig AbstractConfigConverter<SlowControlConfig>::fromJsonObject(const QJsonObject &m)
{
    SlowControlConfig out;
    update_value(m, "enabled", &out.enabled);
    update_value(m, "port", &out.port);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<SlowControlConfig>::toJsonValue(const SlowControlConfig &in)
{
    return toJsonObject(in);
}

template<>
SlowControlConfig AbstractConfigConverter<SlowControlConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<SlowControlConfig>::registerSubTypes()
{
}

// ---

template<>
bool AbstractConfigConverter<RcConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RcConfig>::toJsonObject(const RcConfig &in)
{
    QVariantMap m;
    m["pc"] = QVariant::fromValue(in.pc);
    m["rcTrigConfig"] = QVariant::fromValue(in.rcTrigConfig);
    m["clients"] = ConfigConverterUtil::toJsonValue(in.clients);
    m["vmeTrigs"] = ConfigConverterUtil::toJsonValue(in.vmeTrigs);
    m["scConfig"] = QVariant::fromValue(in.scConfig);
    return fromVariantMap(m);
}

template<>
RcConfig AbstractConfigConverter<RcConfig>::fromJsonObject(const QJsonObject &m)
{
    RcConfig out;
    out.pc = QVariant::fromValue(m["pc"]).value<RcProgramConfig>();
    out.rcTrigConfig = QVariant::fromValue(m["rcTrigConfig"]).value<RcTrigDeviceConfig>();
    auto clientMap = ConfigConverterUtil::toMap<ClientIndex, RcClientConfig>(m["clients"]);
    for(auto it=clientMap.cbegin(); it!=clientMap.cend(); ++it)
        out.clients.insert(it.key(), *it);
    out.vmeTrigs = ConfigConverterUtil::toMap<DeviceIndex, VmeTrigConfig>(m["vmeTrigs"]);
    update_value(m, "scConfig", &out.scConfig);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<RcConfig>::toJsonValue(const RcConfig &in)
{
    return toJsonObject(in);
}

template<>
RcConfig AbstractConfigConverter<RcConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RcConfig>::registerSubTypes()
{
    AbstractConfigConverter<RcProgramConfig>::registerTypes();
    AbstractConfigConverter<RcClientConfig>::registerTypes();
    AbstractConfigConverter<TrigSourceConfig>::registerTypes();
    AbstractConfigConverter<RcTrigDeviceConfig>::registerTypes();
    AbstractConfigConverter<ClientIndex>::registerTypes();
    AbstractConfigConverter<VmeTrigConfig>::registerTypes();
    AbstractConfigConverter<SlowControlConfig>::registerTypes();
}

