/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RCSIGNALROUTER_H
#define RCSIGNALROUTER_H

class ConfigurationManager;
class DeviceDispatcher;
class DeviceIndex;
class DiscoverDialog;
class DlnpCalibrationTriggerDialog;
class EvNumChecker;
class LldpDiscovery;
class RcMainWindow;
class RcClientManager;
class RcClientManagerWidget;
class RcConfigDb;
class RcConfigDispatcher;
class RcCoreApp;
class RemoteControlServer;
class SlowControlDialog;
class SlowControlServer;
class TrigDevice;
class TrigDeviceController;
class TrigMainForm;
class VmeTrigsController;
class RcFsm;
class StatusPublisher;

class RcSignalRouter
{
public:
    // Singleton
    static RcSignalRouter & instance()
    {
        // thread-safe since C++11
        static RcSignalRouter obj;
        return obj;
    }

    // delete copy and move constructors and assign operators
    RcSignalRouter(RcSignalRouter const &) = delete;
    RcSignalRouter(RcSignalRouter &&) = delete;
    RcSignalRouter &operator=(RcSignalRouter const &) = delete;
    RcSignalRouter &operator=(RcSignalRouter &&) = delete;

    void addRcCoreApp(RcCoreApp *p)
    { rcCoreApp = p; }

    void addClientManager(RcClientManager *p)
    { clientManager = p; }

    void addClientManagerWidget(RcClientManagerWidget *p)
    { clientManagerWidget = p; }

    void addEvNumChecker(EvNumChecker *p)
    { evNumChecker = p; }

    void addMainWindow(RcMainWindow *p)
    { mainWindow = p; }

    void addConfigurationManager(ConfigurationManager *p)
    { configurationManager = p; }

    void addRcConfigDb(RcConfigDb *p)
    { configDb = p; }

    void addRcConfigDispatcher(RcConfigDispatcher *p)
    { rcConfigDispatcher = p; }

    void addRemoteControlServer(RemoteControlServer *p)
    { remoteControlServer = p; }

    void addSlowControlServer(SlowControlServer *p)
    { slowControlServer = p; }

    void addTrigMainForm(TrigMainForm *p)
    { trigMainForm = p; }

    void addTrigDeviceController(TrigDeviceController *p)
    { trigDeviceController = p; }

    void addVmeTrigsController(VmeTrigsController *p)
    { vmeTrigsController = p; }

    void addRcFsm(RcFsm *p)
    { rcFsm = p; }

    void addSlowControlDialog(SlowControlDialog *p)
    { slowControlDialog = p; }

    void addStatusPublisher(StatusPublisher *p)
    { statusPublisher = p; }

    void setCalibrationTriggerDialog(DlnpCalibrationTriggerDialog *p)
    { dlnpCalibrationTriggerDialog = p; }

    void removeRemoteControlServer();

    void connectCore();
    void connectAll();
    void connect_TrigDevice(TrigDevice *trigDevice);

private:
    RcSignalRouter();

    void connect_RcClientManager();
    void connect_RcClientManagerWidget();
    void connect_Configuration();
    void connect_EvNumChecker();
    void connect_EvNumChecker(const TrigDevice *trigDevice);
    void connect_RcMainWindow();
    void connect_StatusPublisher();
    void connect_PnpServer();
    void connect_RemoteControlServer();
    void connect_SlowControlServer();
    void connect_TrigMainForm();
    void connect_TrigDeviceController();
    void connect_VmeTrigsController();

    RcCoreApp *rcCoreApp = nullptr;
    RcConfigDb * configDb = nullptr;
    RcClientManager *clientManager = nullptr;
    RcClientManagerWidget *clientManagerWidget = nullptr;
    RcConfigDispatcher * rcConfigDispatcher = nullptr;
    TrigDeviceController * trigDeviceController = nullptr;
    EvNumChecker *evNumChecker = nullptr;
    RcMainWindow *mainWindow = nullptr;
    ConfigurationManager *configurationManager = nullptr;
    RemoteControlServer *remoteControlServer = nullptr;
    SlowControlDialog *slowControlDialog = nullptr;
    SlowControlServer *slowControlServer = nullptr;
    TrigMainForm *trigMainForm = nullptr;
    VmeTrigsController *vmeTrigsController = nullptr;
    RcFsm *rcFsm = nullptr;
    StatusPublisher *statusPublisher = nullptr;
    DlnpCalibrationTriggerDialog *dlnpCalibrationTriggerDialog = nullptr;
};

#endif // RCSIGNALROUTER_H
