/*
**    Copyright 2014-2015 Ivan Filippov
**    Copyright 2014-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RUNCONTROLREGS_H
#define RUNCONTROLREGS_H

#include <cstdint>
#include "mregdev/MregDeviceAdcm.h"

enum {
    REG_CSR_CTRL        = mlink::MregDeviceAdcm::ADCM16_CTRL_REG,

    // Run Control Core
    REG_RC_CTRL        = 0x80,
    REG_RC_CHANNEL_CNT = 0x81, // read-only
    REG_RC_TRIG_SRC    = 0x82,
    REG_RC_TRIG_STATE  = 0x83, // read-only
    REG_RC_XOFF_MASK   = 0x84,
    REG_RC_XOFF_STATE  = 0x85, // read-only
    REG_RC_TRIG_REJECT_DELAY = 0x86,
    REG_RC_XT_STATE    = 0x87, // read-only
    REG32_RC_TRIG_PERIODICAL = 0x88,
    REG32_RC_TRIG_RANDOM    = 0x8A,
    REG32_RC_SPILLPLOT_ADDR = 0x90, // read-only
    REG32_RC_SPILLPLOT_STEP = 0x92,
    REG_RC_SPILLPLOT_SOURCE = 0x96,
    REG32_RC_TRIG_PDEAD    = 0x9A, // new since r.25367
    REG32_RC_TIH_S1    = 0x9C, // new since r.25460
    REG32_RC_TIH_S2    = 0x9E, // new since r.25460
    REG32_RC_BUILD_PARAM_0         = 0xA0, // new since r.25460
    REG32_RC_BUILD_PARAM_1         = 0xA2, // new since r.25460
    REG32_RC_BUILD_PARAM_2         = 0xA4, // new since r.25460
    REG32_RC_BUILD_PARAM_3         = 0xA6, // new since r.25460
    REG64_RC_GLOB_EV_NUM_LOAD      = 0xA8, // new since r.28813
    REG64_RC_GLOB_EV_NUM           = 0xAC, // new since r.28813

    // Run Control Counters  // read-only
    REG64_CNT_BASE   = 0x800, // see CNT_INDEX
    REG64_CNT_XOFF_HI_BASE  = 0xA00, // cnt[8]
    REG64_CNT_XOFF_LOW_BASE = 0xB00, // cnt[9]
    REG64_CNT_XOFF_EVENTS_BASE = 0xD00,
};

constexpr const quint16 REG_CSR_CTRL_BIT_SOFTRESET = 0x0001;
constexpr const quint16 REG_CSR_CTRL_BIT_LOCK_COUNTERS = 0x0002;
constexpr const int NUM_COUNTERS = 0x40;

constexpr const uint16_t REG64_CNT_BLOCK_SIZE = 16; // 16 64-bit registers in each block
constexpr const uint16_t REG64_CNT_BLOCK_COUNT = 8;
constexpr const uint16_t REG64_CNT_TOTAL_NUM = REG64_CNT_BLOCK_SIZE * REG64_CNT_BLOCK_COUNT;

typedef union {
    uint16_t all;
    struct {
        uint16_t clear: 1;
        uint16_t reserved_13_1: 13;
        uint16_t softspill: 1;
        uint16_t run: 1;
    } bit;
} reg_spill_ctrl_t;

typedef union {
    uint16_t all[4];
    struct {
        uint16_t ver: 1;
        uint16_t wr_ok: 1;
        uint16_t reserved_2_3: 2;
        uint16_t sel: 4; // 2 bits before r.25461
        uint16_t phase: 5;
        uint16_t reserved_13_14_15: 3;
    } bit;
} reg_build_param_t;

#endif // RUNCONTROLREGS_H
