#include "SlowControlDialog.h"
#include "ui_SlowControlDialog.h"

SlowControlDialog::SlowControlDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::SlowControlDialog)
{
    ui->setupUi(this);
    blockControl = true;
    ui->spinBoxPort->setValue(SlowControlConfig::SLOW_CONTROL_DEF_PORT);
    blockControl = false;
}

SlowControlDialog::~SlowControlDialog()
{
    delete ui;
}

void SlowControlDialog::setSlowControlConfig(const SlowControlConfig &newConfig)
{
    if(config.has_value()) {
        if(config.value() == newConfig)
            return;
    } else {
        ui->checkBoxEnabled->setEnabled(true);
        ui->spinBoxPort->setEnabled(true);
    }

    config = newConfig;

    const auto &cfg = config.value();
    blockControl = true;

    ui->checkBoxEnabled->setChecked(cfg.enabled);
    ui->spinBoxPort->setValue(cfg.port);

    blockControl = false;
}

void SlowControlDialog::onSlowCntStatusUpdated(quint16 tcpPort, bool listening, bool portInUse)
{
    QString txt;
    if(listening) {
        txt = QString("started on port:%1").arg(tcpPort);
        if(!portInUse)
            txt += " (free)";
    } else {
        txt = "stopped";
    }

    auto l = ui->labelStatus;
    l->setText(txt);
    l->setStyleSheet(QString("QLabel { color : %1; }")
                         .arg(listening ? (portInUse ? "green" : "red") : "black"));
}

void SlowControlDialog::on_checkBoxEnabled_stateChanged(int en)
{
    if(blockControl)
        return;

    Q_ASSERT(config.has_value());
    if(!config.has_value())
        return;

    config.value().enabled = en;
    emit slowControlConfigChanged(config.value());
}


void SlowControlDialog::on_spinBoxPort_valueChanged(int port)
{
    if(blockControl)
        return;

    Q_ASSERT(config.has_value());
    if(!config.has_value())
        return;

    config.value().port = port;
    emit slowControlConfigChanged(config.value());
}

