/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "SlowControlServer_p.h"

#include <QJsonArray>
#include <QJsonDocument>
#include <QJsonObject>
#include <QVariant>

QJsonObject SlowControlServerPrivate::getRcDevice() const
{
    QJsonObject json;
    json["state"] = rcState;
    json["descr"] = rcDescr;
    json["spillCount"] = QString::number(spillCount);
    json["evNumber"] = QString::number(evNumber);
    return json;
}

QJsonObject SlowControlServerPrivate::getSpillJson() const
{
    QJsonObject json;
    json["spillCount"] = QJsonValue::fromVariant(lastSpillPlot.spillCount);
    json["spillCurrentTimeMs"] = QJsonValue::fromVariant(lastSpillPlot.spillCurrentTimeMs);
    json["stepSec"] = lastSpillPlot.stepSec;
    json["startOffset"] = lastSpillPlot.startOffset;
    QJsonArray plotArray;
    foreach (const quint32 &v, lastSpillPlot.data)
        plotArray.push_back(QJsonValue::fromVariant(v));
    json["data"] = plotArray;
    return json;
}

QJsonObject SlowControlServerPrivate::getTrigJson() const
{
    QJsonObject json;
    json["spillCount"] = QJsonValue::fromVariant(lastTrigData.spillCount);
    json["spillCurrentTimeMs"] = QJsonValue::fromVariant(lastTrigData.spillCurrentTimeMs);
//    json["stepSec"] = lastTrigData.stepSec;
    json["overflowCount"] = QJsonValue::fromVariant(lastTrigData.overflowCount);
    // px
    QJsonArray px;
    foreach (const double &v, lastTrigData.px)
        px.push_back(QJsonValue::fromVariant(v));
    json["px"] = px;
    // py
    QJsonArray plotArray;
    foreach (const quint32 &v, lastTrigData.data)
        plotArray.push_back(QJsonValue::fromVariant(v));
    json["data"] = plotArray;
    return json;
}

QJsonObject SlowControlServerPrivate::getDaq() const
{
    QJsonObject json;
    json["state"] = daqState;
    json["status"] = daqStatus;
    json["runIndex"] = runId.runIndex;
    json["runNumber"] = runId.runNumber;
    if(!lastSpillPlot.data.isEmpty())
        json["spillPlot"] = getSpillJson();
    if(!lastTrigData.data.isEmpty())
        json["trigPlot"] = getTrigJson();
    QJsonObject rcJson = getRcDevice();

    if(!rcJson.isEmpty())
        json["RunControlDevice"] = rcJson;
    return json;
}

QString SlowControlServerPrivate::getRcBlock() const
{
    return QString("\"RunControlDevice\":{\"state\":\"%1\",\"descr\":\"%2\"}")
            .arg(rcState, rcDescr);
}

QByteArray SlowControlServerPrivate::constructResponse() const
{
    QJsonObject json;
    QJsonObject daqJson = getDaq();
    json["DAQ"] = daqJson;

    QJsonDocument jDoc(json);
    QString jsonLine = jDoc.toJson(QJsonDocument::Compact);
    jsonLine.append("\r\n");
    return jsonLine.toLatin1();
}
