﻿/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TRIGDEVICECONTROLLER_H
#define TRIGDEVICECONTROLLER_H

#include "AbstractDeviceController.h"

#include <QMap>

#include "fsm/FsmState.h"

class DeviceIndex;
class TrigDeviceControllerPrivate;
struct TrigStatus;
struct RcTrigDeviceConfig;
struct RcRunId;

class TrigDeviceController : public AbstractDeviceController
{
    Q_OBJECT
public:
    explicit TrigDeviceController(QObject *parent = nullptr);
    virtual ~TrigDeviceController() override {}
    QSet<DeviceId> getDeiveIdFilter() override;
    QVariant getStopEvNum() const;
    void setMegaMasterFlag(bool mm);

signals:
    void stateChanged(const FsmState &) const;
    void printFsmInput() const;

public slots:
    void setConfig(const RcConfig &newConfig) override;
    void setDeviceList(const DeviceIndexEnabledMap &devices) override;

    void onRunStart(const RcRunId &, const QVariant &startEvNum);
    void onRunStop(const QVariant &evNum, bool force = false);
    void onPrepareStop();
    void onCloseGlobTrig();
    void onOpenGlobTrig();

private:
    void addTrigDevice(const DeviceIndex &index);
    void addMStreamDump(const DeviceIndex &index);
    void removeTrigDevice(const DeviceIndex &index);

    Q_DECLARE_PRIVATE(TrigDeviceController)
    TrigDeviceControllerPrivate * const d_ptr;
};

#endif // TRIGDEVICECONTROLLER_H
