#include "TrigStatusConverter.h"

#include "TrigStatus.h"

#include "base/DeviceIndexConverter.h"
//#include "base/ConfigConverterUtil.h"

constexpr const char *k_devIndex = "devIndex";
// trig
constexpr const char *k_curTrigState = "curTrigState";
constexpr const char *k_curXOffState = "curXOffState";
constexpr const char *k_curXOffTimeoutState = "curXOffTimeoutState";
// spill
constexpr const char *k_spillPlotAddr = "spillPlotAddr";
constexpr const char *k_spillState = "spillState";
constexpr const char *k_spillCount = "spillCount";
constexpr const char *k_spillCurrentTimeMs = "spillCurrentTimeMs";
constexpr const char *k_spillTtlLastPeriodMs = "spillTtlLastPeriodMs";
constexpr const char *k_spillWrLastPeriodMs = "spillWrLastPeriodMs";
// WR
constexpr const char *k_wrStatus = "wrStatus";
constexpr const char *k_ncuTai = "ncuTai";
constexpr const char *k_wrUptime = "wrUptime";
constexpr const char *k_wrOtherErrorCount = "wrOtherErrorCount";
constexpr const char *k_wrCodeErrorCount = "wrCodeErrorCount";
constexpr const char *k_wrLinkLostCount = "wrLinkLostCount";
constexpr const char *k_wrSyncLostCount = "wrSyncLostCount";
constexpr const char *k_cnt = "cnt"; // <cntType, <ch, cntVal> >
constexpr const char *k_xOffHiTimeoutCnt = "xOffHiTimeoutCnt"; // <ch, cntVal>
constexpr const char *k_xOffLowTimeoutCnt = "xOffLowTimeoutCnt"; // <ch, cntVal>
constexpr const char *k_xOffEventCnt = "xOffEventCnt"; // <ch, cntVal>
// other
constexpr const char *k_fwRev = "fwRev";
constexpr const char *k_nch = "nch";
//    connectStChange";
constexpr const char *k_ident = "ident";
constexpr const char *k_temp = "temp";
constexpr const char *k_fwStr = "fwStr";
constexpr const char *k_trigSrc = "trigSrc";

constexpr const char *k_globEvNum = "globEvNum";

template<>
bool AbstractConfigConverter<TrigStatus>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TrigStatus>::toJsonObject(const TrigStatus &in)
{
    QVariantMap m;
    m[k_devIndex] = QVariant::fromValue(in.getDeviceIndex());
    // trig
    m[k_curTrigState] = QVariant::fromValue(in.coreStatus.curTrigState);
    m[k_curXOffState] = QVariant::fromValue(in.coreStatus.curXOffState);
    m[k_curXOffTimeoutState] = QVariant::fromValue(in.coreStatus.curXOffTimeoutState);
    // spill
    m[k_spillPlotAddr] = QVariant::fromValue(in.coreStatus.spillPlotAddr);
//    m[k_spillState] = QVariant::fromValue(in.spillState);
    if (in.spillStatus.has_value()) {
        const auto &spill = in.spillStatus.value();
        m[k_spillCount] = QVariant::fromValue(spill.count);
        m[k_spillCurrentTimeMs] = QVariant::fromValue(spill.curSpillTimeMs);
        m[k_spillTtlLastPeriodMs] = QVariant::fromValue(spill.ttlLastPeriodMs);
        m[k_spillWrLastPeriodMs] = QVariant::fromValue(spill.wrLastPeriodMs);
    }
    // WR
//    m[k_wrStatus] = QVariant::fromValue(in.wrStatus);
//    m[k_ncuTai] = QVariant::fromValue(in.ncuTai);
    m[k_wrUptime] = QVariant::fromValue(in.wr.Uptime);
    m[k_wrOtherErrorCount] = QVariant::fromValue(in.wr.errors.OtherErrorCount);
    m[k_wrCodeErrorCount] = QVariant::fromValue(in.wr.errors.CodeErrorCount);
    m[k_wrLinkLostCount] = QVariant::fromValue(in.wr.errors.LinkLostCount);
    m[k_wrSyncLostCount] = QVariant::fromValue(in.wr.errors.SyncLostCount);
//    m[k_cnt] = QVariant::fromValue(in.cnt); // <cntType, <ch, cntVal> >
//    m[k_xOffHiTimeoutCnt] = QVariant::fromValue(in.xOffHiTimeoutCnt); // <ch, cntVal>
//    m[k_xOffLowTimeoutCnt] = QVariant::fromValue(in.xOffLowTimeoutCnt); // <ch, cntVal>
//    m[k_xOffEventCnt] = QVariant::fromValue(in.xOffEventCnt); // <ch, cntVal>
//    m[k_l0RegCnt] = QVariant::fromValue(in.l0RegCnt); // <ch, cntVal>
    // other
//    m[k_fwRev] = QVariant::fromValue(in.fwRev);
    m[k_nch] = QVariant::fromValue(in.nch);
    //    connectStChange);
    m[k_ident] = QVariant::fromValue(in.getDeviceIdent());
    m[k_temp] = ConfigConverterUtil::toJsonValue(in.getTemps());
//    m[k_fwStr] = QVariant::fromValue(in.fwStr);
    m[k_trigSrc] = QVariant::fromValue(in.coreStatus.trigSrc);

//    m[k_xOffEventCnt_base] = QVariant::fromValue(in.xOffEventCnt_base);
    m[k_globEvNum] = QVariant::fromValue(in.coreStatus.globEvNum);

    return fromVariantMap(m);
}

template<>
TrigStatus AbstractConfigConverter<TrigStatus>::fromJsonObject(const QJsonObject &m)
{
    TrigStatus out;
//    out.runEnable = QVariant::fromValue(m["runEnable"]).value<bool>();
//    out.runId = QVariant::fromValue(m["runId"]).value<RcRunId>();
//    out.localRunNumber = QVariant::fromValue(m["localRunNumber"]).value<int>();
//    out.allowRemoteControl = QVariant::fromValue(m["allowRemoteControl"]).value<bool>();
//    out.enableStatusDump = QVariant::fromValue(m["enableStatusDump"]).value<bool>();
//    out.statusDumpDirName = QVariant::fromValue(m["statusDumpDirName"]).value<QString>();

    return out;
}

template<>
QJsonValue AbstractConfigConverter<TrigStatus>::toJsonValue(const TrigStatus &in)
{
    return toJsonObject(in);
}

template<>
TrigStatus AbstractConfigConverter<TrigStatus>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TrigStatus>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
}
