#ifndef TCPCLIENT_H
#define TCPCLIENT_H

#include <QHostAddress>
#include <QTimer>
#include <QElapsedTimer>
class QTcpSocket;
class QSemaphore;

class TcpClient : public QObject
{
    Q_OBJECT
public:
    TcpClient(QSemaphore *analyzerSem, QSemaphore *writerSem, QObject *parent=nullptr);
    ~TcpClient() override;

    void startReadOut();
    void stopReadOut();
    QString getHostAddr() { return hostAddr.toString(); }
    int getHostPort() { return hostPort; }

public slots:
    void setAddress(QHostAddress, int port);
    void startWrite(QString fileName, bool rawData);
    void stopWrite();

private slots:
    void requestNewData();
    void readData();
    void connected();
    void disconnected();
    void sendStatus();
    void displayError(QAbstractSocket::SocketError socketError);

signals:
    void dataAccuired(QByteArray, bool);
    void connectedStateChanged(bool state);
    void addLog(QString);
    void statusUpdated(bool connOk, double dataRate) const;

private:
    QSemaphore * const analyzerSem; // free space in analyzer queue
    QSemaphore * const writerSem; // free space in writer queue (raw or decoded)
    bool enabled;
    QTcpSocket * const tcpSocket;
    QHostAddress hostAddr;
    int hostPort;
    QTimer reconnectTimer;
    QTimer delayReadTimer;
    QTimer statusTimer;
    int connectingAttempts;
    quint64 lastDataRecieved;
    QElapsedTimer statusPeriod;
    quint64 dataRecieved;
    bool rawWriteEnabled;
};

#endif // TCPCLIENT_H
