#ifndef TQDCDEVICE_H
#define TQDCDEVICE_H

#include "ModularDeviceStatus.h"
#include "ModularMregDevice.h"
#include "cal/CalResult.h"

#include <optional>

class Ad5328Module;
class AdcSpiLtm9011Module;
class CalConfig;
class DeserializersControlModule;
class DeviceModuleMapper;
class HitCoreModule;
class LegacyDeviceModule;
class TdcCoreModule;
class TqdcDacCalibrator;
class TtlIoModule;
struct RcRunId;

class TqdcDevice : public ModularMregDevice
{
    Q_OBJECT
public:
    enum NetworkPortType {SFP0, SFP1, SFP2};

    explicit TqdcDevice(const DeviceIndex &index, QObject *parent = nullptr);
    virtual ~TqdcDevice() override;

    static const std::initializer_list<DeviceId> getSupportedDeviceIds();

    void readStatistics();
    void clearStatistics();
    static QString networkPortName(int portIndex);
    virtual QSharedPointer<AdcSpiBaseModule> getAdcSpiModule() const;

signals:
    void deviceInfoUpdated(DeviceIndex index, int maxSampleCnt);
    void calResultChanged(const CalResult &newCalResult);
    void ttlIoStartupCfg(const TtlIoConfig &cfg);

public slots:
    void setCalResult(const ConfigKey &new_config_key, const CalResult &newCalResult);
    bool softReset();
    void startTqdcCalibration(const ConfigKey &cal_key);
    void preRunReset(const QVariant &startEvNum, const RcRunId &rcRunId);

protected slots:
    virtual void on_deviceConnected() override;

private:
    std::optional<CalResult> performTqdcCalibration();
    void writeConfigWithPause();
    bool writeSetupImpl(bool setupCacheValid = false);
    void writeSetupForce();

    bool StartRun();
    bool StopRun();
    int getNumberOfChannels() const;
    bool setupAdc();
//    QString adcStatusStr(int status);
    bool waitEndOfRun();
    bool calibrateBaseline(MultichannelBaselineStats *result);
    std::optional<DacCalResult> calibrateDac();

protected:
    bool readStatus() override;
    bool writeConfig() override;
    void enforceConfigConstraints(ModularDeviceConfig *config) override;

    ModularDeviceStatus statusCache;
    CalResult calResult;

    QSharedPointer<LegacyDeviceModule> legacyDevice;

    TqdcDacCalibrator * const dacCalibrator;

    quint16 trig_csr = 0;
};

#endif // TQDCDEVICE_H
