/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TTVXSAPPCONFIGDISPATCHER_H
#define TTVXSAPPCONFIGDISPATCHER_H

#include "lib-common/BaseConfigDispatcher.h"
#include <boost/shared_ptr.hpp>

class RunControlAppConfig;
class DeviceIndexEnabledMap;
struct RcClientConfigMap;

class TtvxsAppConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT
    TtvxsAppConfigDispatcher(QObject *parent = nullptr);
    Q_DISABLE_COPY_MOVE(TtvxsAppConfigDispatcher);
public:
    // Singleton
    static TtvxsAppConfigDispatcher & instance()
    {
        // thread-safe since C++11
        static TtvxsAppConfigDispatcher obj;
        return obj;
    }

    template<class T>
    void subscribeConfig(T *obj) {
        connect(obj, &T::configChanged,
                this, &TtvxsAppConfigDispatcher::setConfig);
        connect(this, &TtvxsAppConfigDispatcher::configChanged,
                obj, &T::setConfig);
        logConnection(obj, "TtvxsAppConfig");
    }
signals: void configChanged(ConfigKey key, const RunControlAppConfig &config) const;
public slots: void setConfig(const RunControlAppConfig &config);

    void setConfigFeLink(const BaseDeviceAppConfig &config);


public:
    template<class T>
    void subscribeRcClientConfigMap(T *obj) {
            connect(this, &TtvxsAppConfigDispatcher::rcClientsConfigChanged,
                    obj, &T::setRcClientsConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::rcClientsConfigChanged,
                    this, &TtvxsAppConfigDispatcher::setRcClientsConfig,
                    Qt::UniqueConnection);
            logConnection(obj, "RcClientConfigMap");
            sendCache();
    }
signals: void rcClientsConfigChanged(const RcClientConfigMap &) const;
public slots: void setRcClientsConfig(const RcClientConfigMap &newConfig);

public slots:
    BaseDeviceAppConfig &getAppConfigCache() override;

protected:
    DeviceIndexEnabledMap getDeviceIndexEnabledMap() const override;
    void sendConfigKeyCache() override;

private:
    boost::shared_ptr<RunControlAppConfig> cache;
};

#endif // TTVXSAPPCONFIGDISPATCHER_H
