/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ConfigSelectorConverter.h"

#include "ConfigurationNameConverter.h"
#include "DaqModuleConverter.h"
#include "ProgramIndexConverter.h"


template<>
bool AbstractConfigConverter<ConfigSelector>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<ConfigSelector>::toJsonObject(const ConfigSelector &in)
{
    QVariantMap m;
    m[k_cs_program_type] = QVariant::fromValue(in.getType());
    m[k_cs_program_index] = QVariant::fromValue(in.getIndex());
    m[k_cs_configuration_name] = QVariant::fromValue(in.getConfigurationName());
    return fromVariantMap(m);
}

template<>
ConfigSelector AbstractConfigConverter<ConfigSelector>::fromJsonObject(const QJsonObject &json)
{
    auto daqModule = QVariant::fromValue(json[k_cs_program_type]).value<DaqModule>();
    auto program_index = QVariant::fromValue(json[k_cs_program_index]).value<ProgramIndex>();
    auto configuration_name = QVariant::fromValue(json[k_cs_configuration_name]).value<ConfigurationName>();
    return ConfigSelector(daqModule, ConfigIndexName(program_index, configuration_name));
}

template<>
QJsonValue AbstractConfigConverter<ConfigSelector>::toJsonValue(const ConfigSelector &in)
{
    return toJsonObject(in);
}

template<>
ConfigSelector AbstractConfigConverter<ConfigSelector>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<ConfigSelector>::registerSubTypes()
{
    AbstractConfigConverter<DaqModule>::registerTypes();
    AbstractConfigConverter<ConfigurationName>::registerTypes();
    AbstractConfigConverter<ProgramIndex>::registerTypes();

//    QMetaType::registerConverter<ConfigSelector, QString>(&toString);
//    QMetaType::registerConverter<QString, ConfigSelector>(&fromString);
}

// Used in mongo lib for get_config_map
QJsonValue getJsonCSFromDaqModule(const DaqModule &dm)
{
    return QJsonObject{{k_cs_program_type, QVariant::fromValue(dm).value<QJsonValue>()}};
}
