/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DEVICEID_H
#define DEVICEID_H

#include <cstdint>

class QString;

enum DeviceId : uint8_t {
    DEVICE_ID_UNKNOWN       = 0,
    DEVICE_ID_FVME2TMWR     = 0x0A, // VME 0A
    DEVICE_ID_TEST          = 0x55,
    DEVICE_ID_SOFTWARE      = 0x56,
    DEVICE_ID_TQDC16        = 0x89, // VME 09
    DEVICE_ID_VME_MSC16V    = 0x8F, // VME 0F
    DEVICE_ID_VME_TDC64V    = 0x90, // VME 10
    DEVICE_ID_VME_TDC32VL   = 0x91, // VME 11
    DEVICE_ID_VME_TDC72VHL  = 0x92, // VME 12
    DEVICE_ID_VME_TTB9V     = 0x93, // VME 13
    DEVICE_ID_8CADC         = 0xB3,
    DEVICE_ID_ADCM16        = 0xB4,
    DEVICE_ID_ADC16LTCM     = 0xB5,
    DEVICE_ID_ADCM163       = 0xB6,
    DEVICE_ID_BPM4100       = 0xB7,
    DEVICE_ID_ADC8E         = 0xB8,
    DEVICE_ID_EVADC         = 0xB9,
    DEVICE_ID_ADC64         = 0xBA,
    DEVICE_ID_NUCLORF       = 0xBB,
    DEVICE_ID_BRIC          = 0xBC,
    DEVICE_ID_MSC8E         = 0xBD,
    DEVICE_ID_ADCM164       = 0xBE,
    DEVICE_ID_ADC8BEDRS     = 0xBF,
    DEVICE_ID_HRB128E       = 0xC1,
    DEVICE_ID_HRB6ASD       = 0xC2,
    DEVICE_ID_ADC32P        = 0xC3,
    DEVICE_ID_TTB9V         = 0xC4,
    DEVICE_ID_ADC64S2       = 0xC5,
    DEVICE_ID_U40VE         = 0xC6, // VME 46
    DEVICE_ID_CTF6          = 0xC8,
    DEVICE_ID_UT24VE        = 0xC9, // VME 49
    DEVICE_ID_ADC64WR       = 0xCA,
    DEVICE_ID_U40VE_8SPI    = 0xCB,
    DEVICE_ID_U40VE_RC      = 0xCC, // VME 4C
    DEVICE_ID_U40VE_IDEAS   = 0xCD,
    DEVICE_ID_U40VE_SI      = 0xCE,
    DEVICE_ID_TTVXS         = 0xCF,
    DEVICE_ID_TDC72VXS      = 0xD0,
    DEVICE_ID_FVME          = 0xD1,
    DEVICE_ID_TDC64VHLE     = 0xD3, // VME 53
    DEVICE_ID_ADC64VE       = 0xD4, // VME 54
    DEVICE_ID_TQDC16VS_E    = 0xD6, // VME 56
    DEVICE_ID_VMM128RO      = 0xDB,
    DEVICE_ID_MSC16VE_TM    = 0xDC,
    DEVICE_ID_MSC16VE_E     = 0xD8,
    DEVICE_ID_ADC64VE_XGE   = 0xD9, // VME 59
    DEVICE_ID_ADC64ECAL     = 0xDE,
    DEVICE_ID_ADC64VE2      = 0xDF,
    DEVICE_ID_CRU16WR       = 0xE0,
    DEVICE_ID_PHOS_FEC      = 0xE1,
    DEVICE_ID_TDC64VLE      = 0xE2,
    DEVICE_ID_TRC           = 0xE3, // UT24VE_BMN_TRC
    DEVICE_ID_TLU40LVDS     = 0xE4,
    DEVICE_ID_TLU16SFP      = 0xE5,
    DEVICE_ID_UT24VE_MPD_TRC= 0xE6,
    DEVICE_ID_AnyDevice     = 0xFF
};

QString getDeviceTypeName(int deviceId);

DeviceId deviceIdFromVmeModuleId(int id);

// compatibility
constexpr const auto DEVICE_ID_VME_TRIG = DEVICE_ID_FVME2TMWR;
constexpr const auto DEVICE_ID_TQDC16VS_DIG = DEVICE_ID_TQDC16;

#endif // DEVICEID_H
