/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef STATISTICSDB_H
#define STATISTICSDB_H

#include <QMap>

class StatisticsDb
{
    struct Item {
        double last = 0;
        double diff = 0;
        double sum = 0;
        quint64 count = 0;
    };

public:
    StatisticsDb() = default;
    static StatisticsDb &instance();

    void clear() {items.clear(); }
    void clear(const QString &key) { items.remove(key); }
    void add(const QString &key, double value) {
        auto &item = items[key];
        item.sum += value;
        if (item.count)
            item.diff = value - item.last;
        item.last = value;
        item.count++;
    }
    qint64 count(const QString &key) const {
        return items[key].count;
    }
    double diff(const QString &key) const {
        return items[key].diff;
    }
    double lastValue(const QString &key) const {
        return items[key].last;
    }
    double avg(const QString &key) const {
        auto item = items[key];
        return (double)item.sum / (item.count ? item.count : 1);
    }

private:
    StatisticsDb(const StatisticsDb &) = default;
    StatisticsDb &operator=(const StatisticsDb &) = default;

    QMap<QString, Item> items;
};

#endif // STATISTICSDB_H
