/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef SDBDATA_H
#define SDBDATA_H

#include <QString>
#include <QVector>

#include "SdbDevice.h"

struct SdbInterconnect
{
    friend bool operator==(const SdbInterconnect &, const SdbInterconnect &) = default;
    bool isValid() const { return valid; }

    bool valid = false;
    QString productName() const { return component.product.name; }
    QString productDate() const { return component.product.date; }
    QString productVersion() const { return component.product.version; }

    uint16_t sdb_records;
    uint8_t sdb_version;
    uint8_t sdb_bus_type;
    SdbComponent component;
};

struct SdbSynthesis
{
    friend bool operator==(const SdbSynthesis &, const SdbSynthesis &) = default;
    bool isValid() const { return valid; }

    bool valid = false;
    QString syn_name;
    QString commit_id;
    QString tool_name;
    QString tool_version;
    QString date; // TODO: QDate
    QString user_name;
};

struct SdbData
{
    friend bool operator==(const SdbData &, const SdbData &) = default;

    SdbInterconnect ic;
    QVector<SdbDevice> devices;
    SdbSynthesis syn;
};

#endif // SDBDATA_H
