/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "DeviceIdFilter.h"

#include "BaseDeviceTableModel.h"
#include "base/DeviceId.h"

DeviceIdFilter::DeviceIdFilter(QObject* parent)
    : QSortFilterProxyModel(parent)
{

}

void DeviceIdFilter::setFilter(const DeviceIdSet &r)
{
    filter = r;
    invalidateFilter();
}

bool DeviceIdFilter::filterAcceptsRow(
        int sourceRow, const QModelIndex &sourceParent) const
{
    if (filter.isEmpty() || filter.contains(DEVICE_ID_AnyDevice))
        return true;

    auto sourceModel = this->sourceModel();
    Q_ASSERT(sourceModel);
    if (!sourceModel)
        return true;
    const QModelIndex &indexDeviceType = sourceModel->index(
                sourceRow, BaseDeviceTableModel::COL_DEVICE_ID, sourceParent);
    bool ok;
    const auto &device_id = sourceModel->data(indexDeviceType, Qt::UserRole).toUInt(&ok);
    if (!ok)
        return true;
    return filter.contains(static_cast<DeviceId>(device_id));
}
