/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DEVICETABLEENTRY_H
#define DEVICETABLEENTRY_H

#include <QVector>

#include "base/DeviceIndex.h"
#include "mldiscover/LldpDiscovery.h"

struct DeviceTableEntry
{
    enum DynamicFlag { Static, Dynamic, DynamicDefault };
    enum EnableFlag { Enable, Disable, EnableDefault };
    DeviceTableEntry() = default;
    DeviceTableEntry(const DeviceIndex &index, const LldpInfo &lldpInfo,
                     DynamicFlag dynamic, EnableFlag enabled)
        : index(index), lldpInfo(lldpInfo), dynamic(dynamic), enabled(enabled) { }
    DeviceIndex index;
    LldpInfo lldpInfo;
    DynamicFlag dynamic = DynamicDefault;
    EnableFlag enabled = EnableDefault;
};
Q_DECLARE_METATYPE(DeviceTableEntry::DynamicFlag);
Q_DECLARE_METATYPE(DeviceTableEntry::EnableFlag);

inline bool operator<(const DeviceTableEntry &lhs, const DeviceTableEntry &rhs)
{
    return lhs.index < rhs.index;
}

inline bool operator==(const DeviceTableEntry &lhs, const DeviceTableEntry &rhs)
{
    return lhs.index == rhs.index &&
            lhs.lldpInfo == rhs.lldpInfo &&
            lhs.dynamic == rhs.dynamic &&
            lhs.enabled == rhs.enabled;
}

#endif // DEVICETABLEENTRY_H
