/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "NetworkFilterProxyModel.h"

#include "BaseDeviceTableModel.h"

NetworkFilterProxyModel::NetworkFilterProxyModel(QObject* parent)
    : QSortFilterProxyModel(parent)
{
}

void NetworkFilterProxyModel::setFilter(const Ipv4PrefixList &filter)
{
    if (this->filter == filter)
        return;
    this->filter = filter;
    invalidateFilter();
}

bool NetworkFilterProxyModel::filterAcceptsRow(
        int sourceRow, const QModelIndex &sourceParent) const
{
    if (filter.isEmpty())
        return true;
    const QModelIndex &index = sourceModel()->index(
                sourceRow, BaseDeviceTableModel::COL_IP, sourceParent);
    const auto &data = sourceModel()->data(index).toString();
    if (data.isEmpty())
        return true;
    bool ok = true;
    quint32 ip = QHostAddress(data).toIPv4Address(&ok);
    if (!ok)
        return true;
    return filter.match(ip);
}
