#include "AdcErcConfigConverter.h"

#include "AdcErcConfig.h"

namespace {
// SparseParams
constexpr const char *k_en = "en";
constexpr const char *k_readCellNum = "readCellNum";
constexpr const char *k_period = "period";
constexpr const char *k_pointNumber = "pointNumber";
// AdcErcConfig
constexpr const char *k_sparseParams = "sparseParams";
}

// -------------- SparseParams
template<>
bool AbstractConfigConverter<SparseParams>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<SparseParams>::toJsonObject(const SparseParams &in)
{
    QVariantMap m;
    m[k_en] = QVariant::fromValue(in.en);
    m[k_readCellNum] = QVariant::fromValue(in.readCellNum);
    m[k_period] = QVariant::fromValue(in.period);
    m[k_pointNumber] = QVariant::fromValue(in.pointNumber);
    return fromVariantMap(m);
}

template<>
SparseParams AbstractConfigConverter<SparseParams>::fromJsonObject(const QJsonObject &in)
{
    SparseParams out;
    update_value(in, k_en, &out.en);
    update_value(in, k_readCellNum, &out.readCellNum);
    update_value(in, k_period, &out.period);
    update_value(in, k_pointNumber, &out.pointNumber);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<SparseParams>::toJsonValue(const SparseParams &in)
{
    return toJsonObject(in);
}

template<>
SparseParams AbstractConfigConverter<SparseParams>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<SparseParams>::registerSubTypes()
{}

// -------------- AdcErcConfig
template<>
bool AbstractConfigConverter<AdcErcConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<AdcErcConfig>::toJsonObject(const AdcErcConfig &in)
{
    QVariantMap m;
    m[k_sparseParams] = QVariant::fromValue(in.sparse);
    return fromVariantMap(m);
}

template<>
AdcErcConfig AbstractConfigConverter<AdcErcConfig>::fromJsonObject(const QJsonObject &in)
{
    AdcErcConfig out;
    update_value(in, k_sparseParams, &out.sparse);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<AdcErcConfig>::toJsonValue(const AdcErcConfig &in)
{
    return toJsonObject(in);
}

template<>
AdcErcConfig AbstractConfigConverter<AdcErcConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<AdcErcConfig>::registerSubTypes()
{
    AbstractConfigConverter<SparseParams>::registerTypes();
}
