/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef BASEDEVICESTATUS_H
#define BASEDEVICESTATUS_H

#include "base/DeviceIndex.h"
#include "ModularDeviceStatus.h"

struct BaseDeviceStatus: public ModularDeviceStatus
{
    class ConnectionFSM
    {
    private:
        typedef enum
        {
            Unknown,
            Unconfigured,
            Ready
        } ConnectionState;
        ConnectionState connectionState = Unknown;
    public:
        bool isReady() const { return connectionState == Ready; }
        void setConnected() {
            connectionState = Unconfigured;
        }
        void setDisconnected() {
            connectionState = Unknown;
        }
        void setUnconfigured() {
            if (connectionState == Ready)
                connectionState = Unconfigured;
        }
        void setReady() {
            if (connectionState == Unconfigured)
                connectionState = Ready;
        }
    };

    explicit BaseDeviceStatus(const DeviceIndex &devIndex = {})
        : ModularDeviceStatus(devIndex) {}

    void setDisconnected() {
        conn.setDisconnected();
    }

    ConnectionFSM conn;
};

#endif // BASEDEVICESTATUS_H
