/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ClockControlConfigConverter.h"

#include "ClockControlConfig.h"

namespace {
constexpr const char *k_pll_bypass = "pll_bypass";
constexpr const char *k_source = "source";
}

template<>
bool AbstractConfigConverter<ClockControlConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<ClockControlConfig>::toJsonObject(const ClockControlConfig &in)
{
    QVariantMap m;
    m[k_pll_bypass] = QVariant::fromValue(in.pll_bypass);
    m[k_source] = QVariant::fromValue(static_cast<quint16>(in.source));
    return fromVariantMap(m);
}

template<>
ClockControlConfig AbstractConfigConverter<ClockControlConfig>::fromJsonObject(const QJsonObject &in)
{
    ClockControlConfig out;
    quint16 source = 0;
    update_value(in, k_pll_bypass, &out.pll_bypass);
    update_value(in, k_source, &source);
    if(source)
        out.source = static_cast<ClockControlType>(source);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<ClockControlConfig>::toJsonValue(const ClockControlConfig &in)
{
    return toJsonObject(in);
}

template<>
ClockControlConfig AbstractConfigConverter<ClockControlConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<ClockControlConfig>::registerSubTypes()
{
}
