#ifndef MPDTRCCONFIG_H
#define MPDTRCCONFIG_H

#include <QMetaType>
#include <QVector>

enum class FhcalInputLogic
{
    CommonOr  = 0x00,
    CommonAnd = 0x01,
    Majority  = 0x02,
    Disabled  = 0x03,
};

enum class FhcalOutputLogic
{
    Or  = 0x00,
    And = 0x01,
};

struct MpdTrcConfig
{
    friend bool operator==(const MpdTrcConfig &, const MpdTrcConfig &) = default;

    constexpr static const bool DEF_CH_EN = true;

    FhcalInputLogic fhcalEastLogic = FhcalInputLogic::Disabled;
    quint8 fhcalEastMajThr = 0;

    FhcalInputLogic fhcalWestLogic = FhcalInputLogic::Disabled;
    quint8 fhcalWestMajThr = 0;

    FhcalOutputLogic fhcalOutputLogic = FhcalOutputLogic::Or;
    QVector<quint16> inputShapers;
    QVector<quint16> inputDelays;
    QVector<quint16> lut;
};
Q_DECLARE_METATYPE(MpdTrcConfig)

static QString getMpdTrcInputChName(const int ch)
{
    switch (ch) {
    case 0: return "FFD East";
    case 1: return "FFD West";
    case 2: return "FFD Vertex";
    case 3: return "TOF";
    case 4: return "Luminosity";
    default:;
    }
    if(ch<=9)
        return QString("FHCAL East #%1").arg(ch-5);
    if(ch<=14)
        return QString("FHCAL West #%1").arg(ch-10);
    if(ch<=16)
        return QString("TPC Laser #%1").arg(ch-15);

    return QString("Unknown ch #%1").arg(ch);
}

#endif // MPDTRCCONFIG_H
