#include "MpdTrcConfigConverter.h"

#include "MpdTrcConfig.h"

#include "base/ConfigConverterUtil.h"

namespace {
// MpdTrcConfig
constexpr const char *k_fhcalEastLogic = "fhcalEastLogic";
constexpr const char *k_fhcalEastMajThr = "fhcalEastMajThr";
constexpr const char *k_fhcalWestLogic = "fhcalWestLogic";
constexpr const char *k_fhcalWestMajThr = "fhcalWestMajThr";
constexpr const char *k_fhcalOutputLogic = "fhcalOutputLogic";
constexpr const char *k_inputShapers = "inputShapers";
constexpr const char *k_inputDelays = "inputDelays";
constexpr const char *k_lut = "LUT";
}

// -------------- MpdTrcConfig
template<>
bool AbstractConfigConverter<MpdTrcConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<MpdTrcConfig>::toJsonObject(const MpdTrcConfig &in)
{
    QVariantMap m;
    m[k_fhcalEastLogic] = QVariant::fromValue(static_cast<quint16>(in.fhcalEastLogic));
    m[k_fhcalEastMajThr] = QVariant::fromValue(in.fhcalEastMajThr);
    m[k_fhcalWestLogic] = QVariant::fromValue(static_cast<quint16>(in.fhcalWestLogic));
    m[k_fhcalWestMajThr] = QVariant::fromValue(in.fhcalWestMajThr);

    m[k_inputShapers] = ConfigConverterUtil::toJsonValue(in.inputShapers);
    m[k_inputDelays] = ConfigConverterUtil::toJsonValue(in.inputDelays);

    m[k_lut] = ConfigConverterUtil::toJsonValue(in.lut);
    return fromVariantMap(m);
}

template<>
MpdTrcConfig AbstractConfigConverter<MpdTrcConfig>::fromJsonObject(const QJsonObject &in)
{
    MpdTrcConfig out;
    quint16 fhcalEastLogic, fhcalWestLogic;
    update_value(in, k_fhcalEastLogic, &fhcalEastLogic);
    update_value(in, k_fhcalEastMajThr, &out.fhcalEastMajThr);
    update_value(in, k_fhcalWestLogic, &fhcalWestLogic);
    update_value(in, k_fhcalWestMajThr, &out.fhcalWestMajThr);
    out.fhcalEastLogic = static_cast<FhcalInputLogic>(fhcalEastLogic);
    out.fhcalWestLogic = static_cast<FhcalInputLogic>(fhcalWestLogic);

    out.inputShapers = ConfigConverterUtil::toVector<quint16>(in[k_inputShapers]);
    out.inputDelays = ConfigConverterUtil::toVector<quint16>(in[k_inputDelays]);
    out.lut = ConfigConverterUtil::toVector<quint16>(in[k_lut]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<MpdTrcConfig>::toJsonValue(const MpdTrcConfig &in)
{
    return toJsonObject(in);
}

template<>
MpdTrcConfig AbstractConfigConverter<MpdTrcConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<MpdTrcConfig>::registerSubTypes()
{
}
