#include "MpdTrcModule.h"

#include "DeviceModuleCommand.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

namespace {
enum {
    REG32_ID        = 0x00,
    // REG_CTRL        = 0x02, // [0] - reset
    // REG_LOCK        = 0x03, // [0] - 1: lock, 0: continuous update
    REG_NUM_CH_IN   = 0x04, // Number of channels, RO static
    REG_NUM_CH_PHY  = 0x05, // Number of channels, RO static

    REG_FHCAL_EAST_LOGIC = 0x08, // TOF sum threshold, R/W
    REG_FHCAL_WEST_LOGIC = 0x09, // TOF sum threshold, R/W
    REG_FHCAL_OUTPUT_LOGIC = 0x0A, // TOF sum threshold, R/W

    REG_INPUT_SHAPER_BASE = 0x40,
    REG_INPUT_DELAY_BASE = 0x80,
    REG_LUT_BASE = 0x100,
};
}

QList<AFI_SDB_ID> MpdTrcModule::getSupportedSdbIds()
{
    return {AFI_SDB_MPD_TRC};
}

bool MpdTrcModule::writeConfig(const MpdTrcConfig &config)
{
    if(!buildParams)
        return {};

    quint16 fhcalEastLogicCode = config.fhcalEastMajThr << 2;
    fhcalEastLogicCode |= static_cast<quint16>(config.fhcalEastLogic);
    quint16 fhcalWestLogicCode = config.fhcalWestMajThr << 2;
    fhcalWestLogicCode |= static_cast<quint16>(config.fhcalWestLogic);

    RegOpVector r;
    r.RegWrite16(REG_FHCAL_EAST_LOGIC, fhcalEastLogicCode);
    r.RegWrite16(REG_FHCAL_WEST_LOGIC, fhcalWestLogicCode);
    r.RegWrite16(REG_FHCAL_OUTPUT_LOGIC, static_cast<quint16>(config.fhcalOutputLogic));
    for(auto ch=0; ch<buildParams->inputChNum; ++ch) {
        r.RegWrite16(REG_INPUT_DELAY_BASE+ch, config.inputDelays.value(ch, 0));
        r.RegWrite16(REG_INPUT_SHAPER_BASE+ch, config.inputShapers.value(ch, 0));
    }

    auto lutRegSize = 1<<(buildParams->phyChNum);
    lutRegSize >>= 4;
    for(auto i=0; i<lutRegSize; ++i) {
        r.RegWrite16(REG_LUT_BASE+i, config.lut.value(i, 0));
    }

    return regOpExecRebased(r);
}

void MpdTrcModule::post_init()
{
    buildParams.reset();
    quint32 id;
    MpdTrcBuildParams bp;

    RegOpVector r;
    r.RegRead32(REG32_ID, &id);
    r.RegRead16(REG_NUM_CH_IN, &bp.inputChNum);
    r.RegRead16(REG_NUM_CH_PHY, &bp.phyChNum);

    if(regOpExecRebased(r) && getSupportedSdbIds().contains(static_cast<AFI_SDB_ID>(id))) {
        buildParams = bp;
    }
}
