#include "MscStreamConfigConverter.h"

#include "MscStreamConfig.h"
#include "base/ConfigConverterUtil.h"

namespace {
constexpr const char *k_enabled = "enabled";
constexpr const char *k_timeout = "timeout";
constexpr const char *k_slice = "slice";
constexpr const char *k_ceMonMask = "ceMonMask";
constexpr const char *k_histCe = "histCe";
}

template<>
bool AbstractConfigConverter<MscStreamConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<MscStreamConfig>::toJsonObject(const MscStreamConfig &in)
{
    QVariantMap m;
    m[k_enabled] = QVariant::fromValue(in.en);
    m[k_timeout] = QVariant::fromValue(in.timeout);
    m[k_slice] = QVariant::fromValue(in.slice);
    if(!in.ceMonitor.isEmpty())
        m[k_ceMonMask] = ConfigConverterUtil::toJsonValue(in.ceMonitor);
    if(!in.histCe.isEmpty())
        m[k_histCe] = ConfigConverterUtil::toJsonValue(in.histCe);
    return fromVariantMap(m);
}

template<>
MscStreamConfig AbstractConfigConverter<MscStreamConfig>::fromJsonObject(const QJsonObject &m)
{
    MscStreamConfig out;
    update_value(m, k_enabled, &out.en);
    update_value(m, k_timeout, &out.timeout);
    update_value(m, k_slice, &out.slice);
    if(m.contains(k_ceMonMask))
        out.ceMonitor = ConfigConverterUtil::toVector<bool>(m[k_ceMonMask]);
    if(m.contains(k_histCe))
        out.histCe = ConfigConverterUtil::toVector<bool>(m[k_histCe]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<MscStreamConfig>::toJsonValue(const MscStreamConfig &in)
{
    return toJsonObject(in);
}

template<>
MscStreamConfig AbstractConfigConverter<MscStreamConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<MscStreamConfig>::registerSubTypes()
{
}
