#ifndef RCCOUNTERS_H
#define RCCOUNTERS_H

#include <QMap>
#include <QMetaType>

enum DevCountersTypes : int {
    DevCountersRc,
    DevCountersTrcIn,
    DevCountersTrcBa,
    DevCountersTrcReduc
};
QString getDevCountersType(DevCountersTypes type, int typeId);

struct BaseDevCounters
{
    bool operator==(const BaseDevCounters &r) const { return cnt == r.cnt; }

    void clear() { cnt.clear(); }

    QMap<int, QMap<int, quint64>> cnt; // <cntType, <ch, cntVal> >
};
Q_DECLARE_METATYPE(BaseDevCounters)

typedef QMap<DevCountersTypes, BaseDevCounters> BaseDevCountersMap;
Q_DECLARE_METATYPE(BaseDevCountersMap)

namespace TrcInputCounters {
quint64 totalCnt(const BaseDevCounters &cnts, int ch);
}

struct RcCounters : public BaseDevCounters
{
    enum CNT_INDEX {
        TIME_TOTAL = 0,
        EDGE_TOTAL = 1,
        TIME_RUN   = 2,
        EDGE_RUN   = 3,
        TIME_SPILL = 4,
        EDGE_SPILL = 5,
        TIME_RUN_SPILL = 6,
        EDGE_RUN_SPILL = 7
    };

    /// First counter index: [0,1] - Total, [2,3] - Run, [4,5] - Spill, [6,7] - Run&Spill
    /// Even - time, odd - edges
    /// cntLevel[*][15] - Elapsed Time
    /// cntLevel[*][14] - XOFF Time

    qint64 trigAll() const { return cnt[1][14]; }
    qint64 trigOnRun() const { return cnt[3][14]; }
    qint64 trigOnSpill() const { return cnt[5][14]; }
    qint64 trigDeadtimeMissed() const { return cnt[7][13]; }
    qint64 trigOnRunOnSpill() const { return cnt[7][14]; }
    qint64 trigAccepted() const { return cnt[7][15]; }

    qint64 trigCountNotRun() const { return trigAll() - trigOnRun(); }
    qint64 trigCountNotSpill() const { return trigAll() - trigOnSpill(); }
    qint64 trigCountNotRunAndSpill() const { return trigOnSpill() - trigOnRunOnSpill(); }
    qint64 trigCountRunAndNotSpill() const { return trigOnRun() - trigOnRunOnSpill(); }
    qint64 trigCountNotRunAndNotSpill() const
    {
        return trigCountNotRun() - trigCountNotRunAndSpill();
    }
    qint64 trigMissed() const { return trigOnRunOnSpill() - trigAccepted(); }
    qint64 trigReadoutMissed() const { return trigMissed() - trigDeadtimeMissed(); }

    qint64 timeXoff() const { return cnt[6][14]; }

    // Elapsed Time
    qint64 timeClock() const { return cnt[0][15]; }
    qint64 timeRun() const { return cnt[2][15]; }
    qint64 timeSpill() const { return cnt[4][15]; }
    qint64 timeReadout() const { return cnt[6][15]; }
    qint64 timeStandby() const
    {
        qint64 v = timeClock() - timeRun();
        return (v > 0) ? v : 0;
    }

    double acceptRatio() const
    {
        if (trigOnRunOnSpill() > 0)
            return static_cast<double>(trigAccepted()) / trigOnRunOnSpill();
        else
            return 0.;
    }

    double readoutMissRatio() const
    {
        if (trigOnRunOnSpill() > 0)
            return static_cast<double>(trigReadoutMissed()) / trigOnRunOnSpill();
        else
            return 0.;
    }
    double deadtimeMissRatio() const
    {
        if (trigOnRunOnSpill() > 0)
            return static_cast<double>(trigDeadtimeMissed()) / trigOnRunOnSpill();
        else
            return 0.;
    }

    double deadTimeRatio() const
    {
        if (timeReadout() > 0)
            return 1.0 * timeXoff() / timeReadout();
        else
            return 0.;
    }
};
Q_DECLARE_METATYPE(RcCounters)

#endif // RCCOUNTERS_H
