/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RUNLOGICCONTROLLER_H
#define RUNLOGICCONTROLLER_H

#include "RunLogic/RunLogicConfig.h"
#include "RunLogic/RunLogicFsm.h"
#include "RunLogic/RunLogicMultiStatus.h"
#include "RunLogic/RunLogicStatus.h"
#include "base/DeviceIndexEnabled.h"
#include "mldiscover/DeviceDispatcher.h"

#include <QObject>
#include <QVariant>

class AsyncDeviceCommander;
class DeviceCommand;

enum class DeviceCommandType : int;

class RunLogicController : public QObject
{
    Q_OBJECT
public:
    enum Action : int {Reset, Run, Pause, Toggle};
    explicit RunLogicController(QObject *parent = nullptr);

    void setDeviceCommander(const AsyncDeviceCommander *deviceCommander);
    void setDeviceDispatcher(DeviceDispatcher *deviceDispatcher);

    RunLogicMultiStatus getStatus() const { return multiStatus; }

    RunLogicFsm fsm;
    RunLogicMultiStatus multiStatus;

signals:
    void runLogicConfigChanged(const RunLogicConfig &config);
    void runLogicStatusChanged(const RunLogicMultiStatus &);
    void stateChanged(int st);
    void submitCommand(const DeviceCommand &command);

public slots:
    bool performAction(int action);
    void commandComplete(const DeviceCommand &command);
    void setDeviceList(const DeviceIndexEnabledMap &new_devices);
    void setRunLogicStatus(const DeviceIndex &deviceIndex, const RunLogicStatus &newStatus);
    void setRunLogicConfig(const RunLogicConfig &newConfig);
    // void setDeviceStatus(const DeviceStatusBase &newStatus);

private:
    DeviceCommand makeCommand(DeviceCommandType type, const QVariant &data = {});
    bool setState(RunLogicFsm::State toState);
    void checkAutoPause();
};

#endif // RUNLOGICCONTROLLER_H
