/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RUNLOGICMULTISTATUS_H
#define RUNLOGICMULTISTATUS_H

#include "RunLogic/RunLogicConfig.h"
#include "RunLogic/RunLogicStatus.h"
#include "base/DeviceIndex.h"
#include "base/DeviceIndexEnabled.h"

#include <QList>

struct RunLogicMultiStatus
{
    void setDeviceList(const DeviceIndexEnabledMap &new_devices);
    void setRunLogicStatus(const DeviceIndex &deviceIndex, const RunLogicStatus &newStatus);
    void setRunLogicConfig(const RunLogicConfig &newConfig);
    RunLogicConfig runLogicConfig() const { return config; }

    bool allIdle() const;
    bool allRunning() const;
    bool allFinished() const;

    double progress() const;
    double minRunTime() const;
    qint64 minEventCount() const;
    QStringList statusText() const;
    QString getFullText() const;

    bool isEmpty = false;
    DeviceIndexSet missing;
    DeviceIndexSet invalid;
    DeviceIndexSet idle;
    DeviceIndexSet running;
    DeviceIndexSet finished;

    QVector<double> multiRunTime;
    QVector<quint32> multiEventCount;
    QVector<double> multiProgress;
    bool anyNotIdle = false;

private:
    void update();

    RunLogicConfig config;
    DeviceIndexSet deviceSet;
    QMap<DeviceIndex, RunLogicStatus> status;
};
Q_DECLARE_METATYPE(RunLogicMultiStatus)

#endif // RUNLOGICMULTISTATUS_H
