/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "StatReadoutConfigConverter.h"
#include "device-modules/StatReadout/StatReadoutConfig.h"

namespace {
constexpr const char *k_enable = "enable";
constexpr const char *k_period = "period";
constexpr const char *k_preset = "preset";
} // namespace

template<>
bool AbstractConfigConverter<StatReadoutConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<StatReadoutConfig>::toJsonObject(const StatReadoutConfig &in)
{
    QVariantMap m;
    m[k_enable] = QVariant::fromValue(in.enable);
    m[k_period] = QVariant::fromValue(in.period);
    m[k_preset] = QVariant::fromValue(in.preset);
    return fromVariantMap(m);
}

template<>
StatReadoutConfig AbstractConfigConverter<StatReadoutConfig>::fromJsonObject(const QJsonObject &in)
{
    StatReadoutConfig out;
    update_value(in, k_enable, &out.enable);
    update_value(in, k_period, &out.period);
    update_value(in, k_preset, &out.preset);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<StatReadoutConfig>::toJsonValue(const StatReadoutConfig &in)
{
    return toJsonObject(in);
}

template<>
StatReadoutConfig AbstractConfigConverter<StatReadoutConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<StatReadoutConfig>::registerSubTypes()
{
    AbstractConfigConverter<StatPreset>::registerTypes();
}
