/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "TrigInfoModule.h"

#include "TrigInfoStatus.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

namespace {
enum {
    REG32_ID = 0x0,
    REG_CONTROL = 0x2,
    REG_LOCK = 0x3,
    REG32_STATUS = 0x4,
    REG32_TRIGBITS = 0x6,
    REG64_GL_EV_NUM = 0x08,
    REG64_GL_EV_TS = 0x0C,
    REG64_LOAD_EV_NUM = 0x10,
};
const quint16 REG_CONTROL_BIT_RESET = 1;
const quint16 REG_CONTROL_BIT_LOAD = 2;
const quint16 REG_LOCK_BIT_LOCK = 1;
} // namespace

TrigInfoModule::TrigInfoModule(const MregBusInterface &bus)
    : BaseDeviceModule(bus)
{
}

QList<AFI_SDB_ID> TrigInfoModule::getSupportedSdbIds()
{
    return {AFI_SDB_TRIG_INFO};
}

std::optional<TrigInfoStatus> TrigInfoModule::readStatus()
{
    TrigInfoStatus status;
    if (!readStatus(&status))
        return {};
    return status;
}

bool TrigInfoModule::readStatus(TrigInfoStatus *result)
{
    TrigInfoStatus status;
    RegOpVector r;
    r.RegRead32(REG32_ID, &status.id);
    r.RegWrite16(REG_LOCK, 0);
    r.RegWrite16(REG_LOCK, REG_LOCK_BIT_LOCK);
    r.RegRead32(REG32_STATUS, &status.status);
    r.RegRead32(REG32_TRIGBITS, &status.trigbits);
    r.RegRead64(REG64_GL_EV_TS, &status.ts);
    r.RegRead64(REG64_GL_EV_NUM, &status.eventNumber);
    r.RegWrite16(REG_LOCK, 0);
    if (!regOpExecRebased(r))
        return {};
    if (result)
        *result = status;
    return true;
}

bool TrigInfoModule::softreset()
{
    RegOpVector r;
    r.RegWrite16(REG_CONTROL, REG_CONTROL_BIT_RESET);
    r.RegWrite16(REG_CONTROL, 0);
    return regOpExecRebased(r);
}

bool TrigInfoModule::writeGlobEvNum(quint64 value, bool reset)
{
    RegOpVector r;
    r.RegWrite64(REG64_LOAD_EV_NUM, value);
    quint16 val = REG_CONTROL_BIT_LOAD | (reset ? REG_CONTROL_BIT_RESET : 0);
    r.RegWrite16(REG_CONTROL, val);
    r.RegWrite16(REG_CONTROL, 0);
    return regOpExecRebased(r);
}
