/*
**    Copyright 2011-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TTVXSCORECONFIG_H
#define TTVXSCORECONFIG_H

#include <QMap>
#include <QMetaType>

typedef union reg_trig_src {
    quint16 all=0;
    struct {
        unsigned int ttl: 1;
        unsigned int sfp: 1;
        unsigned int unused2: 12;
        unsigned int random_pulser: 1;
        unsigned int fixed_pulser: 1;
    } bit;
} reg_trig_src;

inline bool operator==(const reg_trig_src &l, const reg_trig_src &r)
{
    return l.all == r.all;
}

Q_DECLARE_METATYPE(reg_trig_src)

struct TtvxsBusySourceConfig
{
    friend bool operator==(const TtvxsBusySourceConfig &, const TtvxsBusySourceConfig &) = default;
    bool vxsPulse = true;
    bool vxsFeLink = false;
};
Q_DECLARE_METATYPE(TtvxsBusySourceConfig)

struct TtvxsCoreConfig
{
    friend bool operator==(const TtvxsCoreConfig &, const TtvxsCoreConfig &) = default;
    void mergeLocalData(const TtvxsCoreConfig &r)
    {
        vxsChMask = r.vxsChMask;
    }

    reg_trig_src trigSrc = {};
    quint32 vxsChMask = 0;
    TtvxsBusySourceConfig busySource;
    bool trig_open = false;
    bool aux_cnt_for_trig = false;
};

Q_DECLARE_METATYPE(TtvxsCoreConfig)

#endif // TTVXSCORECONFIG_H
