#include "Ut24TtlIoConfig.h"

#include "Ut24TtlIoBuildParams.h"

#include <QMap>
#include <QList>

namespace {
static const QMap<Ut24TtlIoOutput, QString> Ut24TtlIoOutput_Names = {
    {Ut24TtlIoOutput::Spill, "Spill"},
    {Ut24TtlIoOutput::L0_trig, "L0_trig"},
    {Ut24TtlIoOutput::L1_trig, "L1_trig"},
    {Ut24TtlIoOutput::Busy, "Busy"},
    {Ut24TtlIoOutput::Logic0, "Logic0"},
    {Ut24TtlIoOutput::Logic1, "Logic1"}
};

static const QMap<Ut24TtlIoOutput, QString> Ut24TtlIoOutput_PrettyNames = {
    {Ut24TtlIoOutput::Spill, "Spill"},
    {Ut24TtlIoOutput::L0_trig, "L0 trigger"},
    {Ut24TtlIoOutput::L1_trig, "L1 trigger"},
    {Ut24TtlIoOutput::Busy, "Busy"},
    {Ut24TtlIoOutput::Logic0, "Logic 0"},
    {Ut24TtlIoOutput::Logic1, "Logic 1"}
};

static const QList<Ut24TtlIoOutput> Ut24TtlIoOutput_List = Ut24TtlIoOutput_Names.keys();
}

void reg_ut24_ttl_io_t::setInput(Ut24TtlIoInput i, const Ut24TtlIoBuildParams &bp)
{
    bit.isOutput = 0;
    switch (i.type) {
    case Ut24TtlIoInputType::Spill:
        Q_ASSERT(i.ch==0);
        bit.cfg = 0;
        return;
    case Ut24TtlIoInputType::Trigger:
        Q_ASSERT(i.ch<bp.trigInNum);
        bit.cfg = 1+i.ch;
        return;
    case Ut24TtlIoInputType::Busy:
        Q_ASSERT(i.ch<bp.busyInNum);
        bit.cfg = 1+bp.trigInNum+i.ch;
        return;
    case Ut24TtlIoInputType::Readout:
        Q_ASSERT(i.ch<2);
        bit.cfg = 1+bp.trigInNum+bp.busyInNum+i.ch;
        return;
    default:
        Q_UNREACHABLE();
    }
}

void reg_ut24_ttl_io_t::setOutput(Ut24TtlIoOutput o)
{
    bit.isOutput = 1;
    bit.cfg = static_cast<int>(o);
}

Ut24TtlIoInput reg_ut24_ttl_io_t::getInput(const Ut24TtlIoBuildParams &bp) const
{
    Ut24TtlIoInput ret;
    Q_ASSERT(bit.isOutput == 0);
    if(bit.cfg==0) {
        ret.type = Ut24TtlIoInputType::Spill;
        ret.ch = 0;
        return ret;
    } else if(bit.cfg<1+bp.trigInNum) {
        ret.type = Ut24TtlIoInputType::Trigger;
        ret.ch = bit.cfg-1;
        return ret;
    } else if(bit.cfg<1+bp.trigInNum+bp.busyInNum) {
        ret.type = Ut24TtlIoInputType::Busy;
        ret.ch = bit.cfg-1-bp.trigInNum;
        return ret;
    } else {
        ret.type = Ut24TtlIoInputType::Readout;
        ret.ch = bit.cfg-1-bp.trigInNum-bp.busyInNum;
        Q_ASSERT(ret.ch<2);
        return ret;
    }
}

Ut24TtlIoOutput reg_ut24_ttl_io_t::getOutput() const
{
    Q_ASSERT(bit.isOutput == 1);
    return static_cast<Ut24TtlIoOutput>(bit.cfg);
}

QString reg_ut24_ttl_io_t::toString(const Ut24TtlIoBuildParams &bp) const
{
    return bit.isOutput ? Ut24TtlIoConfig::getOutputName(getOutput())
                        : Ut24TtlIoConfig::getInputName(getInput(bp));
}

QList<Ut24TtlIoInput> Ut24TtlIoConfig::getInputs(const Ut24TtlIoBuildParams &bp)
{
    QList<Ut24TtlIoInput> ret;
    reg_ut24_ttl_io_t reg;
    reg.bit.isOutput = 0;
    reg.bit.cfg = 0;
    while(reg.bit.cfg < 1+bp.trigInNum+bp.busyInNum+2) {
        ret << reg.getInput(bp);
        ++reg.bit.cfg;
    }
    return ret;
}

QList<Ut24TtlIoOutput> Ut24TtlIoConfig::getOutputs()
{
    return Ut24TtlIoOutput_List;
}

QString Ut24TtlIoConfig::getInputName(const Ut24TtlIoInput &type)
{
    switch (type.type) {
    case Ut24TtlIoInputType::Spill:
        return "Spill";
    case Ut24TtlIoInputType::Trigger:
        return QString("Trigger #%1").arg(type.ch);
    case Ut24TtlIoInputType::Busy:
        return QString("Busy #%1").arg(type.ch);
    case Ut24TtlIoInputType::Readout:
        return QString("Readout #%1").arg(type.ch);
    default:
        Q_UNREACHABLE();
    }
}

QString Ut24TtlIoConfig::getOutputName(const Ut24TtlIoOutput &type)
{
    return Ut24TtlIoOutput_Names.value(type);
}

QString Ut24TtlIoConfig::getPrettyInputName(const Ut24TtlIoInput &type)
{
    return QString(getInputName(type)).replace(' ', '_');
}

QString Ut24TtlIoConfig::getPrettyOutputName(const Ut24TtlIoOutput &type)
{
    auto it = Ut24TtlIoOutput_PrettyNames.constFind(type);
    if(it != Ut24TtlIoOutput_PrettyNames.cend())
        return *it;
    return Ut24TtlIoOutput_PrettyNames.value(type);
}
