#ifndef UT24TTLIOCONFIG_H
#define UT24TTLIOCONFIG_H

#include <QMetaType>
#include <QVector>

struct Ut24TtlIoBuildParams;

enum class Ut24TtlIoInputType
{
    Spill   = 0x00,
    Trigger = 0x01,
    Busy    = 0x02,
    Readout = 0x03,
};

struct Ut24TtlIoInput
{
    Ut24TtlIoInputType type = Ut24TtlIoInputType::Spill;
    int ch=0;
};

enum class Ut24TtlIoOutput
{
    Spill   = 0x00,
    L0_trig = 0x01,
    L1_trig = 0x02,
    Busy    = 0x03,
    Logic0  = 0x04,
    Logic1  = 0x05,
};

typedef union reg_ut24_ttl_io_t {
    void setInput(Ut24TtlIoInput i, const Ut24TtlIoBuildParams &bp);
    void setOutput(Ut24TtlIoOutput o);
    Ut24TtlIoInput getInput(const Ut24TtlIoBuildParams &bp) const;
    Ut24TtlIoOutput getOutput() const;
    QString toString(const Ut24TtlIoBuildParams &bp) const;

    uint16_t all = 0;
    struct {
        uint16_t cfg: 15;
        uint16_t isOutput: 1;
    } bit;
} reg_ut24_ttl_io_t;
Q_DECLARE_METATYPE(reg_ut24_ttl_io_t)

inline bool operator==(const reg_ut24_ttl_io_t &l, const reg_ut24_ttl_io_t &r)
{
    return l.all == r.all;
}

struct Ut24TtlIoConfig
{
    static QList<Ut24TtlIoInput> getInputs(const Ut24TtlIoBuildParams &bp);
    static QList<Ut24TtlIoOutput> getOutputs();
    static QString getInputName(const Ut24TtlIoInput &type);
    static QString getOutputName(const Ut24TtlIoOutput &type);
    static QString getPrettyInputName(const Ut24TtlIoInput &type);
    static QString getPrettyOutputName(const Ut24TtlIoOutput &type);

    friend bool operator==(const Ut24TtlIoConfig &l, const Ut24TtlIoConfig &r) = default;

    QVector<reg_ut24_ttl_io_t> ttlCfg;
};
Q_DECLARE_METATYPE(Ut24TtlIoConfig)

#endif // UT24TTLIOCONFIG_H
