#include "Ut24TtlIoModule.h"

#include "DeviceModuleCommand.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

namespace {
enum {
    REG32_ID = 0x00, // - Identification cef86b29, 32 bits, RO static
    REG_CTRL = 0x02, // - Control, R/W // [0] - softreset
    REG_CH_NUM = 0x04, // Number of channels, RO static
    REG_IN_TRIGS_NUM = 0x05, // Number of input triggers, RO static
    REG_IN_BUSY_NUM = 0x06, // Number of input busy signals (xoffs) , RO statcic

    REG_CH_BASE = 0x10, // R/W, 16 bit per ch
};
}

QList<AFI_SDB_ID> Ut24TtlIoModule::getSupportedSdbIds()
{
    return {AFI_SDB_UT24VE_TTL_IO_CTRL};
}

bool Ut24TtlIoModule::writeConfig(const Ut24TtlIoConfig &config)
{
    if(!buildParams)
        return {};

    RegOpVector r;
    for(auto ch=0; ch<buildParams->chNum; ++ch) {
        reg_ut24_ttl_io_t defReg;
        if(initConfig)
            defReg = initConfig->ttlCfg.value(ch);
        r.RegWrite16(REG_CH_BASE+ch, config.ttlCfg.value(ch, defReg).all);
    }

    return regOpExecRebased(r);
}

void Ut24TtlIoModule::post_init()
{
    buildParams.reset();
    initConfig.reset();
    quint32 id;
    Ut24TtlIoBuildParams bp;

    RegOpVector r;
    r.RegRead32(REG32_ID, &id);
    r.RegRead16(REG_CH_NUM, &bp.chNum);
    r.RegRead16(REG_IN_TRIGS_NUM, &bp.trigInNum);
    r.RegRead16(REG_IN_BUSY_NUM, &bp.busyInNum);

    if(regOpExecRebased(r) && getSupportedSdbIds().contains(static_cast<AFI_SDB_ID>(id))) {
        buildParams = bp;
        if(auto cfg=readConfig())
            initConfig = cfg;
    }
}

std::optional<Ut24TtlIoConfig> Ut24TtlIoModule::readConfig()
{
    if(!buildParams)
        return {};

    Ut24TtlIoConfig cfg;
    cfg.ttlCfg.resize(buildParams->chNum);
    RegOpVector r;
    for(auto ch=0; ch<buildParams->chNum; ++ch) {
        r.RegRead16(REG_CH_BASE+ch, &cfg.ttlCfg[ch].all);
    }

    if(regOpExecRebased(r))
        return cfg;

    return {};
}
