/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef WAVEBLCCONFIG_H
#define WAVEBLCCONFIG_H

#include <QMap>
#include <QMetaType>

class WaveBlcChannelConfig
{
public:
    enum Polarity {Positive = 0, Negative = 1};
    enum Mode {Bypass = 0, Correction = 1, Debug = 2, Test = 3};
    friend bool operator==(const WaveBlcChannelConfig &l, const WaveBlcChannelConfig &r) = default;
    static QPair<int, int> mafRoundoffRange() { return {0, 7}; }
    static QPair<int, int> thrRange() { return {4, 0xFFFC}; }

    Polarity polarity = Polarity::Positive;
    Mode mode = Mode::Bypass;
    quint16 mafLength = 16;
    quint16 mafRoundoff = 0;
    qint16 thr = 100;
    qint16 staticOffset = 0;
};
Q_DECLARE_METATYPE(WaveBlcChannelConfig)

class WaveBlcConfig
{
public:
    friend bool operator==(const WaveBlcConfig &, const WaveBlcConfig &) = default;
    void mergeLocalData(const WaveBlcConfig &)
    {
        // nothing
    }
    WaveBlcChannelConfig getChannel(int channel) const
    {
        auto found = channels.find(channel);
        if (found == channels.end())
            return common;
        return *found;
    }
    void setChannel(int channel, const WaveBlcChannelConfig &channelConfig)
    {
        channels.insert(channel, channelConfig);
    }
    WaveBlcChannelConfig common;
    qint16 commonOffset = 0;

private:
    QMap<int, WaveBlcChannelConfig> channels;
};
Q_DECLARE_METATYPE(WaveBlcConfig)

#endif // WAVEBLCCONFIG_H
