/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "WaveBlcStatus.h"
#include "sdb/AfiSdbId.h"

bool WaveBlcStatus::isValid() const
{
    const int MAX_CHANNEL_REGS = 8;
    const int MAX_CHANNEL_COUNT = 128;
    bool ok = (id == AFI_SDB_WAVE_BLC)
        && (channelCount > 0 && channelCount <= MAX_CHANNEL_COUNT)
        && !arrayAddress.empty();
    for (auto it = arrayAddress.begin(); it != arrayAddress.end(); ++it) {
        auto offset = it.value();
        ok &= (offset >= 0x10 && offset <= MAX_CHANNEL_REGS*MAX_CHANNEL_COUNT);
    }
    return ok;
}
