#ifndef ADCSERDESCALIBRATION_H
#define ADCSERDESCALIBRATION_H

struct AdcDeserCheckStatus;
class AdcSpiBaseModule;
class AdcDataTestModule;
class DeserializersControlModule;
class MlinkCsrModule;
class ModularMregDevice;

#include <QMap>
#include <QObject>
#include <QSharedPointer>
#include <QVector>

#include "AdcClkStatus.h"

class AdcSerDesCalibration : public QObject
{
    Q_OBJECT
public:
    explicit AdcSerDesCalibration(ModularMregDevice *parent);

    void init();
    void deinit();

    AdcClkStatus getStatus() const { return delayStatus; }

    bool calibrate(bool hardMode, const QString &resDir);
    bool checkAdcDeserConfig(const AdcSerDesDelayMap &delayBase,
                               bool hardMode = false,
                               bool printFaliedCh = false);
    static void saveResultsToFile(const DeviceIndex &index,
                                  const AdcClkStatus &delayStatus,
                                  const QString &resDir);

signals:
    void progressUpdated(int percentage);
    void adcDeserConfigChecked(const DeviceIndex &, const AdcDeserCheckStatus &st) const;

private:
    void generateSpiPatternSet();
    QMap<int, bool> patternTestByChannel();
    QMap<int, bool> TestAdcCustomPatternByChannel(const quint16 &pattern);
    inline int getNumberOfChannels() const { return desCnt*desCh; }
    double readTemp() const;
    QString getFirmwareVersionStr() const;
    quint8 adcBits() const;

    ModularMregDevice * dev = nullptr;
    QSharedPointer<AdcDataTestModule> adcDataTest;
    QSharedPointer<AdcSpiBaseModule> adcSpi;
    QSharedPointer<DeserializersControlModule> desCtrl;
    QSharedPointer<MlinkCsrModule> mlinkCsrModule;

    QVector<quint16> spiPatternSet;
    quint8 desCnt = 0;
    quint8 desCh=0; // channels per des
    int clkDelay=0;
    int dataDelay=0;

    bool hardPatternTestEnabled = false;
    bool prbsMode = false;

    AdcClkStatus delayStatus; // stores results of last calibrate()
};

#endif // ADCSERDESCALIBRATION_H
