#include "RcTrigControlWidget.h"
#include "ui_RcTrigControlWidget.h"

#include <QComboBox>
#include <QLabel>
#include <QSizePolicy>

#include <QDebug>

#include "base/Globals.h"

RcTrigControlWidget::RcTrigControlWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::RcTrigControlWidget)
{
    ui->setupUi(this);

    ui->checkBoxTrigLvds->hide();
}

RcTrigControlWidget::~RcTrigControlWidget()
{
    delete ui;
}

void RcTrigControlWidget::updateExtTrigGroup()
{
//    bool moreThanOne = config->trigInMoreThatOne();
//    {return rcTrigConfig.deviceIndex.getDevId() == DEVICE_ID_UT24VE;}
    bool moreThanOne = false;
    ui->checkBoxTrigIn->setText(QString("External TTL")
                                .append(moreThanOne ? " 1" : ""));
    ui->checkBoxTrigIn2->setVisible(moreThanOne);
    ui->checkBoxTrigIn3->setVisible(moreThanOne);
    ui->checkBoxTrigIn4->setVisible(moreThanOne);
}

void RcTrigControlWidget::restoreGuiFromConfig()
{
    // trigChannelConfig->updateGui();
    ++blockSignal;

    bool oldState6 = ui->checkBoxTrigIn->blockSignals(true);
    bool oldState7 = ui->checkBoxTrigPeriod->blockSignals(true);
    bool oldState8 = ui->checkBoxTrigRandom->blockSignals(true);
    bool oldState9 = ui->doubleSpinBoxTrigTimerFreq->blockSignals(true);
    bool oldState10 = ui->doubleSpinBoxTrigRandomFreq->blockSignals(true);
    bool oldState11 = ui->doubleSpinBoxTrigRandomDeadTime->blockSignals(true);

    ui->radioButtonTrigOpen->blockSignals(true);
    if(cfg.trigOpen)
        ui->radioButtonTrigOpen->setChecked(true);
    else
        ui->radioButtonTrigClose->setChecked(true);
    ui->radioButtonTrigOpen->blockSignals(false);

    ui->checkBoxTrigIn->setChecked(cfg.trigSrc & RcTrigConfig::TrigMaskTtl_1);
    ui->checkBoxTrigIn2->setChecked(cfg.trigSrc & RcTrigConfig::TrigMaskTtl_2);
    ui->checkBoxTrigIn3->setChecked(cfg.trigSrc & RcTrigConfig::TrigMaskTtl_3);
    ui->checkBoxTrigIn4->setChecked(cfg.trigSrc & RcTrigConfig::TrigMaskTtl_4);
    ui->checkBoxTrigPeriod->setChecked(cfg.trigSrc & RcTrigConfig::TrigMaskTimer);
    ui->checkBoxTrigRandom->setChecked(cfg.trigSrc & RcTrigConfig::TrigMaskRandom);
    ui->checkBoxTrigLvds->setChecked(cfg.trigSrc & RcTrigConfig::TrigMaskLvds);
    ui->doubleSpinBoxTrigTimerFreq->setMinimum(RcTrigConfig::getTrigTimerMinFreqHz());
    ui->doubleSpinBoxTrigTimerFreq->setMaximum(std::min(10e6, RcTrigConfig::getTrigTimerMaxFreqHz()));
    ui->doubleSpinBoxTrigTimerFreq->setValue(cfg.trigTimerFreq);
    ui->doubleSpinBoxTrigTimerFreq->setEnabled(cfg.trigSrc & RcTrigConfig::TrigMaskTimer);
    ui->doubleSpinBoxTrigRandomFreq->setMinimum(RcTrigConfig::getTrigTimerMinFreqHz());
    ui->doubleSpinBoxTrigRandomFreq->setMaximum(std::min(10e6, RcTrigConfig::getTrigTimerMaxFreqHz()));
    ui->doubleSpinBoxTrigRandomFreq->setValue(cfg.trigRandomFreq);
    ui->doubleSpinBoxTrigRandomFreq->setEnabled(cfg.trigSrc & RcTrigConfig::TrigMaskRandom);

    ui->doubleSpinBoxTrigRandomFreq->setMinimum(0);
    ui->doubleSpinBoxTrigRandomFreq->setMaximum(1e6);
    ui->doubleSpinBoxTrigRandomDeadTime->setValue(cfg.trigRandomDeadTimeUs);

    updateExtTrigGroup();

    ui->checkBoxTrigIn->blockSignals(oldState6);
    ui->checkBoxTrigPeriod->blockSignals(oldState7);
    ui->checkBoxTrigRandom->blockSignals(oldState8);
    ui->doubleSpinBoxTrigTimerFreq->blockSignals(oldState9);
    ui->doubleSpinBoxTrigRandomFreq->blockSignals(oldState10);
    ui->doubleSpinBoxTrigRandomDeadTime->blockSignals(oldState11);
    --blockSignal;
}

void RcTrigControlWidget::setTrigSrcMask(quint16 mask, bool checked)
{
    if (checked)
        cfg.trigSrc |= mask;
    else
        cfg.trigSrc &= ~mask;
    emit rcTrigCfgChanged(cfg);
}

void RcTrigControlWidget::setRcTrigCfg(const RcTrigConfig &ttlCfg)
{
    if(cfg == ttlCfg)
        return;
    cfg = ttlCfg;

    restoreGuiFromConfig();
}

void RcTrigControlWidget::onFsmStateChange(const FsmState &state)
{
    bool en = state==FsmState::Ready || state==FsmState::Error
            || Globals::getDebugMode();
    setEnabled(en);
}

void RcTrigControlWidget::on_radioButtonTrigOpen_toggled(bool open)
{
    cfg.trigOpen = open;
    emit rcTrigCfgChanged(cfg);
}

void RcTrigControlWidget::on_doubleSpinBoxTrigTimerFreq_valueChanged(double arg1)
{
    cfg.trigTimerFreq = arg1;
    emit rcTrigCfgChanged(cfg);
}

void RcTrigControlWidget::on_doubleSpinBoxTrigRandomFreq_valueChanged(double arg1)
{
    cfg.trigRandomFreq = arg1;
    emit rcTrigCfgChanged(cfg);
}

void RcTrigControlWidget::on_doubleSpinBoxTrigRandomDeadTime_valueChanged(double arg1)
{
    cfg.trigRandomDeadTimeUs = arg1;
    emit rcTrigCfgChanged(cfg);
}

void RcTrigControlWidget::on_checkBoxTrigIn_toggled(bool checked)
{
    setTrigSrcMask(0x1, checked);
}

void RcTrigControlWidget::on_checkBoxTrigIn2_toggled(bool checked)
{
    setTrigSrcMask(0x2, checked);
}

void RcTrigControlWidget::on_checkBoxTrigIn3_toggled(bool checked)
{
    setTrigSrcMask(0x4, checked);
}

void RcTrigControlWidget::on_checkBoxTrigIn4_toggled(bool checked)
{
    setTrigSrcMask(0x8, checked);
}

void RcTrigControlWidget::on_checkBoxTrigLvds_toggled(bool checked)
{
//    ui->spinBoxLvdsMask->setVisible(prevTrigStatus->customAttr.isL0Module && checked);
//    ui->labelLvdsMask->setVisible(prevTrigStatus->customAttr.isL0Module && checked);
    setTrigSrcMask(RcTrigConfig::TrigMaskLvds, checked);
}

void RcTrigControlWidget::on_checkBoxTrigPeriod_toggled(bool checked)
{
    ui->doubleSpinBoxTrigTimerFreq->setEnabled(checked);
    setTrigSrcMask(RcTrigConfig::TrigMaskTimer, checked);
}

void RcTrigControlWidget::on_checkBoxTrigRandom_toggled(bool checked)
{
    ui->doubleSpinBoxTrigRandomFreq->setEnabled(checked);
    setTrigSrcMask(RcTrigConfig::TrigMaskRandom, checked);
}
