/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TRGICTRLSTATWIDGET_H
#define TRGICTRLSTATWIDGET_H

#include "TrigCtrlStat/TrigCtrlStatConfig.h"
#include "base/DeviceIndex.h"
#include "fsm/FsmState.h"

#include <optional>
#include <QWidget>
#include <QMap>

namespace Ui {
class TrigCtrlStatWidget;
}

class QAbstractButton;
struct DeviceIndexEnabledMap;
struct TrigCtrlBuildParams;

enum TrigSource : quint32;

class TrigCtrlStatWidget : public QWidget
{
    Q_OBJECT

public:
    explicit TrigCtrlStatWidget(QWidget *parent = nullptr);
    virtual ~TrigCtrlStatWidget() override;
    void updateExtTrigGroup();

public slots:
    void setTrigCtrlStatConfig(const TrigCtrlStatConfig &newConfig);
    void setTrigCtrlBp(const DeviceIndex &index, const TrigCtrlBuildParams &bp);
    void setDeviceList(const DeviceIndexEnabledMap &map);
    void onFsmStateChange(const FsmState &state);

private slots:
    // Trig Input
    void on_doubleSpinBoxTimerFreq_valueChanged(double arg1);
    void on_doubleSpinBoxRandomFreq_valueChanged(double arg1);
    void on_doubleSpinBoxRandomMinPeriod_valueChanged(double arg1);
    void on_doubleSpinBoxDeadTime_valueChanged(double arg1);

    void onTrigSourceButtonToggle(bool checked);

    void on_spinBoxAnalogThrDelay_valueChanged(int arg1);
    void on_spinBoxDigitalThrDelay_valueChanged(int arg1);

signals:
    void trigCtrlStatConfigChanged(const TrigCtrlStatConfig &config);

private:
    void restoreGuiFromConfig();
    void updateEnableState();
    void updateLabels();
    void updateTrigSourceCaps();
    void updateTrigModClock();
    std::optional<double> getClockStep() const;

    Ui::TrigCtrlStatWidget *ui;
    TrigCtrlStatConfig cfg;
    QMap<QAbstractButton *, TrigCtrlStatConfig::TrigSource> sourceMap;
    DeviceIndexSet devSet;
    QMap<DeviceIndex, TrigCtrlBuildParams> devBuildParams;
    quint32 trigSourceCaps = ~0u;
    std::optional<double> trigModClockStep;
};

#endif // TRGICTRLSTATWIDGET_H
