#include "SingleUt24TtlWidget.h"

#include <QComboBox>
#include <QLabel>
#include <QSizePolicy>

#include <QDebug>

#include "base/Globals.h"

SingleUt24TtlWidget::SingleUt24TtlWidget(int ch, const Ut24TtlIoBuildParams &bp, QWidget *parent)
    : QWidget(parent)
    , ch(ch)
    , lab(new QLabel(this))
    , cb(new QComboBox(this))
    , buildParams(bp)
{
    lab->setText(QString("LEMO #%1").arg(ch+1));
    lab->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);

    QSizePolicy sp = cb->sizePolicy();
    sp.setHorizontalPolicy(QSizePolicy::Fixed);
    cb->setSizePolicy(sp);
    // cb->setSizeAdjustPolicy(QComboBox::AdjustToContents);
    updateItems();
}

SingleUt24TtlWidget::~SingleUt24TtlWidget()
{
    delete cb;
    delete lab;
}

void SingleUt24TtlWidget::setRegConfig(const reg_ut24_ttl_io_t &regCfg)
{
    const auto &targ = QVariant::fromValue(regCfg.all);
    int id = cb->findData(targ);
    if(id == -1) {
        cb->insertSeparator(cb->count());
        cb->addItem(QString("Unknown config 0x%1").arg(regCfg.all,0,16), regCfg.all);
        id = cb->findData(targ);
    }
    cb->setCurrentIndex(id);
}

void SingleUt24TtlWidget::onUt24TtlIoBuildParamsChange(const Ut24TtlIoBuildParams &bp)
{
    if(buildParams && *buildParams == bp)
        return;
    buildParams = bp;
    QSignalBlocker block(cb);
    auto curData = cb->currentData();
    updateItems();
    setRegConfig({.all = curData.toUInt()});
}

void SingleUt24TtlWidget::updateItems() const
{
    while(cb->count())
        cb->removeItem(0);
    QStringList cbToolTip;
    if(!buildParams) {
        cb->setToolTip({});
        return;
    }

    bool isTrc = Globals::getDaqModule() == DaqModule::Type::MpdTrc;
    bool isRc = Globals::getDaqModule() == DaqModule::Type::RunControl;
    cbToolTip << "Inputs:";
    for(auto in : Ut24TtlIoConfig::getInputs(*buildParams)) {
        reg_ut24_ttl_io_t r;
        if(isTrc && (in.type == Ut24TtlIoInputType::Spill ||
                      in.type == Ut24TtlIoInputType::Busy))
            continue;
        if(isRc && in.type == Ut24TtlIoInputType::Readout)
            continue;
        r.setInput(in, *buildParams);
        auto chName = Ut24TtlIoConfig::getInputName(in);
        if(in.type == Ut24TtlIoInputType::Trigger && trigInputNameMapFunc)
            chName = trigInputNameMapFunc(in.ch);
        cb->addItem("In: " + chName, r.all);
        cbToolTip << QString("%1 - %2").arg(in.ch).arg(chName);
    }
    cb->insertSeparator(cb->count());
    cbToolTip << "" << "Outputs:";
    for(auto out : Ut24TtlIoConfig::getOutputs()) {
        if(isTrc && (out == Ut24TtlIoOutput::Spill))
            continue;
        reg_ut24_ttl_io_t r;
        r.setOutput(out);
        auto name = Ut24TtlIoConfig::getPrettyOutputName(out);
        cb->addItem("Out: " + name, r.all);
        cbToolTip << QString("%1 - %2").arg(static_cast<uint>(out)).arg(name);
    }
    cb->setToolTip(cbToolTip.join('\n'));
}

void SingleUt24TtlWidget::setTrigInputNameMap(const std::function<QString (const int)> &newTrigInputNameMap)
{
    trigInputNameMapFunc = newTrigInputNameMap;
    updateItems();
}
