#include "Ut24TtlIoWidget.h"
#include "ui_Ut24TtlIoWidget.h"
#include "SingleUt24TtlWidget.h"
#include <QComboBox>
#include <QLabel>
#include <QSizePolicy>

#include <QDebug>

#include "base/Globals.h"

Ut24TtlIoWidget::Ut24TtlIoWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::Ut24TtlIoWidget)
{
    ui->setupUi(this);
}

Ut24TtlIoWidget::~Ut24TtlIoWidget()
{
    delete ui;
}

void Ut24TtlIoWidget::setTrigInputNameMap(const std::function<QString (const int)> &f)
{
    trigInputNameMapFunc = f;
    for(SingleUt24TtlWidget *w : singleTtlWidgets)
        w->setTrigInputNameMap(f);
}

void Ut24TtlIoWidget::setUt24TtlIoConfig(const DeviceIndex &index, const Ut24TtlIoConfig &cfg)
{
    if(configMap.value(index) == cfg)
        return;
    configMap[index] = cfg;
    curDev = index;

    while(singleTtlWidgets.size() > cfg.ttlCfg.size())
        removeTtl();

    for(SingleUt24TtlWidget *w : singleTtlWidgets)
        w->setRegConfig(cfg.ttlCfg.value(w->ch));

    while(singleTtlWidgets.size() < cfg.ttlCfg.size())
        addTtl();
}

void Ut24TtlIoWidget::onFsmStateChange(const FsmState &state)
{
    bool en = state==FsmState::Ready || state==FsmState::Error
        || Globals::getDebugMode();
    setEnabled(en);
}

void Ut24TtlIoWidget::setUt24TtlIoBuildParams(const DeviceIndex &index, const Ut24TtlIoBuildParams &bp)
{
    if(buildParamsMap.value(index) == bp)
        return;
    buildParamsMap[index] = bp;
    if(curDev != index)
        return;
    for(SingleUt24TtlWidget *w : singleTtlWidgets)
        w->onUt24TtlIoBuildParamsChange(bp);
}

void Ut24TtlIoWidget::ttlItemChanged(int index)
{
    for(SingleUt24TtlWidget *w : singleTtlWidgets) {
        if(w->cb == sender()) {
            Q_ASSERT(configMap.value(curDev).ttlCfg.size() > w->ch);
            auto &ttlCfg = configMap[curDev].ttlCfg[w->ch];
            auto cbData = w->cb->itemData(index);
            auto newVal = cbData.toUInt();
            if(ttlCfg.all != newVal) {
                ttlCfg.all = newVal;
                emit ut24TtlIoConfigChanged(curDev, configMap[curDev]);
            }
            return;
        }
    }
}

void Ut24TtlIoWidget::addTtl()
{
    auto id = singleTtlWidgets.size();
    const auto &regCfg = configMap[curDev].ttlCfg[id];
    auto w = new SingleUt24TtlWidget(id, buildParamsMap.value(curDev), this);
    if(trigInputNameMapFunc)
        w->setTrigInputNameMap(trigInputNameMapFunc);
    singleTtlWidgets.append(w);
    ui->gridLayout->addWidget(w->lab, 100-id/2, 2*(id%2), 1, 1);
    ui->gridLayout->addWidget(w->cb, 100-id/2, 1+2*(id%2), 1, 1);

    w->setRegConfig(regCfg);
    connect(w->cb, QOverload<int>::of(&QComboBox::currentIndexChanged),
            this, &Ut24TtlIoWidget::ttlItemChanged);
}

void Ut24TtlIoWidget::removeTtl()
{
    auto w = singleTtlWidgets.takeLast();
    ui->gridLayout->removeWidget(w->lab);
    ui->gridLayout->removeWidget(w->cb);
    delete w;
}
