#ifndef UT24TTLIOWIDGET_H
#define UT24TTLIOWIDGET_H

#include <QWidget>
#include <QMap>

#include "base/DeviceIndex.h"
#include "Ut24TtlIo/Ut24TtlIoBuildParams.h"
#include "Ut24TtlIo/Ut24TtlIoConfig.h"
#include "fsm/FsmState.h"

class SingleUt24TtlWidget;

namespace Ui {
class Ut24TtlIoWidget;
}

class Ut24TtlIoWidget : public QWidget
{
    Q_OBJECT

public:
    explicit Ut24TtlIoWidget(QWidget *parent = nullptr);
    ~Ut24TtlIoWidget();
    void setTrigInputNameMap(const std::function<QString (const int)> &f);

public slots:
    void setUt24TtlIoConfig(const DeviceIndex &index, const Ut24TtlIoConfig &cfg);
    void onFsmStateChange(const FsmState &state);
    void setUt24TtlIoBuildParams(const DeviceIndex &index, const Ut24TtlIoBuildParams &bp);

private slots:
    void ttlItemChanged(int index);

signals:
    void ut24TtlIoConfigChanged(const DeviceIndex &index, const Ut24TtlIoConfig &config) const;

private:
    void addTtl();
    void removeTtl();

    Ui::Ut24TtlIoWidget *ui;
    QVector<SingleUt24TtlWidget *> singleTtlWidgets;

    QMap<DeviceIndex, Ut24TtlIoConfig> configMap;
    QMap<DeviceIndex, Ut24TtlIoBuildParams> buildParamsMap;
    DeviceIndex curDev;
    std::function<QString(const int)> trigInputNameMapFunc = nullptr;
};


#endif // UT24TTLIOWIDGET_H
