/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef WAVERECWIDGET_H
#define WAVERECWIDGET_H

#include "WaveRec/AdcBuildParams.h"
#include "WaveRec/WaveRecConfig.h"
#include <QWidget>

namespace Ui {
class WaveRecWidget;
}

class WaveRecWidget : public QWidget
{
    Q_OBJECT

public:
    explicit WaveRecWidget(QWidget *parent = nullptr);
    ~WaveRecWidget();

public slots:
    void setWaveRecConfig(const WaveRecConfig &newConfig);
    // void setBuildParams(const AdcBuildParams &newParams);

signals:
    void waveRecConfigChanged(const WaveRecConfig &config);

private slots:
    void on_comboBoxZsMode_currentIndexChanged(int index);
    void on_spinBoxThreshold_valueChanged(int thr);
    void on_spinBoxLatency_valueChanged(int lat);
    void on_spinBoxMatchWin_valueChanged(int win);
    void on_spinBoxAdcWin_valueChanged(int val);
    void on_spinBoxAdcPreWin_valueChanged(int preWin);

private:
    void restoreGuiFromConfig();
    void updateMatchWinMaximum() const;
    void updateWidgetWarnings();

    Ui::WaveRecWidget *ui;
    AdcBuildParams buildParams;
    WaveRecConfig config;
};

#endif // WAVERECWIDGET_H
