#include "FeLinkStatusDialog.h"
#include "ui_FeLinkStatusDialog.h"

#include <QHeaderView>
#include <QPushButton>
#include <QTabWidget>
#include <QTreeView>
#include <QTimer>

#include "base/ClientIndex.h"

namespace {
static constexpr auto *DIALOG_TITLE = "FE-Link Status";
}

FeLinkStatusDialog::FeLinkStatusDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::FeLinkStatusDialog)
    , pendingTimer(new QTimer(this))
    , model(new FeLinkTreeModel(this))
{
    ui->setupUi(this);
    ui->treeView->setModel(model);
    connect(pendingTimer, &QTimer::timeout,
            this, &FeLinkStatusDialog::processPendingStatus);
    connect(model, &QAbstractItemModel::modelReset,
            this, &FeLinkStatusDialog::modelReseted);
    if(auto b = ui->buttonBox->button(QDialogButtonBox::Reset)) {
        b->setText("Clear stat");
        connect(b, &QPushButton::clicked, model, &FeLinkTreeModel::clearStats);
    }
    if(auto b = ui->buttonBox->button(QDialogButtonBox::RestoreDefaults)) {
        b->setText("Restore view");
        connect(b, &QPushButton::clicked, this, &FeLinkStatusDialog::modelReseted);
    }
    pendingTimer->setSingleShot(true);
    pendingTimer->setInterval(100);
}

FeLinkStatusDialog::~FeLinkStatusDialog()
{
    delete ui;
}

void FeLinkStatusDialog::setClientIndex(const ClientIndex &ci)
{
    setWindowTitle(QString("%1 (%2)").arg(DIALOG_TITLE).arg(ci.toString()));
}

void FeLinkStatusDialog::setDeviceList(const DeviceIndexEnabledMap &devices_)
{
    model->setDeviceList(devices_);

    const DeviceIndexSet &newDeviceSet = devices_.getEnabled();
    if(readerDeviceSet == newDeviceSet)
        return;

    const auto &toDel = readerDeviceSet-newDeviceSet;
    readerDeviceSet = newDeviceSet;

    for(auto i : toDel) {
        feLinkStatusMap.remove(i);
        pendingFeLinkStatusMap.remove(i);
    }
}

void FeLinkStatusDialog::setFeLinkStatus(const DeviceIndex &readerIndex, const QMap<int, FeLinkStatus> &st)
{
    if(!readerDeviceSet.contains(readerIndex))
        return;
    pendingFeLinkStatusMap[readerIndex] = st;
    if(!pendingTimer->isActive())
        processPendingStatus();
}

void FeLinkStatusDialog::onDeviceConnected(const DeviceIndex &devIndex)
{
    pendingFeLinkStatusMap.remove(devIndex);
    model->onDeviceConnected(devIndex);
}

void FeLinkStatusDialog::processPendingStatus()
{
    if(onFreeze)
        return;

    for(auto readIt=pendingFeLinkStatusMap.cbegin(); readIt!=pendingFeLinkStatusMap.cend(); ++readIt){
        feLinkStatusMap[readIt.key()] = *readIt;
    }

    for(auto it=feLinkStatusMap.cbegin(); it!=feLinkStatusMap.cend(); ++it) {
        model->feLinkStatusUpdated(it.key(), it.value());
    }

    pendingTimer->start();
}

void FeLinkStatusDialog::modelReseted()
{
    ui->treeView->expandAll();
    for(auto c=0; c<model->columnCount(); ++c)
        ui->treeView->resizeColumnToContents(c);
}

void FeLinkStatusDialog::on_checkBoxFreeze_toggled(bool checked)
{
    onFreeze = checked;
    if(onFreeze)
        pendingTimer->stop();
    else
        processPendingStatus();
}

