#ifndef FELINKSTATUSDIALOG_H
#define FELINKSTATUSDIALOG_H

#include <QDialog>

#include "base/DeviceIndexEnabled.h"

#include "device-modules/FeLinkStatus/FeLinkStatus.h"
#include "FeLinkTree/FeLinkTreeModel.h"
#include "mldiscover/LldpDiscovery.h"

class ClientIndex;

namespace Ui {
class FeLinkStatusDialog;
}

class QTreeView;
class QTimer;

class FeLinkStatusDialog : public QDialog
{
    Q_OBJECT

public:
    explicit FeLinkStatusDialog(QWidget *parent = nullptr);
    ~FeLinkStatusDialog();
    FeLinkTreeModel *getModel() const { return model; }

signals:
    void selectedDeviceChanged(const DeviceIndex &);
    void deviceListChanged(const DeviceIndexEnabledMap &);

public slots:
    void setClientIndex(const ClientIndex &ci);
    void setDeviceList(const DeviceIndexEnabledMap &devices_);
    void setFeLinkStatus(const DeviceIndex &readerIndex, const QMap<int, FeLinkStatus> &st);
    void onDeviceConnected(const DeviceIndex &devIndex);

private slots:
    void processPendingStatus();
    void modelReseted();

    void on_checkBoxFreeze_toggled(bool checked);

private:
    Ui::FeLinkStatusDialog *ui;
    QTimer * const pendingTimer;
    DeviceIndexSet readerDeviceSet;
    DeviceIndexSet masterDevices;
    QMap<DeviceIndex, QMap<int, FeLinkStatus> > feLinkStatusMap; // by readerDevice
    QMap<DeviceIndex, QMap<int, FeLinkStatus> > pendingFeLinkStatusMap; // by readerDevice
    FeLinkTreeModel * const model;
    LldpInfoCache lldpCache;
    bool onFreeze = false;
};

#endif // FELINKSTATUSDIALOG_H
