#include "PnpBaseMainWindow.h"


#include <QLabel>
#include <QMenu>
#include <QMenuBar>
#include <QSignalBlocker>
#include <QStatusBar>

#include "configuration-manager/ConfigurationManager.h"
#include "pnp-server/PNPMessage.h"
#include "metric-sender/MetricSender.h"
#include "widgets/LogDialog.h"

PnpBaseMainWindow::PnpBaseMainWindow(QWidget *parent)
    : BaseMainWindow(parent)
    , program_description(new ProgramDescription)
    , labelFsmState(new QLabel(this))
    , configuration_manager(new ConfigurationManager(this))
    , logDialog(new LogDialog(this))
{
    qRegisterMetaType<ProgramDescription>();

    connect(this, &PnpBaseMainWindow::actionLog, logDialog, &LogDialog::show);
}

PnpBaseMainWindow::~PnpBaseMainWindow()
{
    emit closeProgram(*program_description);
    delete program_description;
}

void PnpBaseMainWindow::configSelectorChanged(const ConfigIndexName &cin)
{
    BaseMainWindow::configSelectorChanged(cin);
    // Initialize MetricSender
    MetricSender::setBaseTag("index", cin.getIndex().toString());
}

void PnpBaseMainWindow::onFsmStateChange(const FsmState &state)
{
    curFsmState = state;
    bool en = state == FsmState::Error || state == FsmState::Ready;
    auto items = {menuItemAllowRemoteControl,
                  menuItemConfigurationManger};
    labelFsmState->setText("FSM: " + fsmStateToString(state));
    for (auto a : items)
        if (a)
            a->setEnabled(en);
}

void PnpBaseMainWindow::setAllowRemoteControl(bool allow)
{
    QSignalBlocker(this->menuItemAllowRemoteControl);
    menuItemAllowRemoteControl->setChecked(allow);
}

void PnpBaseMainWindow::delayedInit()
{
    BaseMainWindow::delayedInit();
    statusBar()->addWidget(labelFsmState);
}

void PnpBaseMainWindow::addMenuFileActions()
{
    createMenuIfAbsent(&menuFile);

    menuItemConfigurationManger = new QAction(tr("Configuration manager..."),
                        menuFile);
    connect(menuItemConfigurationManger, &QAction::triggered,
            this, &PnpBaseMainWindow::actionConfigurationManager);
    menuFile->insertAction(menuFile->actions().value(0, nullptr),
                           menuItemConfigurationManger);

    menuFile->addSeparator();

    BaseMainWindow::addMenuFileActions();
}

void PnpBaseMainWindow::addMenuOptionsActions()
{
    createMenuIfAbsent(&menuOptions);

    menuItemAllowRemoteControl = new QAction(tr("Allow &Remote Control"), menuOptions);
    menuItemAllowRemoteControl->setCheckable(true);
    connect(menuItemAllowRemoteControl, &QAction::toggled,
            this, &PnpBaseMainWindow::allowRemoteControlChanged);
    menuOptions->insertAction(menuOptions->actions().value(0, nullptr),
                              menuItemAllowRemoteControl);

    BaseMainWindow::addMenuOptionsActions();
}

void PnpBaseMainWindow::addMenuToolsActions()
{
    menuTools->addAction(tr("Log viewer..."), this, &PnpBaseMainWindow::actionLog);

    BaseMainWindow::addMenuToolsActions();
}
