/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef READOUTAPPMAINWINDOW_H
#define READOUTAPPMAINWINDOW_H

#include "PnpBaseMainWindow.h"
#include "BaseDeviceAppConfig.h"

struct AdcDeserCheckStatus;
class AdcMpdTrigDialog;
class AdcSerDesCalDialog;
class ClockControlDialog;
struct ClockDividerStatus;
class ConfigKey;
class CrosspointSwitchDilog;
class DeviceIndex;
class ModularDeviceStatus;
class DeviceStatusDialog;
class DeviceStatusTableViewFancy;
class DiscoverDialog;
class FeLinkStatusDialog;
class HptdcStatusDialog;
class MStreamStatisticDialog;
class NetworkPortConfigDialog;
struct RemoteControlServerStatus;
class SparseDialog;
class ZmqDialog;

class QLabel;

enum AFI_SDB_ID : uint32_t;

class ReadoutAppMainWindow: public PnpBaseMainWindow
{
    Q_OBJECT

public:
    ReadoutAppMainWindow(QWidget *parent = nullptr);
    virtual DeviceStatusTableViewFancy *getDeviceStatusTableView() = 0;
    QAction *getActionDiscoverDialog() const { return menuItemDiscoverDialog; }

signals:
    void actionAdcMpdTrigDialog() const;
    void actionAdcSerDesCalibration() const;
    void actionClockControlConfig() const;
    void actionCpsConfig() const; // Crosspoint switch
    void actionDeviceStatus() const;
    void actionDeviceStatusZmqConfig() const;
    void actionDiscoverDialog() const;
    void actionFelinkStatus() const;
    void actionHptdcStatus() const;
    void actionMstreamStatistics() const;
    void actionNetworkPortConfig() const;
    void actionSparseDialog() const;
    void requestAdcClkDivSync() const;


    void setStatus(const ModularDeviceStatus &r); // relay to dialogs
    void setMultiStatus(const QMap<DeviceIndex, ModularDeviceStatus> &r); // relay to dialogs
    void errOnTrigOnXOffChanged(bool) const;
    void errOnSerDesUnlockChanged(bool) const;
    void errOnFeLinkErrChanged(bool isErr, bool isWarn) const;
    void showRunLogicChanged(bool) const;
    void configChanged(const BaseDeviceAppConfig &) const;

private:
    QAction *addToolAction(const QString &name,
                           void (ReadoutAppMainWindow::*pt2ConstMember)() const,
                           const QVariantList sdbList = {});
    void updateAdcDesCheckLabel(const QVariantMap &map);

    QAction *menuItemDiscoverDialog = nullptr;
    QAction *menuItemErrorOnXOff = nullptr;
    QAction *menuItemErrorOnSerDesUnlock = nullptr;
    QAction *menuItemErrorOnFeLinkErr = nullptr;
    QAction *menuItemWarnOnFeLinkErr = nullptr;
    QAction *menuItemShowRunLogic = nullptr;
    QAction *menuItemPeriodicalDesCheck = nullptr;
    QLabel *adcDesCheckLabel = nullptr;
    QLabel *clockDividerLabel = nullptr;

private slots:
    void sendShowRunLogicState() const;
    void sendErrOnFeLink(bool) const;
    void setPeriodicalDeserializerCheck(bool);
    void confirmAdcClockDividerSync() const;

public slots:
    virtual void onFsmStateChange(const FsmState &state) override;
    void setErrOnTrigOnXOff(bool err);
    void setErrOnSerDesUnlock(bool err);
    void setErrOnFeLinkErr(bool isErr, bool isWarn);
    void setDeviceSdbList(const DeviceIndex &, const QVariantList &devSdbList) const;
    void adcDeserConfigChecked(const DeviceIndex &index, const AdcDeserCheckStatus &st);
    void clockDividerStatusUpdated(const DeviceIndex &index, const ClockDividerStatus &st);
    virtual void setConfig(const ConfigKey &, const BaseDeviceAppConfig &_config);
    virtual void onRemoteStatusUpdated(const RemoteControlServerStatus &status);

protected:
    virtual void addMenuFileActions() override;
    virtual void addMenuOptionsActions() override;
    virtual void addMenuToolsActions() override;
    virtual void addMenuToolsConfigureActions();
    virtual void saveAppGuiSettings(QSettings &) const override;
    virtual void loadAppGuiSettings(const QSettings &) override;
    virtual void updateControlState() {}

    constexpr static const char* k_sdb_list = "sdb_list";
    BaseDeviceAppConfig config;
    bool underRemoteCtrl = false;

public:
    AdcMpdTrigDialog * const adcMpdTrigDialog;
    AdcSerDesCalDialog * const adcSerDesCalDialog;
    ClockControlDialog * const clockControlDialog;
    CrosspointSwitchDilog * const crosspointSwitchDilog;
    DeviceStatusDialog * const deviceStatusDialog;
    DiscoverDialog * const discoverDialog;
    FeLinkStatusDialog * const feLinkStatusDialog;
    HptdcStatusDialog * const hptdcStatusDialog;
    MStreamStatisticDialog * const mStreamStatisticDialog;
    NetworkPortConfigDialog * const networkPortConfigDialog;
    SparseDialog * const sparseDialog;
    ZmqDialog * const zmqDialog;
};

#endif // READOUTAPPMAINWINDOW_H
