#ifndef MSTREAMMREGDEVGLOBAL_H
#define MSTREAMMREGDEVGLOBAL_H

#include <QObject>
#include <optional>

#include "base/DeviceIndex.h"

class BaseConfigDispatcher;
class ConfigKey;
class DeviceStateController;
class MStreamDump;
class DevStreamDecoder;
class ModularDeviceStatus;
class ModularMregDevice;
struct DecodedData;
struct MStreamStat;
struct ReceiverStat;

class MstreamMregDevGlobal : public QObject
{
    Q_OBJECT
public:
    explicit MstreamMregDevGlobal(ModularMregDevice * const _dev,
                                  ModularDeviceStatus &status,
                                  QObject *parent = nullptr);
    ~MstreamMregDevGlobal() override;

    void setDeleteFlag() const;
    DeviceIndex getDeviceIndex() const { return index; }
    void setConfigDispatcher(BaseConfigDispatcher *cd);

    template<class T>
    const T *getDevice() const {
        static_assert (std::is_base_of<ModularMregDevice,T>::value,
                      "T must be derived of ModularMregDevice" );
        return qobject_cast<const T*>(dev);
    }
    bool isDeviceConnected() const;
    void setCreateDataDecoder(bool v);

signals:
    void msdump_reset(DeviceIndex index, int seqNum,
                      QString fn=QString(), bool doStatClear = false);
    void preRunCheck() const;
    void msdumpReseted(const DeviceIndex &index, int seqNum) const;
    void configWritten(const DeviceIndex &index, const ConfigKey &cfgKey);

    // queued signals from device thread
    void deviceConnected(const DeviceIndex &);
    void deviceDisconnected(const DeviceIndex &);
    void deviceStateChanged(const DeviceIndex &, bool);

    // resend from MStreamDump
    void stat_updated(const MStreamStat &stat);
    void stat_from_receiver_updated(const ReceiverStat &stat);
    void mstreamConnectionStateChanged(const DeviceIndex &deviceIndex, bool);
    void deviceDataUpdated(const DecodedData &data);

public slots:
    virtual void resetMStream(int seqNum);
    void setDataDecodeEnabled(bool en) const;

protected slots:
    void onMstreamConnectionStateChanged(const DeviceIndex &deviceIndex, bool state);
    void onDeviceStateChange(const DeviceIndex &deviceIndex, bool state);

protected:
    virtual void postAddMStreamDump(MStreamDump *) {}

private:
    void addMStreamDump();
    void addDataDecoder();

    const DeviceIndex index;
    ModularMregDevice * const dev;
    ModularDeviceStatus &status;
    MStreamDump * mstream = nullptr;
    bool createDataDecoder = false;
    DevStreamDecoder * dataDecoder = nullptr;
    BaseConfigDispatcher *configDispatcher = nullptr;
    DeviceStateController * const deviceStateController;
    bool mstreamConnectionState = false;
};

#endif // MSTREAMMREGDEVGLOBAL_H
