/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MQTTPUBLISHER_H
#define MQTTPUBLISHER_H

#include <QObject>
#include <QTimer>
#include <QUrl>

#include "../AbstractPublisher.h"

namespace QMQTT { class Client; }

class MqttPublisher: public AbstractPublisher
{
    Q_OBJECT
public:
    MqttPublisher(const QString &serviceName, QObject *parent = nullptr);
    ~MqttPublisher();
    void pub(const QString &topic, const QByteArray &text) override;
    bool event(QEvent* ev) override;

private slots:
    void connected();
    void disconnected();
    void error(const int error);
    void deferredInit();

private:
    void createConnection();
    void deleteConnection();

    bool enabled = true;
    QMQTT::Client *mqtt = nullptr;
    QUrl url;
    QTimer * reconnectTimer = nullptr;
};

#endif // MQTTPUBLISHER_H
