/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "AbstractConfigDb.h"
#include <QTimer>

#include "base/AbstractConfigConverter.h"
#include "base/Globals.h"

AbstractConfigDb::AbstractConfigDb(QObject *parent):
    QObject(parent),
    cs(Globals::getInitialConfigSelector()),
    write_timer(new QTimer(this))
{
    QTimer::singleShot(0, this, &AbstractConfigDb::startUpInit);

    write_timer->setSingleShot(true);
    write_timer->setInterval(1000);
    connect(write_timer, &QTimer::timeout,
            this, &AbstractConfigDb::write_request);
}

AbstractConfigDb::~AbstractConfigDb()
{
    if(write_timer->isActive()) {
        write_timer->stop();
        BaseConfig::writeConfig(cs, obj);
    }
}

void AbstractConfigDb::startUpInit()
{
    connectToDb();
    update_all();
}

void AbstractConfigDb::configSelectorChanged(const ConfigIndexName &cin)
{
    if(!initialized)
        return;
    if(cin == cs.getConfigIndexName())
        return;

    cs = ConfigSelector(cs.getType(), cin);
    update_config_from_db();

    emit switch_to_config(cin);
}

void AbstractConfigDb::write_config(QJsonObject obj_)
{
    obj = obj_;
    write_request();
}

void AbstractConfigDb::write_request()
{
    if(write_timer == QObject::sender()) {
        if(write_request_flag){
            BaseConfig::writeConfig(cs, obj);
            write_request_flag = false;
        }
    }
    else{
        if(write_timer->isActive()){
            write_request_flag = true;
        }
        else{
            BaseConfig::writeConfig(cs, obj);
            write_request_flag = false;
            write_timer->start();
        }
    }
}

//void AbstractConfigDb::setConfig(const T &newConfig)
//{
//    if (newConfig == config)
//        return;
//    config = newConfig;
//    write_pod_config(newConfig);
//}

void AbstractConfigDb::update_all()
{
    emit switch_to_config(cs.getConfigIndexName());
    initialized = true;
    update_config_from_db();
}

void AbstractConfigDb::saveRunDocument(const RunDocument &rd)
{
    BaseConfig::writeRunDocument(rd);
}


