
#include "mongo_access.h"
#include "MongoDbLogger.h"

#include <mongocxx/uri.hpp>
#include <bsoncxx/stdx/make_unique.hpp>
#include <mongocxx/exception/exception.hpp>
#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(lc_mongodb)
Q_LOGGING_CATEGORY(lc_mongodb, "afi.mongodb")
}

mongo_access::mongo_access()
    : _instance(bsoncxx::stdx::make_unique<mongocxx::instance>(
                    bsoncxx::stdx::make_unique<MongoDbLogger>()
                    ))
{}

mongo_access::~mongo_access()
{
    destroyed = true;
}

mongo_access& mongo_access::instance()
{
    try{
        static mongo_access instance;
        Q_ASSERT(!instance.destroyed);
        return instance;
    } catch (const std::exception& xcp){
        qFatal("%s", xcp.what());
    }
}

bool mongo_access::configure(const std::string &uri)
{
    try{
        _pool = bsoncxx::stdx::make_unique<mongocxx::pool>(mongocxx::uri(uri));
    } catch (const std::exception& xcp){
        qCritical(lc_mongodb) << __func__ << xcp.what();
        return false;
    }
    return true;
}

mongocxx::pool::entry mongo_access::get_connection()
{
    if (!_pool) {
        Q_ASSERT(_pool);
        qFatal("pool not initialized");
    }
    try{
        return _pool->acquire();
    } catch (const std::exception& xcp){
        qFatal("%s", xcp.what());
    }
}

bsoncxx::stdx::optional<mongocxx::pool::entry> mongo_access::try_get_connection()
{
    if (!_pool) {
        Q_ASSERT(_pool);
        qFatal("pool not initialized");
    }
    try{
        return _pool->try_acquire();
    } catch (const std::exception& xcp){
        qCritical(lc_mongodb) << __func__ << xcp.what();
        return {};
    }
}
