/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RECEIVERSTAT_H
#define RECEIVERSTAT_H

#include "base/DeviceIndex.h"

struct ReceiverStat {
    ReceiverStat (DeviceIndex _index=DeviceIndex(), int n = 0):
        index(_index)
    {
        clear(n);
    }
    void clear(int n) {
        noFree=0;
        didntHelp=0;
        free=0;
        checkFail=0;
        noData=0;
        pckSizeCnt.clear();
        pckSizeCnt.resize(n+1);
    }
    ReceiverStat &operator-=(const ReceiverStat &r)
    {
        acquiredAtMs -= r.acquiredAtMs;
        noFree -= r.noFree;
        didntHelp -= r.didntHelp;
        free -= r.free;
        checkFail -= r.checkFail;
        noData -= r.noData;
        // other fields not subtractable
        return *this;
    }
    const ReceiverStat operator-(const ReceiverStat &rhs) const
    { return ReceiverStat(*this) -= rhs; }

    DeviceIndex index;
    quint64 acquiredAtMs = 0;
    quint32 noFree = 0; // loop started with no free tasks
    quint32 didntHelp = 0; // processEvents didn't help to free tasks
    quint32 free = 0; // loops with free tasks
    quint32 checkFail = 0; // check_header failed
    quint32 noData = 0; // loops no data
    QVector<quint32> pckSizeCnt;
};
Q_DECLARE_METATYPE(ReceiverStat)


#endif // RECEIVERSTAT_H
