#ifndef XMLDISCOVER_H
#define XMLDISCOVER_H

#include "PNPMessage.h"

#include <QObject>

#include "base/DaqModule.h"

class MultiCastListener;
class QTimer;
class QUdpSocket;

class PNPServer : public QObject
{
    Q_OBJECT
public:
    explicit PNPServer(QObject *parent = nullptr);
    virtual ~PNPServer();
    static QVector<QHostAddress> getLocalHostAddresses();

public slots:
    void enableSearch();
    void startSearch(DaqModuleSet targets = {}, bool single = true);
    void stopSearch();
    void changeProgramDescription(const ProgramDescription &pd);
    void updateProgramDescription(const ProgramDescription &pd);
    void closeProgram(const ProgramDescription &pd);
    void addProgToDebugPrint(const QUuid &id);

signals:
    void programDiscovered(const ProgramDescription &, int packetSize) const;
    void programClosed(const ProgramDescription &) const;

protected slots:
    virtual bool decode(const QByteArray &data, const QHostAddress &host, int ttl=-1);

private slots:
    void sendSearchRequest();
    void deferredSearchResponce();

private:
    void sendProgDescr(ProgramDescription pd);
    void sendMulticastMsg(const QString &msg);
    void sendMulticastMsg(const QByteArray &data);
    void print(QString s) const;
    static quint16 getPnpPort();

    QList<ProgramDescription> progDesc;
    bool searchEnabled = false;
    DaqModuleSet searchTargets; // ProgramDescription::type
    QTimer * const searchTimer;
    MultiCastListener * const multiCastListener;
    QUdpSocket * const multiCastSender;
    QMap<QUuid, quint32> seqMap;
    QSet<QUuid> searchRequestSet;
    QTimer * const deferredSearchResponceTimer;
    QElapsedTimer lastSearchTime;
    bool sendXml = true;
    bool sendRaw = false;
    QSet<QUuid> progsToDebugPrint;
};

#endif // XMLDISCOVER_H
