/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef REMOTECONTROLTLVSERVER_H
#define REMOTECONTROLTLVSERVER_H

#include <QObject>
#include <optional>

class QTcpServer;
class QTcpSocket;
struct RemoteControlServerStatus;
class TypeLengthHeader;

class RemoteControlTlvServer : public QObject
{
    Q_OBJECT
public:
    explicit RemoteControlTlvServer(quint16 preferedTcpPort, QObject *parent = nullptr);
    ~RemoteControlTlvServer() override;
    int getTcpPort() const;
    bool isFree() const;
    bool isListening() const;

    void send_byte_array(quint32 type, const QByteArray &mess) const;
    void send_json(const QJsonObject &obj) const;

signals:
    void connectionStatusUpdated(const RemoteControlServerStatus &) const;
    void tlvDataReceived(const TypeLengthHeader &hdr, const QByteArray &data);
    void connectionAccepted();

public slots:
    void startListen(bool state);

private slots:
    void updateConnections();
    void onDisconnected();
    void readData();

private:
    void listen();
    void closeServer();
    void closeSocket();
    void updateStatus() const;
    bool isConnectionAlive() const;
    bool isDataSizeEnoughForHdr() const;
    TypeLengthHeader *findHeader();

    QTcpSocket *tcpSocket = nullptr;
    QTcpServer * const qTcpServer;
    RemoteControlServerStatus * const status;
    QByteArray data;
    std::optional<int> prevDataTrashed;
};

#endif // REMOTECONTROLTLVSERVER_H
