#include "ZmqDialog.h"
#include "ui_ZmqDialog.h"

#include "ZmqPublisher.h"

ZmqDialog::ZmqDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ZmqDialog)
{
    ui->setupUi(this);
}

ZmqDialog::~ZmqDialog()
{
    delete ui;
}

void ZmqDialog::setZmqConfig(const ZmqConfig &config)
{
    cfg = config;
    block_change_signal = true;
    ui->checkBoxEnable->setChecked(cfg.enabled);
    ui->checkBoxTest->setChecked(cfg.test_mode);
    ui->checkBoxSendAll->setChecked(cfg.send_all);
    (cfg.auto_port ? ui->radioButtonPortAuto : ui->radioButtonPortManual)
            ->setChecked(true);
    ui->spinBoxPort->setValue(cfg.port);
    ui->spinBoxPort->setEnabled(!cfg.auto_port);
    ui->checkBoxPrint->setChecked(cfg.print_messages);
    block_change_signal = false;
    if(ui->spinBoxPort->value() != cfg.port) {
        cfg.port = ui->spinBoxPort->value();
        emit zmqConfigChanged(cfg);
    }
}

void ZmqDialog::on_checkBoxEnable_toggled(bool en)
{
    if(block_change_signal) return;
    cfg.enabled = en;
    emit zmqConfigChanged(cfg);
}

void ZmqDialog::on_checkBoxTest_toggled(bool en)
{
    if(block_change_signal) return;
    cfg.test_mode = en;
    emit zmqConfigChanged(cfg);
}

void ZmqDialog::on_checkBoxSendAll_stateChanged(int st)
{
    if(block_change_signal) return;
    cfg.send_all = st == Qt::Checked;
    emit zmqConfigChanged(cfg);
}

void ZmqDialog::on_radioButtonPortAuto_toggled(bool checked)
{
    if(block_change_signal) return;
    cfg.auto_port = checked;
    ui->spinBoxPort->setEnabled(!cfg.auto_port);
    emit zmqConfigChanged(cfg);
}

void ZmqDialog::on_spinBoxPort_valueChanged(int p)
{
    if(block_change_signal) return;
    cfg.port = p;
    emit zmqConfigChanged(cfg);
}

void ZmqDialog::on_checkBoxPrint_toggled(bool print)
{
    if(block_change_signal) return;
    cfg.print_messages = print;
    emit zmqConfigChanged(cfg);
}

void ZmqDialog::statusUpdated(const ZmqStatus &s)
{
    QString str = s.working ? "Connected" : "Disconnected";
    QPalette palette = ui->labelConnected->palette();
    palette.setColor(ui->labelConnected->foregroundRole(), s.working ? Qt::darkGreen : Qt::red);
    ui->labelConnected->setPalette(palette);
    ui->labelConnected->setText(str);
    ui->labelPortAuto->setText((s.working && s.port) ? QString::number(s.port) : "-");

    str.clear();
    if(s.working)
        str = QString("packages: %1").arg(s.packages);
    ui->labelStatus->setText(str);
}
